@@ -37,6 +37,8 @@
 
 /* delimiter for lists of exit aliases */
 #define EXIT_DELIMITER ';'
+/* delimiter for lists of player name aliases */
+#define ALIAS_DELIMITER ';'
 
 #define QUIT_COMMAND "QUIT"
 #define WHO_COMMAND "WHO"
@@ -164,6 +166,7 @@ struct options_table {
   char room_flags[BUFFER_LEN];		/**< Space-separated list of flags to set on newly created rooms. */
   char exit_flags[BUFFER_LEN];		/**< Space-separated list of flags to set on newly created exits. */
   char thing_flags[BUFFER_LEN];		/**< Space-separated list of flags to set on newly created things. */
+  char channel_flags[BUFFER_LEN];	/**< Space-separated list of flags to set on newly created channels. */
   int warn_interval;	/**< Interval between warning checks */
   time_t warn_counter;	/**< Time since last warning check */
   dbref base_room;	/**< Room which floating checks consider as the base */
@@ -200,6 +203,7 @@ struct options_table {
   char ip_addr[64];	/**< What ip address should the server bind to? */
   char ssl_ip_addr[64];	/**< What ip address should the server bind to? */
   int player_name_spaces;	/**< Can players have multiword names? */
+  int max_aliases;		/**< Maximum allowed aliases per player */
   int forking_dump;	/**< Should we fork to dump? */
   int restrict_building;	/**< Is the builder power required to build? */
   int free_objects;	/**< If builder power is required, can you create without it? */
@@ -378,6 +382,7 @@ int cf_time(const char *opt, const char *val, void *loc, int maxval,
 #define RESTRICTED_BUILDING (options.restrict_building)
 #define NO_FORK (!options.forking_dump)
 #define PLAYER_NAME_SPACES (options.player_name_spaces)
+#define MAX_ALIASES (options.max_aliases)
 #define SAFER_UFUN (options.safer_ufun)
 #define NOISY_WHISPER (options.noisy_whisper)
 #define POSSESSIVE_GET (options.possessive_get)
@@ -409,6 +414,7 @@ int cf_time(const char *opt, const char *val, void *loc, int maxval,
 #define MUSH_IP_ADDR (options.ip_addr)
 #define SSL_IP_ADDR (options.ssl_ip_addr)
 #define MAX_ATTRCOUNT (options.max_attrcount)
+#define HARD_MAX_ATTRCOUNT 1000000
 #define FLOAT_PRECISION (options.float_precision)
 #define RECURSION_LIMIT (options.func_nest_lim)
 #define FUNCTION_LIMIT (options.func_invk_lim)
