@@ -121,8 +121,10 @@ extern void sql_shutdown(void);
 #define NA_NOSPOOF        0x1000	/**< Message comes via a NO_SPOOF object. */
 #define NA_PARANOID       0x2000	/**< Message comes via a PARANOID object. */
 #define NA_NOPREFIX       0x4000	/**< Don't use @prefix when forwarding */
-#define NA_INTERACTION  (NA_INTER_HEAR|NA_INTER_SEE|NA_INTER_PRESENCE)	/**< Message follows interaction rules */
 #define NA_SPOOF        0x8000	/**< @ns* message, overrides NOSPOOF */
+#define NA_INTER_LOCK  0x10000	/**< Message subject to @lock/interact even if not otherwise marked */
+#define NA_INTERACTION  (NA_INTER_HEAR|NA_INTER_SEE|NA_INTER_PRESENCE|NA_INTER_LOCK)	/**< Message follows interaction rules */
+#define NA_PROMPT       0x20000	 /**< Message is a prompt, add GOAHEAD */
 
 /** A notify_anything lookup function type definition */
 typedef dbref (*na_lookup) (dbref, void *);
@@ -157,8 +159,12 @@ extern char *ns_esnotify(dbref speaker, na_lookup func, void *fdata, int para);
 
 /** Basic 'notify player with message */
 #define notify(p,m)           notify_anything(orator, na_one, &(p), NULL, 0, m)
+/** Notify player as a prompt */
+#define notify_prompt(p,m)           notify_anything(orator, na_one, &(p), NULL, NA_PROMPT, m)
 /** Notify puppet with message, even if owner's there */
 #define notify_must_puppet(p,m)           notify_anything(orator, na_one, &(p), NULL, NA_MUST_PUPPET, m)
+/** Notify puppet with message as prompt, even if owner's there */
+#define notify_prompt_must_puppet(p,m)           notify_anything(orator, na_one, &(p), NULL, NA_MUST_PUPPET|NA_PROMPT, m)
 /** Notify player with message, as if from somethign specific */
 #define notify_by(t,p,m)           notify_anything(t, na_one, &(p), NULL, 0, m)
 /** Notfy player with message, but only puppet propagation */
@@ -191,6 +197,7 @@ extern char ucbuff[];
 #endif
 
 /* From cque.c */
+struct real_pcre;
 struct eval_context {
   char *wenv[10];		  /**< working environment (%0-%9) */
   char renv[NUMQ][BUFFER_LEN];	  /**< working registers q0-q9,qa-qz */
@@ -199,8 +206,13 @@ struct eval_context {
   int process_command_port;   /**< port number that a command came from */
   dbref cplr;			  /**< initiating player */
   char ccom[BUFFER_LEN];	  /**< raw command */
+  char ucom[BUFFER_LEN];      /**< evaluated command */
   int break_called;	      /**< Has the break command been called? */
   char break_replace[BUFFER_LEN];  /**< What to replace the break with */
+  struct real_pcre *re_code;		  /**< The compiled re */
+  int re_subpatterns;	      /**< The number of re subpatterns */
+  int *re_offsets;	      /**< The offsets for the subpatterns */
+  char *re_from;	      /**< The positions of the subpatterns */
 };
 
 typedef struct eval_context EVAL_CONTEXT;
@@ -212,6 +224,10 @@ extern void do_halt(dbref owner, const char *ncom, dbref victim);
 extern void parse_que(dbref player, const char *command, dbref cause);
 extern int queue_attribute_base
   (dbref executor, const char *atrname, dbref enactor, int noparent);
+extern ATTR *queue_attribute_getatr(dbref executor, const char *atrname,
+				    int noparent);
+extern int queue_attribute_useatr(dbref executor, ATTR *a, dbref enactor);
+
 /** Queue the code in an attribute, including parent objects */
 #define queue_attribute(a,b,c) queue_attribute_base(a,b,c,0)
 /** Queue the code in an attribute, excluding parent objects */
@@ -252,6 +268,7 @@ enum look_type { LOOK_NORMAL, LOOK_TRANS, LOOK_AUTO, LOOK_CLOUDYTRANS,
 extern void look_room(dbref player, dbref loc, enum look_type style);
 extern void do_look_around(dbref player);
 extern void do_look_at(dbref player, const char *name, int key);
+extern char *decompose_str(char *what);
 
 /* From memcheck.c */
 extern void add_check(const char *ref);
@@ -283,6 +300,7 @@ extern char *mush_crypt(const char *key);
 /* From player.c */
 extern int password_check(dbref player, const char *password);
 extern dbref lookup_player(const char *name);
+extern dbref lookup_player_name(const char *name);
 /* from player.c */
 extern dbref create_player(const char *name, const char *password,
 			   const char *host, const char *ip);
@@ -298,11 +316,13 @@ extern int is_strict_integer(const char *str);
 
 /* From plyrlist.c */
 void clear_players(void);
-void add_player(dbref player, const char *alias);
+void add_player(dbref player);
+void add_player_alias(dbref player, const char *alias);
 void delete_player(dbref player, const char *alias);
+void reset_player_list(dbref player, const char *oldname, const char *oldalias,
+		       const char *name, const char *alias);
 
 /* From predicat.c */
-
 extern char *WIN32_CDECL tprintf(const char *fmt, ...)
   __attribute__ ((__format__(__printf__, 1, 2)));
 
@@ -327,19 +347,22 @@ extern int controls(dbref who, dbref what);
 extern int can_pay_fees(dbref who, int pennies);
 extern void giveto(dbref who, int pennies);
 extern int payfor(dbref who, int cost);
+extern int quiet_payfor(dbref who, int cost);
 extern int nearby(dbref obj1, dbref obj2);
 extern int get_current_quota(dbref who);
 extern void change_quota(dbref who, int payment);
 extern int ok_name(const char *name);
 extern int ok_command_name(const char *name);
-extern int ok_player_name(const char *name, dbref player);
+extern int ok_function_name(const char *name);
+extern int ok_player_name(const char *name, dbref player, dbref thing);
+extern int ok_player_alias(const char *alias, dbref player, dbref thing);
 extern int ok_password(const char *password);
 extern int ok_tag_attribute(dbref player, const char *params);
 extern dbref parse_match_possessor(dbref player, const char **str);
 extern void page_return(dbref player, dbref target, const char *type,
 			const char *message, const char *def);
 extern char *grep_util(dbref player, dbref thing, char *pattern,
-		       char *lookfor, int len, int insensitive);
+		       char *lookfor, int sensitive, int wild);
 extern dbref where_is(dbref thing);
 extern int charge_action(dbref player, dbref thing, const char *awhat);
 dbref first_visible(dbref player, dbref thing);
@@ -402,6 +425,7 @@ strdup(const char *s)
 
 
     extern ansi_string *parse_ansi_string(const char *src) __attribute_malloc__;
+    extern void flip_ansi_string(ansi_string *as);
     extern void free_ansi_string(ansi_string *as);
     extern void populate_codes(ansi_string *as);
     extern void depopulate_codes(ansi_string *as);
@@ -494,7 +518,7 @@ strdup(const char *s)
 #define object_header(p,l) unparse_object(p,l)
     extern const char *unparse_object_myopic(dbref player, dbref loc);
     extern const char *unparse_room(dbref player, dbref loc);
-    extern int nameformat(dbref player, dbref loc, char *tbuf1);
+    extern int nameformat(dbref player, dbref loc, char *tbuf1, char *defname);
     extern const char *accented_name(dbref thing);
 
 /* From utils.c */
@@ -503,6 +527,17 @@ strdup(const char *s)
     extern void parse_anon_attrib(dbref player, char *str, dbref *thing,
 				  ATTR **attrib);
     extern void free_anon_attrib(ATTR *attrib);
+    typedef struct _ufun_attrib {
+      dbref thing;
+      char contents[BUFFER_LEN];
+      int pe_flags;
+      char *errmess;
+    } ufun_attrib;
+    extern int fetch_ufun_attrib(char *attrname, dbref executor,
+				 ufun_attrib * ufun, int accept_lambda);
+    extern int call_ufun(ufun_attrib * ufun, char **wenv_args, int wenv_argc,
+			 char *ret, dbref executor, dbref enactor,
+			 PE_Info * pe_info);
     extern int member(dbref thing, dbref list);
     extern int recursive_member(dbref disallow, dbref from, int count);
     extern dbref remove_first(dbref first, dbref what);
@@ -511,6 +546,8 @@ strdup(const char *s)
 				const char *check) __attribute_malloc__;
     extern void mush_free(Malloc_t RESTRICT ptr, const char *RESTRICT check);
     extern long get_random_long(long low, long high);
+    extern char *fullalias(dbref it);
+    extern char *shortalias(dbref it);
     extern char *shortname(dbref it);
     extern dbref absolute_room(dbref it);
     int can_interact(dbref from, dbref to, int type);
@@ -569,11 +606,18 @@ strdup(const char *s)
     extern char *strip_braces(char const *line);
     extern void save_global_regs(const char *funcname, char *preserve[]);
     extern void restore_global_regs(const char *funcname, char *preserve[]);
+    extern void free_global_regs(const char *funcname, char *preserve[]);
+    extern void init_global_regs(char *preserve[]);
     extern void load_global_regs(char *preserve[]);
     extern void save_global_env(const char *funcname, char *preserve[]);
     extern void restore_global_env(const char *funcname, char *preserve[]);
-    extern int delim_check
-      (char *buff, char **bp, int nfargs, char **fargs, int sep_arg, char *sep);
+    extern void save_global_nxt(const char *funcname, char *preservew[],
+				char *preserver[], char *valw[], char *valr[]);
+    extern void restore_global_nxt(const char *funcname, char *preservew[],
+				   char *preserver[], char *valw[],
+				   char *valr[]);
+    extern int delim_check(char *buff, char **bp, int nfargs, char **fargs,
+			   int sep_arg, char *sep);
     extern int get_gender(dbref player);
     extern int gencomp(dbref player, char *a, char *b, char *sort_type);
     extern const char *do_get_attrib(dbref executor, dbref thing,
@@ -616,7 +660,8 @@ strdup(const char *s)
     void local_flags(FLAGSPACE * flags);
 
 /* funlist.c */
-    void do_gensort(dbref player, char *s[], int n, char *sort_type);
+    void do_gensort(dbref player, char *keys[], char *strs[], int n,
+		    char *sort_type);
 
 /* sig.c */
     /** Type definition for signal handlers */
