@@ -32,8 +32,8 @@
 #define FN_LOGNAME 0x1000
 /* Log function name and args */
 #define FN_LOGARGS 0x2000
-
-
+/* Localize function registers */
+#define FN_LOCALIZE     0x4000
 
 #ifndef HAVE_FUN_DEFINED
 typedef struct fun FUN;
@@ -98,7 +98,7 @@ extern void do_function_restrict(dbref player, const char *name,
 extern void do_function_restore(dbref player, const char *name);
 extern void do_list_functions(dbref player, int lc);
 extern char *list_functions(void);
-extern void do_function(dbref player, char *name, char **argv);
+extern void do_function(dbref player, char *name, char **argv, int preserve);
 extern void do_function_toggle(dbref player, char *name, int toggle);
 extern void do_function_report(dbref player, char *name);
 extern void do_function_delete(dbref player, char *name);
