@@ -27,7 +27,7 @@ extern void fcache_load(dbref player);
 extern void hide_player(dbref player, int hide);
 enum motd_type { MOTD_MOTD, MOTD_WIZ, MOTD_DOWN, MOTD_FULL, MOTD_LIST };
 extern void do_motd(dbref player, enum motd_type key, const char *message);
-extern void do_poll(dbref player, const char *message);
+extern void do_poll(dbref player, const char *message, int clear);
 /* From cque.c */
 extern void do_wait
   (dbref player, dbref cause, char *arg1, char *cmd, int until);
@@ -43,6 +43,7 @@ extern void do_restart_com(dbref player, const char *arg1);
 enum hook_type { HOOK_BEFORE, HOOK_AFTER, HOOK_IGNORE, HOOK_OVERRIDE };
 extern void do_hook(dbref player, char *command, char *obj, char *attrname,
 		    enum hook_type flag);
+extern void do_hook_list(dbref player, char *command);
 
 
 /* From compress.c */
@@ -111,7 +112,7 @@ extern void do_cpattr
   (dbref player, char *oldpair, char **newpair, int move, int noflagcopy);
 enum edit_type { EDIT_FIRST, EDIT_ALL };
 extern void do_gedit(dbref player, char *it, char **argv,
-		     enum edit_type target);
+		     enum edit_type target, int doit);
 extern void do_trigger(dbref player, char *object, char **argv);
 extern void do_use(dbref player, const char *what);
 extern void do_parent(dbref player, char *name, char *parent_name);
@@ -128,12 +129,14 @@ enum wall_type { WALL_ALL, WALL_RW, WALL_WIZ };
 extern void do_wall(dbref player, const char *message, enum wall_type target,
 		    int emit);
 extern void do_page(dbref player, const char *arg1, const char *arg2,
-		    dbref cause, int noeval, int multipage, int override);
+		    dbref cause, int noeval, int multipage, int override,
+		    int has_eq);
 extern void do_page_port(dbref player, const char *arg1, const char *arg2);
 extern void do_think(dbref player, const char *message);
 #define PEMIT_SILENT 0x1
 #define PEMIT_LIST   0x2
 #define PEMIT_SPOOF  0x4
+#define PEMIT_PROMPT 0x8
 extern void do_emit(dbref player, const char *tbuf1, int flags);
 extern void do_pemit
   (dbref player, const char *arg1, const char *arg2, int flags);
