@@ -60,6 +60,8 @@ int add_lock_raw(dbref player, dbref thing, lock_type type,
 		 boolexp key, int flags);
 void free_locks(lock_list *ll);
 int eval_lock(dbref player, dbref thing, lock_type ltype);
+int eval_lock_with(dbref player, dbref thing, lock_type ltype, dbref env0,
+		   dbref env1);
 int fail_lock(dbref player, dbref thing, lock_type ltype, const char *def,
 	      dbref loc);
 void do_unlock(dbref player, const char *name, lock_type type);
@@ -68,6 +70,8 @@ void do_lock(dbref player, const char *name, const char *keyname,
 void init_locks(void);
 void clone_locks(dbref player, dbref orig, dbref clone);
 void do_lset(dbref player, char *what, char *flags);
+void do_list_locks(dbref player, const char *arg, int lc, const char *label);
+void list_locks(char *buff, char **bp, const char *name);
 const char *lock_flags(lock_list *ll);
 const char *lock_flags_long(lock_list *ll);
 void list_lock_flags(char *buff, char **bp);
@@ -108,6 +112,8 @@ extern const lock_type Control_Lock;	/* Who can control this object? */
 extern const lock_type Dropto_Lock;	/* Who follows the dropto of this room? */
 extern const lock_type Destroy_Lock;	/* Who can @dest me if I'm dest_ok? */
 extern const lock_type Interact_Lock;
+extern const lock_type MailForward_Lock;	/* Who can forward mail to me */
+extern const lock_type Take_Lock;	/* Who can take from the contents of this object? */
 /* Declare new lock types here! */
 
 #endif				/* __LOCK_H */
