@@ -54,6 +54,7 @@ struct pe_info {
   int nest_depth;		/**< Depth of function nesting, for DEBUG */
   int call_depth;		/**< Function call counter */
   Debug_Info *debug_strings;	/**< DEBUG infromation */
+  int arg_count;		/**< Number of arguments passed to function */
 };
 
 
@@ -77,6 +78,23 @@ struct text_queue {
 };
 
 /* Descriptor foo */
+/** Using Pueblo, Smial, Mushclient, Simplemu, or some other
+ *  *  pueblo-style HTML aware client */
+#define CONN_HTML 0x1
+/** Using a client that understands telnet options */
+#define CONN_TELNET 0x2
+/** Send a telnet option to test client */
+#define CONN_TELNET_QUERY 0x4
+/** Connection that should be close on load from reboot.db */
+#define CONN_CLOSE_READY 0x8
+/** Validated connection from an SSL concentrator */
+#define CONN_SSL_CONCENTRATOR 0x10
+/** Player would like to receive newlines after prompts, because
+ *  * their client mucks up output after a GOAHEAD */
+#define CONN_PROMPT_NEWLINES 0x20
+/** Default connection, nothing special */
+#define CONN_DEFAULT (CONN_PROMPT_NEWLINES)
+
 #define DOING_LEN 40
 /** Pueblo checksum length.
  * Pueblo uses md5 now, but if they switch to sha1, this will still
@@ -107,16 +125,6 @@ struct descriptor_data {
   int cmds;		/**< Number of commands sent */
   int hide;		/**< Hide status */
   char doing[DOING_LEN];	/**< Player's doing string */
-#ifdef NT_TCP
-  /* these are for the Windows NT TCP/IO */
-  char input_buffer[512];	/**< WinNT: buffer for reading */
-  char output_buffer[512];	/**< WinNT: buffer for writing */
-  OVERLAPPED InboundOverlapped;	/**< WinNT: for asynchronous reading */
-  OVERLAPPED OutboundOverlapped;	/**< WinNT: for asynchronous writing */
-  BOOL bWritePending;		/**< WinNT: true if in process of writing */
-  BOOL bConnectionDropped;	/**< WinNT: true if we cannot send to player */
-  BOOL bConnectionShutdown;	/**< WinNT: true if connection has been shutdown */
-#endif
   struct descriptor_data *next;	/**< Next descriptor in linked list */
   struct descriptor_data *prev;	/**< Previous descriptor in linked list */
   struct mail *mailp;	/**< Pointer to start of player's mail chain */
