@@ -7,7 +7,7 @@
 
 #ifdef HAS_OPENSSL
 
-SSL_CTX *ssl_init(void);
+SSL_CTX *ssl_init(char *private_key_file, char *ca_file, int req_client_cert);
 SSL *ssl_setup_socket(int sock);
 void ssl_close_connection(SSL * ssl);
 SSL *ssl_listen(int sock, int *state);
