@@ -31,12 +31,22 @@
 #pragma warning( disable : 4761)	/* disable warning re conversion */
 #endif
 
+/** Attribute error - too many attribs */
+#define AE_TOOMANY -4
+/** Attribute error - invalid name */
+#define AE_BADNAME -3
+/** Attribute error - attempt to overwrite a safe attribute */
+#define AE_SAFE -2
+/** Attribute error - general failure */
+#define AE_ERROR -1
+
 /** A string tree of attribute names in use, to save us memory since
  * many are duplicated.
  */
 StrTree atr_names;
 /** Table of attribute flags. */
-extern PRIV attr_privs[];
+extern PRIV attr_privs_set[];
+extern PRIV attr_privs_view[];
 
 /** A flag to show if we're in the middle of a @wipe (this changes
  * behaviour for atr_clr()).  Yes, this is gross and ugly, but it
@@ -204,16 +214,43 @@ int
 string_to_atrflag(dbref player, char const *p)
 {
   int f;
-  f = string_to_privs(attr_privs, p, 0);
+  f = string_to_privs(attr_privs_set, p, 0);
   if (!f)
     return -1;
   if (!Hasprivs(player) && (f & AF_MDARK))
     return -1;
   if (!See_All(player) && (f & AF_WIZARD))
     return -1;
+  f &= ~AF_INTERNAL;
   return f;
 }
 
+/** Convert a string of attribute flags to a pair of bitmasks.
+ * Given a space-separated string of attribute flags, look them up
+ * and return bitmasks of those to set or clear 
+ * if player is permitted to use all of those flags.
+ * \param player the dbref to use for privilege checks.
+ * \param p a space-separated string of attribute flags.
+ * \param setbits pointer to address of bitmask to set.
+ * \param clrbits pointer to address of bitmask to clear.
+ * \return setbits or -1 on error.
+ */
+int
+string_to_atrflagsets(dbref player, char const *p, int *setbits, int *clrbits)
+{
+  int f;
+  *setbits = *clrbits = 0;
+  f = string_to_privsets(attr_privs_set, p, setbits, clrbits);
+  if (f <= 0)
+    return -1;
+  if (!Hasprivs(player) && ((*setbits & AF_MDARK) || (*clrbits & AF_MDARK)))
+    return -1;
+  if (!See_All(player) && ((*setbits & AF_WIZARD) || (*clrbits & AF_WIZARD)))
+    return -1;
+  f &= ~AF_INTERNAL;
+  return *setbits;
+}
+
 /** Convert an attribute flag bitmask into a list of the full
  * names of the flags.
  * \param mask the bitmask of attribute flags to display.
@@ -222,7 +259,7 @@ string_to_atrflag(dbref player, char const *p)
 const char *
 atrflag_to_string(int mask)
 {
-  return privs_to_string(attr_privs, mask);
+  return privs_to_string(attr_privs_view, mask);
 }
 
 /*======================================================================*/
@@ -407,7 +444,7 @@ can_create_attr(dbref player, dbref obj, char const *atr_name, int flags)
   AL_NAME(atr) = atr_name;
   set_default_flags(atr, flags);
   if (Cannot_Write_This_Attr(player, atr, 1))
-    return 0;
+    return AE_ERROR;
 
   strcpy(missing_name, atr_name);
   atr = List(obj);
@@ -426,16 +463,17 @@ can_create_attr(dbref player, dbref obj, char const *atr_name, int flags)
     }
     if (Cannot_Write_This_Attr(player, atr, 1)) {
       missing_name[0] = '\0';
-      return 0;
+      return AE_ERROR;
     }
     *p = '`';
   }
 
-  if (AttrCount(obj) + num_new > MAX_ATTRCOUNT) {
+  if ((AttrCount(obj) + num_new) >
+      (Many_Attribs(obj) ? HARD_MAX_ATTRCOUNT : MAX_ATTRCOUNT)) {
     do_log(LT_ERR, player, obj,
 	   T("Attempt by %s(%d) to create too many attributes on %s(%d)"),
 	   Name(player), player, Name(obj), obj);
-    return 0;
+    return AE_TOOMANY;
   }
 
   return 1;
@@ -528,13 +566,6 @@ atr_new_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
   }
 }
 
-/** Attribute error - invalid name */
-#define AE_BADNAME -3
-/** Attribute error - attempt to overwrite a safe attribute */
-#define AE_SAFE -2
-/** Attribute error - general failure */
-#define AE_ERROR -1
-
 /** Add an attribute to an object, safely.
  * \verbatim
  * This is the function that should be called in hardcode to add
@@ -575,8 +606,9 @@ atr_add(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
 
   /* make a new atr, if needed */
   if (!ptr) {
-    if (!can_create_attr(player, thing, atr, flags))
-      return AE_ERROR;
+    int res = can_create_attr(player, thing, atr, flags);
+    if (res < 0)
+      return res;
 
     strcpy(missing_name, atr);
     ptr = List(thing);
@@ -1051,11 +1083,13 @@ void
 atr_cpy(dbref dest, dbref source)
 {
   ATTR *ptr;
+  int max_attrs;
 
+  max_attrs = (Many_Attribs(dest) ? HARD_MAX_ATTRCOUNT : MAX_ATTRCOUNT);
   List(dest) = NULL;
   for (ptr = List(source); ptr; ptr = AL_NEXT(ptr))
     if (!AF_Nocopy(ptr)
-	&& (AttrCount(dest) < MAX_ATTRCOUNT)) {
+	&& (AttrCount(dest) < max_attrs)) {
       atr_new_add(dest, AL_NAME(ptr), atr_value(ptr),
 		  AL_CREATOR(ptr), AL_FLAGS(ptr), AL_DEREFS(ptr));
       AttrCount(dest)++;
@@ -1219,6 +1253,11 @@ atr_comm_match(dbref thing, dbref player, int type, int end, char const *str,
     if (!s)
       continue;
     *s++ = '\0';
+    if (type == '^' && !AF_Ahear(ptr)) {
+      if ((thing == player && !AF_Mhear(ptr))
+	  || (thing != player && AF_Mhear(ptr)))
+	continue;
+    }
 
     if (AF_Regexp(ptr)) {
       /* Turn \: into : */
@@ -1504,23 +1543,51 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
   upcasestr(name);
   if (!strcmp(name, "ALIAS") && IsPlayer(thing)) {
     old = atr_get_noparent(thing, "ALIAS");
+    tbuf1[0] = '\0';
     if (old) {
       /* Old alias - we're allowed to change to a different case */
       strcpy(tbuf1, atr_value(old));
-      if (s && (!*s || (strcasecmp(s, tbuf1) && !ok_player_name(s, player)))) {
-	notify(player, T("That is not a valid alias."));
+      if (s && !*s) {
+	notify_format(player, T("'%s' is not a valid alias."), s);
 	return -1;
       }
-      if (Can_Write_Attr(player, thing, old))
-	delete_player(thing, tbuf1);
+      if (s && strcasecmp(s, tbuf1)) {
+	int opae_res = ok_player_alias(s, player, thing);
+	switch (opae_res) {
+	case OPAE_INVALID:
+	  notify_format(player, T("'%s' is not a valid alias."), s);
+	  break;
+	case OPAE_TOOMANY:
+	  notify_format(player, T("'%s' contains too many aliases."), s);
+	  break;
+	case OPAE_NULL:
+	  notify_format(player, T("Null aliases are not valid."));
+	  break;
+	}
+	if (opae_res != OPAE_SUCCESS)
+	  return -1;
+      }
     } else {
       /* No old alias */
-      if (s && *s && !ok_player_name(s, player)) {
-	notify(player, T("That is not a valid alias."));
-	return -1;
+      if (s && *s) {
+	int opae_res = ok_player_alias(s, player, thing);
+	switch (opae_res) {
+	case OPAE_INVALID:
+	  notify_format(player, T("'%s' is not a valid alias."), s);
+	  break;
+	case OPAE_TOOMANY:
+	  notify_format(player, T("'%s' contains too many aliases."), s);
+	  break;
+	case OPAE_NULL:
+	  notify_format(player, T("Null aliases are not valid."));
+	  break;
+	}
+	if (opae_res != OPAE_SUCCESS)
+	  return -1;
       }
     }
   } else if (s && *s && (!strcmp(name, "FORWARDLIST")
+			 || !strcmp(name, "MAILFORWARDLIST")
 			 || !strcmp(name, "DEBUGFORWARDLIST"))) {
     /* You can only set this to dbrefs of things you're allowed to
      * forward to. If you get one wrong, we puke.
@@ -1539,9 +1606,15 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
 	notify_format(player, T("Invalid dbref #%d in %s."), fwd, name);
 	return -1;
       }
-      if (!Can_Forward(thing, fwd)) {
-	notify_format(player, T("I don't think #%d wants to hear from you."),
-		      fwd);
+      if ((!strcmp(name, "FORWARDLIST") || !strcmp(name, "DEBUGFORWARDLIST"))
+	  && !Can_Forward(thing, fwd)) {
+	notify_format(player, T("I don't think #%d wants to hear from %s."),
+		      fwd, Name(thing));
+	return -1;
+      }
+      if (!strcmp(name, "MAILFORWARDLIST") && !Can_MailForward(thing, fwd)) {
+	notify_format(player, T("I don't think #%d wants %s's mail."), fwd,
+		      Name(thing));
 	return -1;
       }
     }
@@ -1571,17 +1644,19 @@ do_set_atr(dbref thing, const char *RESTRICT atr, const char *RESTRICT s,
     } else
       notify(player, T("That attribute cannot be changed by you."));
     return 0;
+  } else if (res == AE_TOOMANY) {
+    notify(player, T("Too many attributes on that object to add another."));
+    return 0;
   } else if (!res) {
     notify(player, T("No such attribute to reset."));
     return 0;
   }
   if (!strcmp(name, "ALIAS") && IsPlayer(thing)) {
-    if (s && *s) {
-      add_player(thing, s);
+    reset_player_list(thing, NULL, tbuf1, NULL, s);
+    if (s && *s)
       notify(player, T("Alias set."));
-    } else {
+    else
       notify(player, T("Alias removed."));
-    }
     return 1;
   } else if (!strcmp(name, "LISTEN")) {
     if (IsRoom(thing))
