@@ -186,6 +186,7 @@ typedef enum bvm_opcode {
   OP_TIP, /**< Tests IP^ARG */
   OP_THOSTNAME,	/**< Tests HOSTNAME^ARG */
   OP_TOBJID, /**< Tests OBJID^ARG */
+  OP_TDBREFLIST,	/**< Tests DBERFLIST^ARG */
   OP_LOADS, /**< Load ARG into S */
   OP_LOADR, /**< Load ARG into R */
   OP_NEGR,  /**< Negate R */
@@ -244,13 +245,14 @@ static struct flag_lock_types flag_locks[] = {
   {"OBJID", OP_TOBJID},
   {"IP", OP_TIP},
   {"HOSTNAME", OP_THOSTNAME},
-  {NULL, 0}
+  {"DBREFLIST", OP_TDBREFLIST}, {
+				 NULL, 0}
 };
 
-static char *
+static unsigned char *
 safe_get_bytecode(boolexp b)
   __attribute_malloc__;
-    static char *get_bytecode(boolexp b, u_int_16 * storelen);
+    static unsigned char *get_bytecode(boolexp b, u_int_16 * storelen);
     static struct boolexp_node *alloc_bool(void) __attribute_malloc__;
     static struct boolatr *alloc_atr(const char *name,
 				     const char *s) __attribute_malloc__;
@@ -309,9 +311,9 @@ safe_get_bytecode(boolexp b)
  * \param b the boolexp to retrieve.
  * \return a malloced copy of the bytecode.
  */
-    static char *safe_get_bytecode(boolexp b)
+    static unsigned char *safe_get_bytecode(boolexp b)
 {
-  char *bytecode;
+  unsigned char *bytecode;
   u_int_16 len;
 
   len = chunk_len(b);
@@ -324,10 +326,10 @@ safe_get_bytecode(boolexp b)
  * \param b The boolexp to retrieve.
  * \return a static copy of the bytecode.
  */
-static char *
+static unsigned char *
 get_bytecode(boolexp b, u_int_16 * storelen)
 {
-  static char bytecode[BUFFER_LEN * 2];
+  static unsigned char bytecode[BUFFER_LEN * 2];
   u_int_16 len;
 
   len = chunk_fetch(b, bytecode, sizeof bytecode);
@@ -348,7 +350,7 @@ boolexp
 dup_bool(boolexp b)
 {
   boolexp r;
-  char *bytecode;
+  unsigned char *bytecode;
   u_int_16 len = 0;
 
   if (b == TRUE_BOOLEXP)
@@ -418,7 +420,8 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
     int arg;
     ATTR *a;
     int r = 0;
-    char *s = NULL, *bytecode, *pc;
+    char *s = NULL;
+    unsigned char *bytecode, *pc;
 
     bytecode = pc = safe_get_bytecode(b);
 
@@ -441,7 +444,7 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
       case OP_PAREN:
 	break;
       case OP_LOADS:
-	s = bytecode + arg;
+	s = (char *) (bytecode + arg);
 	break;
       case OP_LOADR:
 	r = arg;
@@ -490,24 +493,24 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
 	else {
 	  char tbuf[BUFFER_LEN];
 	  strcpy(tbuf, atr_value(a));
-	  r = local_wild_match(bytecode + arg, tbuf);
+	  r = local_wild_match((char *) bytecode + arg, tbuf);
 	}
 	boolexp_recursion--;
 	break;
       case OP_TEVAL:
 	boolexp_recursion++;
-	r = check_attrib_lock(player, target, s, bytecode + arg);
+	r = check_attrib_lock(player, target, s, (char *) bytecode + arg);
 	boolexp_recursion--;
 	break;
       case OP_TFLAG:
 	/* Note that both fields of a boolattr struct are upper-cased */
-	if (sees_flag("FLAG", target, player, bytecode + arg))
+	if (sees_flag("FLAG", target, player, (char *) bytecode + arg))
 	  r = 1;
 	else
 	  r = 0;
 	break;
       case OP_TPOWER:
-	if (sees_flag("POWER", target, player, bytecode + arg))
+	if (sees_flag("POWER", target, player, (char *) bytecode + arg))
 	  r = 1;
 	else
 	  r = 0;
@@ -515,7 +518,7 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
       case OP_TOBJID:
 	{
 	  dbref d;
-	  d = parse_objid(bytecode + arg);
+	  d = parse_objid((char *) bytecode + arg);
 	  r = (player == d);
 	  break;
 	}
@@ -523,7 +526,7 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
 	{
 	  CHAN *chan;
 	  boolexp_recursion++;
-	  find_channel(bytecode + arg, &chan, target);
+	  find_channel((char *) bytecode + arg, &chan, target);
 	  r = chan && onchannel(player, chan);
 	  boolexp_recursion--;
 	}
@@ -542,7 +545,7 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
 	    r = 0;
 	  else {
 	    char *p = least_idle_ip(Owner(player));
-	    r = p ? quick_wild(bytecode + arg, p) : 0;
+	    r = p ? quick_wild((char *) bytecode + arg, p) : 0;
 	  }
 	}
 	boolexp_recursion--;
@@ -558,7 +561,7 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
 	    r = 0;
 	  else {
 	    char *p = least_idle_hostname(Owner(player));
-	    r = p ? quick_wild(bytecode + arg, p) : 0;
+	    r = p ? quick_wild((char *) bytecode + arg, p) : 0;
 	  }
 	}
 	boolexp_recursion--;
@@ -579,6 +582,29 @@ eval_boolexp(dbref player /* The player trying to pass */ ,
 	  break;
 	}
 	break;
+      case OP_TDBREFLIST:
+	{
+	  char *idstr, *curr, *orig;
+	  dbref mydb;
+
+	  r = 0;
+	  a = atr_get(target, bytecode + arg);
+	  if (!a)
+	    break;
+
+	  orig = safe_atr_value(a);
+	  idstr = trim_space_sep(orig, ' ');
+
+	  while ((curr = split_token(&idstr, ' ')) != NULL) {
+	    mydb = parse_objid(curr);
+	    if (mydb == player) {
+	      r = 1;
+	      break;
+	    }
+	  }
+	  free((Malloc_t) orig);
+	}
+	break;
       default:
 	do_log(LT_ERR, 0, 0, "Bad boolexp opcode %d %d in object #%d",
 	       op, arg, target);
@@ -629,15 +655,15 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
 {
   static char boolexp_buf[BUFFER_LEN];
   char *buftop = boolexp_buf;
-  char *bytecode = NULL;
+  unsigned char *bytecode = NULL;
 
   if (b == TRUE_BOOLEXP)
     safe_str("*UNLOCKED*", boolexp_buf, &buftop);
   else {
     bvm_opcode op;
     int arg;
-    char *pc;
-    char *s = NULL;
+    unsigned char *pc;
+    unsigned char *s = NULL;
 
     bytecode = pc = get_bytecode(b, NULL);
 
@@ -672,7 +698,7 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
       case OP_PAREN:
 	if (arg == 0) {
 	  int pstack = 1, parg;
-	  char *tpc = pc;
+	  unsigned char *tpc = pc;
 	  while (1) {
 	    if ((bvm_opcode) * tpc == OP_PAREN) {
 	      memcpy(&parg, tpc + 1, sizeof parg);
@@ -741,6 +767,9 @@ unparse_boolexp(dbref player, boolexp b, enum u_b_f flag)
       case OP_THOSTNAME:
 	safe_format(boolexp_buf, &buftop, "HOSTNAME^%s", bytecode + arg);
 	break;
+      case OP_TDBREFLIST:
+	safe_format(boolexp_buf, &buftop, "DBREFLIST^%s", bytecode + arg);
+	break;
       }
     }
   }
@@ -1539,7 +1568,7 @@ emit_bytecode(struct bvm_asm *a, int derefs)
   boolexp b;
   struct bvm_asmnode *i;
   struct bvm_strnode *s;
-  char *pc, *bytecode;
+  unsigned char *pc, *bytecode;
   u_int_16 len, blen;
 
   if (!a)
@@ -1583,6 +1612,7 @@ emit_bytecode(struct bvm_asm *a, int derefs)
     case OP_TCHANNEL:
     case OP_TIP:
     case OP_THOSTNAME:
+    case OP_TDBREFLIST:
       i->arg = blen + offset_to_string(a, i->arg);
       break;
     default:
@@ -1816,6 +1846,9 @@ print_bytecode(boolexp b)
     case OP_THOSTNAME:
       printf("THOSTNAME \"%s\"\n", bytecode + arg);
       break;
+    case OP_TDBREFLIST:
+      printf("TDBREFLIST \"%s\"\n", bytecode + arg);
+      break;
     case OP_LOADS:
       printf("LOADS \"%s\"\n", bytecode + arg);
       break;
@@ -1870,7 +1903,7 @@ warning_lock_type(const boolexp l)
 void
 check_lock(dbref player, dbref i, const char *name, boolexp be)
 {
-  char *pc, *bytecode;
+  unsigned char *pc, *bytecode;
   bvm_opcode op;
   int arg;
   char *s = NULL;
@@ -1885,7 +1918,7 @@ check_lock(dbref player, dbref i, const char *name, boolexp be)
     case OP_RET:
       return;
     case OP_LOADS:
-      s = bytecode + arg;
+      s = (char *) bytecode + arg;
       break;
     case OP_TCONST:
     case OP_TCARRY:
