@@ -238,7 +238,7 @@ COMMAND (cmd_disable) {
 
 COMMAND (cmd_doing) {
   if (SW_ISSET(sw, SWITCH_HEADER))
-    do_poll(player, arg_left);
+    do_poll(player, arg_left, 0);
   else
     do_doing(player, arg_left);
 }
@@ -262,10 +262,9 @@ COMMAND (cmd_dump) {
 }
 
 COMMAND (cmd_edit) {
-  if (SW_ISSET(sw, SWITCH_FIRST))
-    do_gedit(player, arg_left, args_right, EDIT_FIRST);
-  else
-    do_gedit(player, arg_left, args_right, EDIT_ALL);
+  do_gedit(player, arg_left, args_right,
+	   SW_ISSET(sw, SWITCH_FIRST) ? EDIT_FIRST : EDIT_ALL,
+	   SW_ISSET(sw, SWITCH_CHECK) ? 0 : 1);
 }
 
 COMMAND (cmd_elock) {
@@ -365,11 +364,11 @@ COMMAND (cmd_function) {
       *args_right[2]++ = '\0';
     }
     if (args_right[1] && *args_right[1])
-      do_function(player, arg_left, args_right);
+      do_function(player, arg_left, args_right, SW_ISSET(sw, SWITCH_PRESERVE));
     else if (arg_left && *arg_left)
       do_function_report(player, arg_left);
     else
-      do_function(player, NULL, NULL);
+      do_function(player, NULL, NULL, 0);
     if (split) {
       if (args_right[2])
 	*--args_right[2] = '/';
@@ -413,7 +412,10 @@ COMMAND (cmd_hook) {
     flags = HOOK_IGNORE;
   else if (SW_ISSET(sw, SWITCH_OVERRIDE))
     flags = HOOK_OVERRIDE;
-  else {
+  else if (SW_ISSET(sw, SWITCH_LIST)) {
+    do_hook_list(player, arg_left);
+    return;
+  } else {
     notify(player, T("You must give a switch for @hook"));
     return;
   }
@@ -424,6 +426,12 @@ COMMAND (cmd_huh_command) {
   notify(player, T("Huh?  (Type \"help\" for help.)"));
 }
 
+COMMAND (cmd_home) {
+  if (!Mobile(player))
+    return;
+  do_move(player, "home", MOVE_NORMAL);
+}
+
 COMMAND (cmd_kick) {
   do_kick(player, arg_left);
 }
@@ -454,6 +462,8 @@ COMMAND (cmd_list) {
     do_list_commands(player, lc);
   else if (SW_ISSET(sw, SWITCH_ATTRIBS))
     do_list_attribs(player, lc);
+  else if (SW_ISSET(sw, SWITCH_LOCKS))
+    do_list_locks(player, arg_left, lc, T("Locks"));
   else if (SW_ISSET(sw, SWITCH_FLAGS))
     do_list_flags("FLAG", player, arg_left, lc, T("Flags"));
   else if (SW_ISSET(sw, SWITCH_POWERS))
@@ -675,8 +685,22 @@ COMMAND (cmd_pemit) {
     do_pemit(player, arg_left, arg_right, flags);
 }
 
+COMMAND (cmd_prompt) {
+  int flags;
+  SPOOF(player, cause, sw);
+  if (SW_ISSET(sw, SWITCH_SILENT))
+    flags = PEMIT_SILENT;
+  else if (SW_ISSET(sw, SWITCH_NOISY))
+    flags = 0;
+  else
+    flags = SILENT_PEMIT ? PEMIT_SILENT : 0;
+  if (!strcmp(cmd->name, "@NSPEMIT"))
+    flags |= PEMIT_SPOOF;
+  do_pemit_list(player, arg_left, arg_right, flags | PEMIT_PROMPT);
+}
+
 COMMAND (cmd_poll) {
-  do_poll(player, arg_left);
+  do_poll(player, arg_left, SW_ISSET(sw, SWITCH_CLEAR));
 }
 
 COMMAND (cmd_poor) {
@@ -1062,19 +1086,16 @@ COMMAND (cmd_leave) {
 }
 
 COMMAND (cmd_page) {
-  char *t;
-  if ((arg_right) && (*arg_right)) {
-    t = arg_right;
-  } else {
-    t = NULL;
-  }
+  int has_eq = 0;
+  if (args_raw && *args_raw && strchr(args_raw, '='))
+    has_eq = 1;
   if (SW_ISSET(sw, SWITCH_PORT))
-    do_page_port(player, arg_left, t);
+    do_page_port(player, arg_left, arg_right);
   else
-    do_page(player, arg_left, t, cause, SW_ISSET(sw, SWITCH_NOEVAL),
+    do_page(player, arg_left, arg_right, cause, SW_ISSET(sw, SWITCH_NOEVAL),
 	    !(SW_ISSET(sw, SWITCH_BLIND) ||
 	      (!(SW_ISSET(sw, SWITCH_LIST)) && (BLIND_PAGE))),
-	    SW_ISSET(sw, SWITCH_OVERRIDE));
+	    SW_ISSET(sw, SWITCH_OVERRIDE), has_eq);
 }
 
 COMMAND (cmd_pose) {
