@@ -58,7 +58,7 @@ int run_hook(dbref player, dbref cause, struct hook_data *hook,
 COMLIST commands[] = {
 
   {"@COMMAND",
-   "ADD ALIAS DELETE EQSPLIT LSARGS RSARGS NOEVAL ON OFF QUIET ENABLE DISABLE RESTRICT",
+   "ADD ALIAS DELETE EQSPLIT LSARGS RSARGS NOEVAL ON OFF QUIET ENABLE DISABLE RESTRICT NOPARSE",
    cmd_command,
    CMD_T_PLAYER | CMD_T_EQSPLIT, 0, 0},
   {"@@", NULL, cmd_null, CMD_T_ANY | CMD_T_NOPARSE, 0, 0},
@@ -79,7 +79,8 @@ COMLIST commands[] = {
   {"@CHANNEL",
    "LIST ADD DELETE RENAME NAME PRIVS QUIET NOISY DECOMPILE DESCRIBE CHOWN WIPE MUTE UNMUTE GAG UNGAG HIDE UNHIDE WHAT TITLE BRIEF RECALL BUFFER SET",
    cmd_channel,
-   CMD_T_ANY | CMD_T_SWITCHES | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
+   CMD_T_ANY | CMD_T_SWITCHES | CMD_T_EQSPLIT | CMD_T_NOGAGGED | CMD_T_RS_ARGS,
+   0, 0},
   {"@CHAT", NULL, cmd_chat, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@CHOWNALL", "PRESERVE", cmd_chownall, CMD_T_ANY | CMD_T_EQSPLIT, "WIZARD",
    0},
@@ -120,7 +121,7 @@ COMLIST commands[] = {
   {"@DRAIN", "ALL ANY", cmd_notify_drain, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@DUMP", "PARANOID DEBUG", cmd_dump, CMD_T_ANY, "WIZARD", 0},
 
-  {"@EDIT", "FIRST", cmd_edit,
+  {"@EDIT", "FIRST CHECK", cmd_edit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_RS_NOPARSE |
    CMD_T_NOGAGGED, 0, 0},
   {"@ELOCK", NULL, cmd_elock, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
@@ -142,13 +143,13 @@ COMLIST commands[] = {
 
   {"@FORCE", "NOEVAL", cmd_force, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
    0, 0},
-  {"@FUNCTION", "DELETE ENABLE DISABLE RESTORE RESTRICT", cmd_function,
+  {"@FUNCTION", "DELETE ENABLE DISABLE PRESERVE RESTORE RESTRICT", cmd_function,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_NOGAGGED, 0, 0},
   {"@GREP", "LIST PRINT ILIST IPRINT", cmd_grep,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE | CMD_T_NOGAGGED, 0, 0},
   {"@HALT", "ALL", cmd_halt, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@HIDE", "NO OFF YES ON", cmd_hide, CMD_T_ANY, 0, 0},
-  {"@HOOK", "AFTER BEFORE IGNORE OVERRIDE", cmd_hook,
+  {"@HOOK", "LIST AFTER BEFORE IGNORE OVERRIDE", cmd_hook,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
    "WIZARD", 0},
   {"@KICK", NULL, cmd_kick, CMD_T_ANY, "WIZARD", 0},
@@ -159,7 +160,8 @@ COMLIST commands[] = {
    0},
   {"@LISTMOTD", NULL, cmd_listmotd, CMD_T_ANY, 0, 0},
 
-  {"@LIST", "LOWERCASE MOTD FLAGS FUNCTIONS POWERS COMMANDS ATTRIBS", cmd_list,
+  {"@LIST", "LOWERCASE MOTD LOCKS FLAGS FUNCTIONS POWERS COMMANDS ATTRIBS",
+   cmd_list,
    CMD_T_ANY, 0, 0},
   {"@LOCK", NULL, cmd_lock,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_SWITCHES | CMD_T_NOGAGGED, 0, 0},
@@ -199,6 +201,8 @@ COMLIST commands[] = {
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, "WIZARD", "CAN_NSPEMIT"},
   {"@NSPEMIT", "LIST SILENT NOISY NOEVAL", cmd_pemit,
    CMD_T_ANY | CMD_T_EQSPLIT, "WIZARD", "CAN_NSPEMIT"},
+  {"@NSPROMPT", "SILENT NOISY NOEVAL", cmd_prompt,
+   CMD_T_ANY | CMD_T_EQSPLIT, "WIZARD", "CAN_NSPEMIT"},
   {"@NSREMIT", "LIST NOEVAL NOISY SILENT SPOOF", cmd_remit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, "WIZARD", "CAN_NSPEMIT"},
   {"@NSZEMIT", NULL, cmd_zemit, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
@@ -216,10 +220,12 @@ COMLIST commands[] = {
 
   {"@PEMIT", "LIST CONTENTS SILENT NOISY NOEVAL SPOOF", cmd_pemit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
-  {"@POLL", NULL, cmd_poll, CMD_T_ANY, 0, 0},
+  {"@POLL", "CLEAR", cmd_poll, CMD_T_ANY, 0, 0},
   {"@POOR", NULL, cmd_poor, CMD_T_ANY, 0, 0},
   {"@POWER", "ADD TYPE LETTER LIST RESTRICT DELETE ALIAS DISABLE ENABLE",
-   cmd_power, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS, "WIZARD", 0},
+   cmd_power, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS, 0, 0},
+  {"@PROMPT", "SILENT NOISY NOEVAL SPOOF", cmd_prompt,
+   CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@PS", "ALL SUMMARY COUNT QUICK", cmd_ps, CMD_T_ANY, 0, 0},
   {"@PURGE", NULL, cmd_purge, CMD_T_ANY, 0, 0},
   {"@QUOTA", "ALL SET", cmd_quota, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
@@ -300,6 +306,7 @@ COMLIST commands[] = {
   {"GIVE", "SILENT", cmd_give, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0,
    0},
   {"GOTO", NULL, cmd_goto, CMD_T_PLAYER | CMD_T_THING, 0, 0},
+  {"HOME", NULL, cmd_home, CMD_T_PLAYER | CMD_T_THING, 0, 0},
   {"INVENTORY", NULL, cmd_inventory, CMD_T_ANY, 0, 0},
 
   {"KILL", NULL, cmd_kill, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
@@ -343,6 +350,10 @@ COMLIST commands[] = {
   {"HUH_COMMAND", NULL, cmd_huh_command,
    CMD_T_ANY | CMD_T_NOPARSE | CMD_T_INTERNAL, 0, 0},
 
+/* A way to let people override the unimplemented message */
+  {"UNIMPLEMENTED_COMMAND", NULL, cmd_unimplemented,
+   CMD_T_ANY | CMD_T_NOPARSE | CMD_T_INTERNAL, 0, 0},
+
   {NULL, NULL, NULL, 0, 0, 0}
 };
 
@@ -856,6 +867,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
   char b;
   int switchnum;
   switch_mask sw;
+  char switch_err[BUFFER_LEN], *se;
   int noeval;
   int noevtoken = 0;
   char *retval;
@@ -951,7 +963,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
      * usual processing. Exits have next priority.  We still pass them 
      * through the parser so @hook on GOTO can work on them.
      */
-    if (can_move(player, p)) {
+    if (strcasecmp(p, "home") && can_move(player, p)) {
       ec = exit_command;
       safe_str("GOTO ", exit_command, &ec);
       safe_str(p, exit_command, &ec);
@@ -1046,6 +1058,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
   SW_ZERO(sw);
   swp = switches;
   *swp = '\0';
+  se = switch_err;
 
   t = NULL;
 
@@ -1064,16 +1077,17 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
 	    strcat(swp, " ");
 	  strcat(swp, swtch);
 	} else {
-	  notify_format(player,
+	  if (se == switch_err)
+	    safe_format(switch_err, &se,
 			T("%s doesn't know switch %s."), cmd->name, swtch);
-	  command_parse_free_args;
-	  return NULL;
 	}
       } else {
 	SW_SET(sw, switchnum);
       }
     }
   }
+
+  *se = '\0';
   if (!t)
     SW_SET(sw, SWITCH_NONE);
   if (noevtoken)
@@ -1141,20 +1155,22 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
       safe_str(ls, commandraw, &c2);
     }
     if (cmd->type & CMD_T_EQSPLIT) {
-      safe_chr('=', commandraw, &c2);
-      if (cmd->type & CMD_T_RS_ARGS) {
-	int rsa_index;
-	/* This is counterintuitive, but rsa[]
-	 * starts at 1. */
-	if (rsa[1]) {
-	  safe_str(rsa[1], commandraw, &c2);
-	  for (rsa_index = 2; rsa[rsa_index]; rsa_index++) {
-	    safe_chr(',', commandraw, &c2);
-	    safe_str(rsa[rsa_index], commandraw, &c2);
+      if (rhs_present) {
+	safe_chr('=', commandraw, &c2);
+	if (cmd->type & CMD_T_RS_ARGS) {
+	  int rsa_index;
+	  /* This is counterintuitive, but rsa[]
+	   * starts at 1. */
+	  if (rsa[1]) {
+	    safe_str(rsa[1], commandraw, &c2);
+	    for (rsa_index = 2; rsa[rsa_index]; rsa_index++) {
+	      safe_chr(',', commandraw, &c2);
+	      safe_str(rsa[rsa_index], commandraw, &c2);
+	    }
 	  }
+	} else {
+	  safe_str(rs, commandraw, &c2);
 	}
-      } else {
-	safe_str(rs, commandraw, &c2);
       }
 #ifdef NEVER
       /* We used to do this, but we're not sure why */
@@ -1170,6 +1186,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
   retval = NULL;
   if (cmd->func == NULL) {
     do_rawlog(LT_ERR, T("No command vector on command %s."), cmd->name);
+    command_parse_free_args;
     return NULL;
   } else {
     char *saveregs[NUMQ];
@@ -1181,6 +1198,13 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
 	  !one_comm_match(cmd->hooks.override.obj, player,
 			  cmd->hooks.override.attrname, commandraw)) {
 	/* Otherwise, we do hook/before, the command, and hook/after */
+	/* But first, let's see if we had an invalid switch */
+	if (*switch_err) {
+	  notify(player, switch_err);
+	  free_global_regs("hook.regs", saveregs);
+	  command_parse_free_args;
+	  return NULL;
+	}
 	run_hook(player, cause, &cmd->hooks.before, saveregs, 1);
 	cmd->func(cmd, player, cause, sw, string, swp, ap, ls, lsa, rs, rsa);
 	run_hook(player, cause, &cmd->hooks.after, saveregs, 0);
@@ -1347,6 +1371,31 @@ restrict_command(const char *name, const char *restriction)
  * with "This command has not been implemented"
  */
 COMMAND (cmd_unimplemented) {
+  char *saveregs[NUMQ];
+
+  if (strcmp(cmd->name, "UNIMPLEMENTED_COMMAND") != 0 &&
+      (cmd = command_find("UNIMPLEMENTED_COMMAND"))) {
+    if (!(cmd->type & CMD_T_DISABLED)) {
+      init_global_regs(saveregs);
+      if (run_hook(player, cause, &cmd->hooks.ignore, saveregs, 1)) {
+	/* If we have a hook/override, we use that instead */
+	if (!has_hook(&cmd->hooks.override) ||
+	    !one_comm_match(cmd->hooks.override.obj, player,
+			    cmd->hooks.override.attrname, "HUH_COMMAND")) {
+	  /* Otherwise, we do hook/before, the command, and hook/after */
+	  run_hook(player, cause, &cmd->hooks.before, saveregs, 1);
+
+	  cmd->func(cmd, player, cause, sw, raw, switches, args_raw,
+		    arg_left, args_left, arg_right, args_right);
+	  run_hook(player, cause, &cmd->hooks.after, saveregs, 0);
+	}
+      }
+      free_global_regs("hook.regs", saveregs);
+      return;
+    }
+  }
+
+  /* Either we were already in UNIMPLEMENTED_COMMAND, or we couldn't find it */
   notify(player, "This command has not been implemented");
 }
 
@@ -1461,11 +1510,15 @@ COMMAND (cmd_command) {
   }
   if (SW_ISSET(sw, SWITCH_ADD)) {
     int flags = CMD_T_ANY;
-    flags |= SW_ISSET(sw, SWITCH_NOEVAL) ? CMD_T_NOPARSE : 0;
+    flags |= SW_ISSET(sw, SWITCH_NOPARSE) ? CMD_T_NOPARSE : 0;
     flags |= SW_ISSET(sw, SWITCH_RSARGS) ? CMD_T_RS_ARGS : 0;
     flags |= SW_ISSET(sw, SWITCH_LSARGS) ? CMD_T_LS_ARGS : 0;
     flags |= SW_ISSET(sw, SWITCH_LSARGS) ? CMD_T_LS_ARGS : 0;
     flags |= SW_ISSET(sw, SWITCH_EQSPLIT) ? CMD_T_EQSPLIT : 0;
+    if (SW_ISSET(sw, SWITCH_NOEVAL))
+      notify(player,
+	     T
+	     ("WARNING: /NOEVAL no longer creates a Noparse command.\n         Use /NOPARSE if that's what you meant."));
     do_command_add(player, arg_left, flags);
     return;
   }
@@ -1591,26 +1644,11 @@ COMMAND (cmd_command) {
       *bp = '\0';
       notify_format(player, "Arguments  : %s", buff);
     }
-    if (Wizard(player)) {
-      if (GoodObject(command->hooks.before.obj))
-	notify_format(player, "@hook/before: #%d/%s",
-		      command->hooks.before.obj,
-		      command->hooks.before.attrname);
-      if (GoodObject(command->hooks.after.obj))
-	notify_format(player, "@hook/after: #%d/%s", command->hooks.after.obj,
-		      command->hooks.after.attrname);
-      if (GoodObject(command->hooks.ignore.obj))
-	notify_format(player, "@hook/ignore: #%d/%s",
-		      command->hooks.ignore.obj,
-		      command->hooks.ignore.attrname);
-      if (GoodObject(command->hooks.override.obj))
-	notify_format(player, "@hook/override: #%d/%s",
-		      command->hooks.override.obj,
-		      command->hooks.override.attrname);
-    }
+    do_hook_list(player, arg_left);
   }
 }
 
+
 /** Display a list of defined commands.
  * This function sends a player the list of commands.
  * \param player the enactor.
@@ -1641,7 +1679,7 @@ list_commands(void)
     command = (COMMAND_INFO *) ptab_nextentry(&ptab_command);
   }
 
-  do_gensort((dbref) 0, (char **) ptrs, nptrs, ALPHANUM_LIST);
+  do_gensort((dbref) 0, (char **) ptrs, NULL, nptrs, ALPHANUM_LIST);
   bp = buff;
   safe_str(ptrs[0], buff, &bp);
   for (i = 1; i < nptrs; i++) {
@@ -1713,9 +1751,12 @@ command_check(dbref player, COMMAND_INFO *cmd)
   check_flags = cmd->flagmask && !null_flagmask("FLAG", cmd->flagmask);
   check_powers = cmd->powers && !null_flagmask("POWER", cmd->powers);
 
-  if (check_flags)
+  if (check_flags && check_powers)
+    ok = !!(has_any_flags_by_mask(player, cmd->flagmask)
+	    || has_any_powers_by_mask(player, cmd->powers));
+  else if (check_flags)
     ok = !!(has_any_flags_by_mask(player, cmd->flagmask));
-  if (!ok && check_powers)
+  else if (check_powers)
     ok = !!(has_any_powers_by_mask(player, cmd->powers));
   if (!ok) {
     mess = T("Permission denied.");
@@ -1871,3 +1912,38 @@ do_hook(dbref player, char *command, char *obj, char *attrname,
     notify_format(player, T("Hook set for %s"), cmd->name);
   }
 }
+
+
+/** List command hooks.
+ * \verbatim
+ * This is the top-level function for @hook/list, @list/hooks, and
+ * the hook-listing part of @command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param command command to list hooks on.
+ */
+void
+do_hook_list(dbref player, char *command)
+{
+  COMMAND_INFO *cmd;
+
+  cmd = command_find(command);
+  if (!cmd) {
+    notify(player, T("No such command."));
+    return;
+  }
+  if (Wizard(player)) {
+    if (GoodObject(cmd->hooks.before.obj))
+      notify_format(player, "@hook/before: #%d/%s",
+		    cmd->hooks.before.obj, cmd->hooks.before.attrname);
+    if (GoodObject(cmd->hooks.after.obj))
+      notify_format(player, "@hook/after: #%d/%s", cmd->hooks.after.obj,
+		    cmd->hooks.after.attrname);
+    if (GoodObject(cmd->hooks.ignore.obj))
+      notify_format(player, "@hook/ignore: #%d/%s",
+		    cmd->hooks.ignore.obj, cmd->hooks.ignore.attrname);
+    if (GoodObject(cmd->hooks.override.obj))
+      notify_format(player, "@hook/override: #%d/%s",
+		    cmd->hooks.override.obj, cmd->hooks.override.attrname);
+  }
+}
