@@ -243,6 +243,8 @@ PENNCONF conftable[] = {
   {"player_name_spaces", cf_bool, &options.player_name_spaces, 2, 0,
    "cosmetic"}
   ,
+  {"max_aliases", cf_int, &options.max_aliases, -1, 0, "limits"}
+  ,
   {"ansi_names", cf_bool, &options.ansi_names, 2, 0, "cosmetic"}
   ,
   {"only_ascii_in_names", cf_bool, &options.ascii_names, 2, 0, "cosmetic"}
@@ -292,6 +294,8 @@ PENNCONF conftable[] = {
   {"unconnected_idle_timeout", cf_time, &options.unconnected_idle_timeout,
    100000, 0, "limits"}
   ,
+  {"keepalive_timeout", cf_time, &options.keepalive_timeout, 10000, 0, "limits"}
+  ,
   {"whisper_loudness", cf_int, &options.whisper_loudness, 100, 0, "limits"}
   ,
   {"starting_quota", cf_int, &options.starting_quota, 10000, 0, "limits"}
@@ -384,6 +388,10 @@ PENNCONF conftable[] = {
   {"thing_flags", cf_flag, options.thing_flags, sizeof options.thing_flags, 0,
    "flags"}
   ,
+  {"channel_flags", cf_flag, options.channel_flags,
+   sizeof options.channel_flags, 0,
+   "flags"}
+  ,
 
   {"safer_ufun", cf_bool, &options.safer_ufun, 2, 0, "funcs"}
   ,
@@ -1029,6 +1037,7 @@ conf_default_set(void)
   options.ancestor_player = -1;
   options.idle_timeout = 0;
   options.unconnected_idle_timeout = 300;
+  options.keepalive_timeout = 300;
   options.dump_interval = 3601;
   strcpy(options.dump_message,
 	 T("GAME: Dumping database. Game may freeze for a minute"));
@@ -1089,6 +1098,7 @@ conf_default_set(void)
   strcpy(options.room_flags, "");
   strcpy(options.exit_flags, "");
   strcpy(options.thing_flags, "");
+  strcpy(options.channel_flags, "");
   options.warn_interval = 3600;
   options.use_dns = 1;
   options.safer_ufun = 1;
@@ -1125,6 +1135,7 @@ conf_default_set(void)
   strcpy(options.ip_addr, "");
   strcpy(options.ssl_ip_addr, "");
   options.player_name_spaces = 0;
+  options.max_aliases = 3;
   options.forking_dump = 1;
   options.restrict_building = 0;
   options.free_objects = 1;
@@ -1598,11 +1609,7 @@ show_compile_options(dbref player)
   notify(player, T(" DNS and ident lookups are handled by the MUSH process."));
 #endif
 
-#ifdef NT_TCP
-  notify(player, T(" Windows NT native TCP/IP networking functions in use."));
-#else
   notify(player, T(" BSD sockets networking in use."));
-#endif
 
 #ifdef HAS_GETDATE
   notify(player, T(" Extended convtime() is supported."));
