@@ -188,10 +188,29 @@ pay_queue(dbref player, const char *command)
   estcost =
     QUEUE_COST +
     (QUEUE_LOSS ? ((get_random_long(0, QUEUE_LOSS - 1) == 0) ? 1 : 0) : 0);
-  if (!payfor(player, estcost)) {
+  if (!quiet_payfor(player, estcost)) {
     notify(Owner(player), T("Not enough money to queue command."));
     return 0;
   }
+  if (!NoPay(player) && (estcost != QUEUE_COST) && Track_Money(Owner(player))) {
+    char *preserve_wnxt[10];
+    char *preserve_rnxt[NUMQ];
+    char *val_wnxt[10];
+    char *val_rnxt[NUMQ];
+    char *preserves[10];
+    char *preserveq[NUMQ];
+    save_global_nxt("pay_queue_save", preserve_wnxt, preserve_rnxt, val_wnxt,
+		    val_rnxt);
+    save_global_regs("pay_queue_save", preserveq);
+    save_global_env("pay_queue_save", preserves);
+    notify_format(Owner(player),
+		  T("GAME: Object %s(%s) lost a %s to queue loss."),
+		  Name(player), unparse_dbref(player), MONEY);
+    restore_global_regs("pay_queue_save", preserveq);
+    restore_global_env("pay_queue_save", preserves);
+    restore_global_nxt("pay_queue_save", preserve_wnxt, preserve_rnxt, val_wnxt,
+		       val_rnxt);
+  }
   if (queue_limit(QUEUE_PER_OWNER ? Owner(player) : player)) {
     notify_format(Owner(player),
 		  T("Runaway object: %s(%s). Commands halted."),
@@ -277,12 +296,25 @@ queue_attribute_base(dbref executor, const char *atrname, dbref enactor,
 		     int noparent)
 {
   ATTR *a;
-  char *start, *command;
 
-  a = (noparent ? atr_get_noparent(executor, strupper(atrname)) :
-       atr_get(executor, strupper(atrname)));
+  a = queue_attribute_getatr(executor, atrname, noparent);
   if (!a)
     return 0;
+  queue_attribute_useatr(executor, a, enactor);
+  return 1;
+}
+
+ATTR *
+queue_attribute_getatr(dbref executor, const char *atrname, int noparent)
+{
+  return (noparent ? atr_get_noparent(executor, strupper(atrname)) :
+	  atr_get(executor, strupper(atrname)));
+}
+
+int
+queue_attribute_useatr(dbref executor, ATTR *a, dbref enactor)
+{
+  char *start, *command;
   start = safe_atr_value(a);
   command = start;
   /* Trim off $-command or ^-command prefix */
@@ -302,6 +334,7 @@ queue_attribute_base(dbref executor, const char *atrname, dbref enactor,
   return 1;
 }
 
+
 /** Queue an entry on the wait or semaphore queues.
  * This function creates and adds a queue entry to the wait queue
  * or the semaphore queue. Wait queue entries are sorted by when
@@ -483,8 +516,10 @@ do_top(int ncom)
 
   for (i = 0; i < ncom; i++) {
 
-    if (!qfirst)
+    if (!qfirst) {
+      strcpy(global_eval_context.ccom, "");
       return i;
+    }
     /* We must dequeue before execution, so that things like
      * queued @kick or @ps get a sane queue image.
      */
@@ -506,6 +541,7 @@ do_top(int ncom)
 	  else
 	    global_eval_context.renv[a][0] = '\0';
 	}
+	global_eval_context.process_command_port = 0;
 	s = entry->comm;
 	global_eval_context.break_called = 0;
 	break_count = 100;
