@@ -1187,7 +1187,7 @@ get_list(FILE * f, dbref i)
     }
 }
 
-extern PRIV attr_privs[];
+extern PRIV attr_privs_view[];
 
 /** Read an attribute list for an object from a file
  * \param f file pointer to read from.
@@ -1211,7 +1211,7 @@ db_read_attrs(FILE * f, dbref i, int count)
     strcpy(name, tmp);
     db_read_this_labeled_dbref(f, "owner", &owner);
     db_read_this_labeled_string(f, "flags", &tmp);
-    flags = string_to_privs(attr_privs, tmp, 0);
+    flags = string_to_privs(attr_privs_view, tmp, 0);
     db_read_this_labeled_number(f, "derefs", &derefs);
     db_read_this_labeled_string(f, "value", &tmp);
     strcpy(value, tmp);
@@ -1400,7 +1400,7 @@ db_read_oldstyle(FILE * f)
       }
       /* check to see if it's a player */
       if (IsPlayer(i)) {
-	add_player(i, NULL);
+	add_player(i);
 	clear_flag_internal(i, "CONNECTED");
       }
       break;
@@ -1666,7 +1666,7 @@ db_read(FILE * f)
 		    i);
 	}
 	if (IsPlayer(i)) {
-	  add_player(i, NULL);
+	  add_player(i);
 	  clear_flag_internal(i, "CONNECTED");
 	}
       }
@@ -1793,7 +1793,7 @@ create_minimal_db(void)
   atr_new_add(god, "MAILCURF", "0", god, AF_LOCKED | AF_NOPROG | AF_WIZARD, 1);
   add_folder_name(god, 0, "inbox");
   PUSH(god, Contents(start_room));
-  add_player(god, NULL);
+  add_player(god);
   s_Pennies(god, START_BONUS);
   local_data_create(god);
   current_state.players++;
