@@ -1125,7 +1125,10 @@ attribute_owner_helper(dbref player __attribute__ ((__unused__)),
 static void
 check_connected_rooms(void)
 {
-  mark_connected(BASE_ROOM);
+  dbref room;
+  for (room = 0; room < db_top; ++room)
+    if ((room == BASE_ROOM) || (IsRoom(room) && Floating(room)))
+      mark_connected(room);
   check_connected_marks();
 }
 
