@@ -66,8 +66,7 @@ static enum cmatch_type find_channel_partial_on(const char *name, CHAN **chan,
 						dbref player);
 static enum cmatch_type find_channel_partial_off(const char *name, CHAN **chan,
 						 dbref player);
-static char *list_cflags(CHAN *c);
-static char *list_cuflags(CHANUSER *u);
+static char *list_cuflags(CHANUSER *u, int verbose);
 static void channel_join_self(dbref player, const char *name);
 static void channel_leave_self(dbref player, const char *name);
 static void do_channel_who(dbref player, CHAN *chan);
@@ -112,6 +111,14 @@ static PRIV priv_table[] = {
   {"NoTitles", 'T', CHANNEL_NOTITLES, CHANNEL_NOTITLES},
   {"NoNames", 'N', CHANNEL_NONAMES, CHANNEL_NONAMES},
   {"NoCemit", 'C', CHANNEL_NOCEMIT, CHANNEL_NOCEMIT},
+  {"Interact", 'I', CHANNEL_INTERACT, CHANNEL_INTERACT},
+  {NULL, '\0', 0, 0}
+};
+
+static PRIV chanuser_priv[] = {
+  {"Quiet", 'Q', CU_QUIET, CU_QUIET},
+  {"Hide", 'H', CU_HIDE, CU_HIDE},
+  {"Gag", 'G', CU_GAG, CU_GAG},
   {NULL, '\0', 0, 0}
 };
 
@@ -1100,11 +1107,11 @@ do_channel(dbref player, const char *name, const char *target, const char *com)
   CHANUSER *u;
   dbref victim;
 
-  if (!name && !*name) {
+  if (!name || !*name) {
     notify(player, T("You need to specify a channel."));
     return;
   }
-  if (!com && !*com) {
+  if (!com || !*com) {
     notify(player, T("What do you want to do with that channel?"));
     return;
   }
@@ -1470,7 +1477,7 @@ do_chat(dbref player, CHAN *chan, const char *arg1)
 		  ChanName(chan));
     return;
   }
-  if (!Chan_Can_Speak(chan, player)) {
+  if (!Loud(player) && !Chan_Can_Speak(chan, player)) {
     if (Chan_Can_See(chan, player))
       notify_format(player,
 		    T("Sorry, you're not allowed to speak on channel <%s>."),
@@ -1687,7 +1694,10 @@ do_chan_admin(dbref player, char *name, const char *perms, int flag)
       return;
     }
     /* get the permissions. Invalid specs default to the default */
-    type = string_to_privs(priv_table, perms, 0);
+    if (!perms || !*perms)
+      type = string_to_privs(priv_table, options.channel_flags, 0);
+    else
+      type = string_to_privs(priv_table, perms, 0);
     if (!Chan_Can(player, type)) {
       notify(player, T("You can't create channels of that type."));
       return;
@@ -1981,6 +1991,9 @@ do_chan_title(dbref player, const char *name, const char *title)
     notify(player, T("You must specify a channel."));
     return;
   }
+  if (!title)
+    title = (const char *) "";
+
   if (strlen(title) >= CU_TITLE_LEN) {
     notify(player, T("Title too long."));
     return;
@@ -2022,8 +2035,8 @@ do_channel_list(dbref player, const char *partname)
   CHANUSER *u;
   char numusers[BUFFER_LEN];
   char cleanname[CHAN_NAME_LEN];
-  const char thirtyblanks[31] = "                              ";
-  char blanks[31];
+  char blanks[CHAN_NAME_LEN];
+  int len;
   int numblanks;
 
   if (SUPPORT_PUEBLO)
@@ -2041,9 +2054,21 @@ do_channel_list(dbref player, const char *partname)
 		TAG_START, TAG_END);
       else
 	sprintf(numusers, "%5ld", ChanNumUsers(c));
-      numblanks = strlen(ChanName(c)) - strlen(cleanname);
-      if (numblanks > 0 && numblanks < 31) {
-	strcpy(blanks, thirtyblanks);
+      /* Display length is strlen(cleanname), but actual length is
+       * strlen(ChanName(c)). There are two different cases:
+       * 1. actual length <= 30. No problems.
+       * 2. actual length > 30, we must reduce the number of
+       * blanks we add by the (actual length-30) because our
+       * %-30s is going to overflow as well.
+       */
+      len = strlen(ChanName(c));
+      numblanks = len - strlen(cleanname);
+      if (numblanks > 0 && numblanks < CHAN_NAME_LEN) {
+	memset(blanks, ' ', CHAN_NAME_LEN - 1);
+	if (len > 30)
+	  numblanks -= (len - 30);
+	if (numblanks < 0)
+	  numblanks = 0;
 	blanks[numblanks] = '\0';
       } else {
 	blanks[0] = '\0';
@@ -2077,51 +2102,24 @@ do_channel_list(dbref player, const char *partname)
 }
 
 static char *
-list_cflags(CHAN *c)
+list_cuflags(CHANUSER *u, int verbose)
 {
   static char tbuf1[BUFFER_LEN];
   char *bp;
 
+  /* We have to handle hide separately, since it can be the player */
   bp = tbuf1;
-  if (Channel_Disabled(c))
-    safe_chr('D', tbuf1, &bp);
-  if (Channel_Player(c))
-    safe_chr('P', tbuf1, &bp);
-  if (Channel_Object(c))
-    safe_chr('O', tbuf1, &bp);
-  if (Channel_Admin(c))
-    safe_chr('A', tbuf1, &bp);
-  if (Channel_Wizard(c))
-    safe_chr('W', tbuf1, &bp);
-  if (Channel_Quiet(c))
-    safe_chr('Q', tbuf1, &bp);
-  if (Channel_CanHide(c))
-    safe_chr('H', tbuf1, &bp);
-  if (Channel_Open(c))
-    safe_chr('o', tbuf1, &bp);
-  if (Channel_NoTitles(c))
-    safe_chr('T', tbuf1, &bp);
-  if (Channel_NoNames(c))
-    safe_chr('N', tbuf1, &bp);
-  if (Channel_NoCemit(c))
-    safe_chr('C', tbuf1, &bp);
-  *bp = '\0';
-  return tbuf1;
-}
-
-static char *
-list_cuflags(CHANUSER *u)
-{
-  static char tbuf1[BUFFER_LEN];
-  char *bp;
-
-  bp = tbuf1;
-  if (Chanuser_Gag(u))
-     safe_chr('G', tbuf1, &bp);
-  if (Chanuser_Quiet(u))
-     safe_chr('Q', tbuf1, &bp);
-  if (Chanuser_Hide(u))
-     safe_chr('H', tbuf1, &bp);
+  if (verbose) {
+    if (Chanuser_Hide(u))
+       safe_str("Hide ", tbuf1, &bp);
+    safe_str(privs_to_string(chanuser_priv, CUtype(u) & ~CU_HIDE), tbuf1, &bp);
+    if (bp > tbuf1 && *bp == ' ')
+      bp--;
+  } else {
+    if (Chanuser_Hide(u))
+       safe_chr('H', tbuf1, &bp);
+    safe_str(privs_to_letters(chanuser_priv, CUtype(u) & ~CU_HIDE), tbuf1, &bp);
+  }
   *bp = '\0';
   return tbuf1;
 }
@@ -2157,7 +2155,10 @@ FUNCTION(fun_cflags)
       return;
     }
     if (nargs == 1) {
-      safe_str(list_cflags(c), buff, bp);
+      if (string_prefix(called_as, "CL"))
+	safe_str(privs_to_string(priv_table, ChanType(c)), buff, bp);
+      else
+	safe_str(privs_to_letters(priv_table, ChanType(c)), buff, bp);
       return;
     }
     thing = match_thing(executor, args[1]);
@@ -2174,11 +2175,48 @@ FUNCTION(fun_cflags)
       safe_str(T("#-1 NOT ON CHANNEL"), buff, bp);
       return;
     }
-    safe_str(list_cuflags(u), buff, bp);
+    safe_str(list_cuflags(u, string_prefix(called_as, "CL") ? 1 : 0), buff, bp);
     break;
   }
 }
 
+/* ARGSUSED */
+FUNCTION(fun_cinfo)
+{
+  /* Can be called as CDESC, CBUFFER, CUSERS, CMSGS */
+  CHAN *c;
+  if (!args[0] || !*args[0]) {
+    safe_str(T("#-1 NO CHANNEL GIVEN"), buff, bp);
+    return;
+  }
+  switch (find_channel(args[0], &c, executor)) {
+  case CMATCH_NONE:
+    safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+    return;
+  case CMATCH_AMBIG:
+    safe_str(T("#-1 AMBIGUOUS CHANNEL NAME"), buff, bp);
+    return;
+  default:
+    if (!Chan_Can_See(c, executor)) {
+      safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+      return;
+    }
+    if (string_prefix(called_as, "CD")) {
+      safe_str(ChanTitle(c), buff, bp);
+    } else if (string_prefix(called_as, "CB")) {
+      if (ChanBufferQ(c) != NULL) {
+	safe_integer(BufferQSize(ChanBufferQ(c)), buff, bp);
+      } else {
+	safe_integer(0, buff, bp);
+      }
+    } else if (string_prefix(called_as, "CU")) {
+      safe_integer(ChanNumUsers(c), buff, bp);
+    } else if (string_prefix(called_as, "CM")) {
+      safe_integer(ChanNumMsgs(c), buff, bp);
+    }
+  }
+}
+
 
 /* ARGSUSED */
 FUNCTION(fun_ctitle)
@@ -2238,6 +2276,57 @@ FUNCTION(fun_ctitle)
   }
 }
 
+/* ARGSUSED */
+FUNCTION(fun_cstatus)
+{
+  /* cstatus(<channel>,<object>) returns the object's status on that chan.
+   * You must pass the channel's see-lock, and
+   * either you must either be able to examine <object>, or
+   * you must be Priv_Who or <object> must not be hidden
+   */
+  CHAN *c;
+  CHANUSER *u;
+  dbref thing;
+
+  if (!args[0] || !*args[0]) {
+    safe_str(T("#-1 NO CHANNEL GIVEN"), buff, bp);
+    return;
+  }
+  switch (find_channel(args[0], &c, executor)) {
+  case CMATCH_NONE:
+    safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+    return;
+  case CMATCH_AMBIG:
+    safe_str(T("#-1 AMBIGUOUS CHANNEL NAME"), buff, bp);
+    return;
+  default:
+    thing = match_thing(executor, args[1]);
+    if (thing == NOTHING) {
+      safe_str(T(e_match), buff, bp);
+      return;
+    }
+    if (!Chan_Can_See(c, executor)) {
+      safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+      return;
+    }
+    u = onchannel(thing, c);
+    if (!u ||(!IsThing(thing) && !Connected(thing))) {
+      /* Easy, they're not on it or a disconnected player */
+      safe_str("Off", buff, bp);
+      return;
+    }
+    /* They're on the channel, but maybe we can't see them? */
+    if (Chanuser_Hide(u) &&
+	!(Priv_Who(executor) || Can_Examine(executor, thing))) {
+      safe_str("Off", buff, bp);
+      return;
+    }
+    /* We can see them, so we report if they're On or Gag */
+    safe_str(Chanuser_Gag(u) ? "Gag" : "On", buff, bp);
+    return;
+  }
+}
+
 FUNCTION(fun_cowner)
 {
   /* Return the dbref of the owner of a channel. */
@@ -2330,7 +2419,7 @@ do_chan_chown(dbref player, const char *name, const char *newowner)
   /* Find the channel */
   test_channel(player, name, c);
   /* Find the victim */
-  if ((victim = lookup_player(newowner)) == NOTHING) {
+  if (!newowner || ((victim = lookup_player(newowner)) == NOTHING)) {
     notify(player, T("CHAT: Invalid owner."));
     return;
   }
@@ -2939,6 +3028,119 @@ FUNCTION(fun_cemit)
   do_cemit(executor, args[0], args[1], flags);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_crecall)
+{
+  CHAN *chan;
+  CHANUSER *u;
+  int start = -1, num_lines = 10;
+  char *p = NULL, *buf, *name;
+  time_t timestamp;
+  char *stamp;
+  dbref speaker;
+  int type;
+  int first = 1;
+  char sep;
+  int showstamp = 0;
+
+  name = args[0];
+  if (!name || !*name) {
+    safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+    return;
+  }
+
+  if (!args[1] || !*args[1]) {
+    /* nothing */
+  } else if (is_integer(args[1])) {
+    num_lines = parse_integer(args[1]);
+    if (num_lines == 0)
+      num_lines = INT_MAX;
+  } else {
+    safe_str(T(e_int), buff, bp);
+    return;
+  }
+  if (!args[2] || !*args[2]) {
+    /* nothing */
+  } else if (is_integer(args[2])) {
+    start = parse_integer(args[2]) - 1;
+  } else {
+    safe_str(T(e_int), buff, bp);
+    return;
+  }
+
+  if (!delim_check(buff, bp, nargs, args, 4, &sep))
+    return;
+
+  if (nargs > 4 && args[4] && *args[4])
+    showstamp = parse_boolean(args[4]);
+
+  if (num_lines < 0) {
+    safe_str(T(e_uint), buff, bp);
+    return;
+  }
+
+  test_channel(executor, name, chan);
+  if (!Chan_Can_See(chan, executor)) {
+    if (onchannel(executor, chan))
+      safe_str(T(e_perm), buff, bp);
+    else
+      safe_str(T("#-1 NO SUCH CHANNEL"), buff, bp);
+    return;
+  }
+
+  u = onchannel(executor, chan);
+  if (!u &&!Chan_Can_Access(chan, executor)) {
+    safe_str(T(e_perm), buff, bp);
+    return;
+  }
+
+  if (!ChanBufferQ(chan)) {
+    safe_str(T("#-1 NO RECALL BUFFER"), buff, bp);
+    return;
+  }
+
+  if (start < 0)
+    start = BufferQNum(ChanBufferQ(chan)) - num_lines;
+  if (isempty_bufferq(ChanBufferQ(chan))
+      || BufferQNum(ChanBufferQ(chan)) <= start) {
+    safe_str(T(e_range), buff, bp);
+    return;
+  }
+
+  while (start > 0) {
+    buf = iter_bufferq(ChanBufferQ(chan), &p, &speaker, &type, &timestamp);
+    start--;
+  }
+  while ((buf = iter_bufferq(ChanBufferQ(chan), &p, &speaker, &type,
+			     &timestamp)) && num_lines > 0) {
+    if (first)
+      first = 0;
+    else
+      safe_chr(sep, buff, bp);
+    if (Nospoof(executor) && GoodObject(speaker)) {
+      char *nsmsg = ns_esnotify(speaker, na_one, &executor,
+				Paranoid(executor) ? 1 : 0);
+      if (!showstamp)
+	safe_format(buff, bp, T("%s %s"), nsmsg, buf);
+      else {
+	stamp = show_time(timestamp, 0);
+	safe_format(buff, bp, T("[%s] %s %s"), stamp, nsmsg, buf);
+      }
+      mush_free(nsmsg, "string");
+    } else {
+      if (!showstamp)
+	safe_str(buf, buff, bp);
+      else {
+	stamp = show_time(timestamp, 0);
+	safe_format(buff, bp, T("[%s] %s"), stamp, buf);
+      }
+    }
+    num_lines--;
+  }
+
+
+}
+
 COMMAND (cmd_cemit) {
   int spflags = !strcmp(cmd->name, "@NSCEMIT") ? PEMIT_SPOOF : 0;
   SPOOF(player, cause, sw);
@@ -2949,49 +3151,49 @@ COMMAND (cmd_cemit) {
 
 COMMAND (cmd_channel) {
   if (switches)
-    do_channel(player, arg_left, arg_right, switches);
+    do_channel(player, arg_left, args_right[1], switches);
   else if (SW_ISSET(sw, SWITCH_LIST))
     do_channel_list(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_ADD))
-    do_chan_admin(player, arg_left, arg_right, 0);
+    do_chan_admin(player, arg_left, args_right[1], 0);
   else if (SW_ISSET(sw, SWITCH_DELETE))
-    do_chan_admin(player, arg_left, arg_right, 1);
+    do_chan_admin(player, arg_left, args_right[1], 1);
   else if (SW_ISSET(sw, SWITCH_NAME))
-    do_chan_admin(player, arg_left, arg_right, 2);
+    do_chan_admin(player, arg_left, args_right[1], 2);
   else if (SW_ISSET(sw, SWITCH_RENAME))
-    do_chan_admin(player, arg_left, arg_right, 2);
+    do_chan_admin(player, arg_left, args_right[1], 2);
   else if (SW_ISSET(sw, SWITCH_PRIVS))
-    do_chan_admin(player, arg_left, arg_right, 3);
+    do_chan_admin(player, arg_left, args_right[1], 3);
   else if (SW_ISSET(sw, SWITCH_RECALL))
-    do_chan_recall(player, arg_left, arg_right, SW_ISSET(sw, SWITCH_QUIET));
+    do_chan_recall(player, arg_left, args_right, SW_ISSET(sw, SWITCH_QUIET));
   else if (SW_ISSET(sw, SWITCH_DECOMPILE))
     do_chan_decompile(player, arg_left, SW_ISSET(sw, SWITCH_BRIEF));
   else if (SW_ISSET(sw, SWITCH_DESCRIBE))
-    do_chan_desc(player, arg_left, arg_right);
+    do_chan_desc(player, arg_left, args_right[1]);
   else if (SW_ISSET(sw, SWITCH_TITLE))
-    do_chan_title(player, arg_left, arg_right);
+    do_chan_title(player, arg_left, args_right[1]);
   else if (SW_ISSET(sw, SWITCH_CHOWN))
-    do_chan_chown(player, arg_left, arg_right);
+    do_chan_chown(player, arg_left, args_right[1]);
   else if (SW_ISSET(sw, SWITCH_WIPE))
     do_chan_wipe(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_MUTE))
-    do_chan_user_flags(player, arg_left, arg_right, 0, 0);
+    do_chan_user_flags(player, arg_left, args_right[1], 0, 0);
   else if (SW_ISSET(sw, SWITCH_UNMUTE))
     do_chan_user_flags(player, arg_left, "n", 0, 0);
   else if (SW_ISSET(sw, SWITCH_HIDE))
-    do_chan_user_flags(player, arg_left, arg_right, 1, 0);
+    do_chan_user_flags(player, arg_left, args_right[1], 1, 0);
   else if (SW_ISSET(sw, SWITCH_UNHIDE))
     do_chan_user_flags(player, arg_left, "n", 1, 0);
   else if (SW_ISSET(sw, SWITCH_GAG))
-    do_chan_user_flags(player, arg_left, arg_right, 2, 0);
+    do_chan_user_flags(player, arg_left, args_right[1], 2, 0);
   else if (SW_ISSET(sw, SWITCH_UNGAG))
     do_chan_user_flags(player, arg_left, "n", 2, 0);
   else if (SW_ISSET(sw, SWITCH_WHAT))
     do_chan_what(player, arg_left);
   else if (SW_ISSET(sw, SWITCH_BUFFER))
-    do_chan_buffer(player, arg_left, arg_right);
+    do_chan_buffer(player, arg_left, args_right[1]);
   else
-    do_channel(player, arg_left, NULL, arg_right);
+    do_channel(player, arg_left, NULL, args_right[1]);
 }
 
 COMMAND (cmd_chat) {
@@ -3059,6 +3261,7 @@ channel_broadcast(CHAN *channel, dbref player, int flags, const char *fmt, ...)
   char tbuf1[BUFFER_LEN * 2];	/* Safety margin as per tprintf */
 #endif
   struct na_cpass nac;
+  int na_flags = NA_INTER_LOCK;
 
   /* Make sure we can write to the channel before doing so */
   if (Channel_Disabled(channel))
@@ -3076,9 +3279,10 @@ channel_broadcast(CHAN *channel, dbref player, int flags, const char *fmt, ...)
 
   nac.u = ChanUsers(channel);
   nac.checkquiet = (flags & CB_CHECKQUIET) ? 1 : 0;
+  if (Channel_Interact(channel))
+    na_flags |= (flags & CB_PRESENCE) ? NA_INTER_PRESENCE : NA_INTER_HEAR;
   notify_anything(player, na_channel, &nac, ns_esnotify,
-		  ((flags & CB_PRESENCE) ? NA_INTER_PRESENCE : NA_INTER_HEAR) |
-		  ((flags & CB_NOSPOOF) ? 0 : NA_SPOOF), tbuf1);
+		  na_flags | ((flags & CB_NOSPOOF) ? 0 : NA_SPOOF), tbuf1);
   if (ChanBufferQ(channel))
     add_to_bufferq(ChanBufferQ(channel), 0,
 		   (flags & CB_NOSPOOF) ? player : NOTHING, tbuf1);
@@ -3093,26 +3297,37 @@ channel_broadcast(CHAN *channel, dbref player, int flags, const char *fmt, ...)
  * \endverbatim
  * \param player the enactor.
  * \param name the name of the channel.
- * \param lines a string given the number of lines to recall (default all).
+ * \param lineinfo point to array containing lines, optoinal start
  * \param quiet if true, don't show timestamps.
  */
 void
-do_chan_recall(dbref player, const char *name, const char *lines, int quiet)
+do_chan_recall(dbref player, const char *name, char *lineinfo[], int quiet)
 {
   CHAN *chan;
   CHANUSER *u;
+  const char *lines;
+  const char *startpos;
   int num_lines = 10;		/* Default if none is given */
+  int start = -1;
+  int all;
   char *p = NULL, *buf;
   time_t timestamp;
   char *stamp;
-  int skip;
   dbref speaker;
   int type;
-
   if (!name || !*name) {
     notify(player, T("You need to specify a channel."));
     return;
   }
+  lines = lineinfo[1];
+  startpos = lineinfo[2];
+  if (startpos && *startpos) {
+    if (!is_integer(startpos)) {
+      notify(player, T("Which line do you want to start recall from?"));
+      return;
+    }
+    start = parse_integer(startpos) - 1;
+  }
   if (lines && *lines) {
     if (is_integer(lines)) {
       num_lines = parse_integer(lines);
@@ -3147,44 +3362,44 @@ do_chan_recall(dbref player, const char *name, const char *lines, int quiet)
     notify(player, T("CHAT: That channel doesn't have a recall buffer."));
     return;
   }
-  if (isempty_bufferq(ChanBufferQ(chan))) {
+  if (start < 0)
+    start = BufferQNum(ChanBufferQ(chan)) - num_lines;
+  if (isempty_bufferq(ChanBufferQ(chan))
+      || (BufferQNum(ChanBufferQ(chan)) <= start)) {
     notify(player, T("CHAT: Nothing to recall."));
     return;
   }
 
-  skip = BufferQNum(ChanBufferQ(chan)) - num_lines;
+  all = (start <= 0 && num_lines >= BufferQNum(ChanBufferQ(chan)));
   notify_format(player, T("CHAT: Recall from channel <%s>"), ChanName(chan));
-
-  do {
+  while (start > 0) {
     buf = iter_bufferq(ChanBufferQ(chan), &p, &speaker, &type, &timestamp);
-    if (skip <= 0) {
-      if (buf) {
-	if (Nospoof(player) && GoodObject(speaker)) {
-	  char *nsmsg =
-	    ns_esnotify(speaker, na_one, &player, Paranoid(player) ? 1 : 0);
-	  if (quiet)
-	    notify_format(player, "%s %s", nsmsg, buf);
-	  else {
-	    stamp = show_time(timestamp, 0);
-	    notify_format(player, "[%s] %s %s", stamp, nsmsg, buf);
-	  }
-	  mush_free(nsmsg, "string");
-	} else {
-	  if (quiet)
-	    notify(player, buf);
-	  else {
-	    stamp = show_time(timestamp, 0);
-	    notify_format(player, "[%s] %s", stamp, buf);
-	  }
-	}
+    start--;
+  }
+  while ((buf = iter_bufferq(ChanBufferQ(chan), &p, &speaker, &type,
+			     &timestamp)) && num_lines > 0) {
+    if (Nospoof(player) && GoodObject(speaker)) {
+      char *nsmsg = ns_esnotify(speaker, na_one, &player,
+				Paranoid(player) ? 1 : 0);
+      if (quiet)
+	notify_format(player, T("%s %s"), nsmsg, buf);
+      else {
+	stamp = show_time(timestamp, 0);
+	notify_format(player, T("[%s] %s %s"), stamp, nsmsg, buf);
+      }
+      mush_free(nsmsg, "string");
+    } else {
+      if (quiet)
+	notify(player, buf);
+      else {
+	stamp = show_time(timestamp, 0);
+	notify_format(player, T("[%s] %s"), stamp, buf);
       }
     }
-    skip--;
-  } while (buf);
-
-
+    num_lines--;
+  }
   notify(player, T("CHAT: End recall"));
-  if (!lines || !*lines)
+  if (!all)
     notify_format(player,
 		  T
 		  ("CHAT: To recall the entire buffer, use @chan/recall %s=0"),
@@ -3204,7 +3419,6 @@ do_chan_buffer(dbref player, const char *name, const char *lines)
 {
   CHAN *chan;
   int size;
-
   if (!name || !*name) {
     notify(player, T("You need to specify a channel."));
     return;
@@ -3229,7 +3443,8 @@ do_chan_buffer(dbref player, const char *name, const char *lines)
       free_bufferq(ChanBufferQ(chan));
       ChanBufferQ(chan) = NULL;
       notify_format(player,
-		    T("CHAT: Channel buffering disabled for channel <%s>."),
+		    T
+		    ("CHAT: Channel buffering disabled for channel <%s>."),
 		    ChanName(chan));
     } else {
       notify_format(player,
@@ -3241,8 +3456,8 @@ do_chan_buffer(dbref player, const char *name, const char *lines)
     if (ChanBufferQ(chan)) {
       /* Resize a buffer */
       ChanBufferQ(chan) = reallocate_bufferq(ChanBufferQ(chan), size);
-      notify_format(player, T("CHAT: Resizing buffer of channel <%s>"),
-		    ChanName(chan));
+      notify_format(player,
+		    T("CHAT: Resizing buffer of channel <%s>"), ChanName(chan));
     } else {
       /* Start a new buffer */
       ChanBufferQ(chan) = allocate_bufferq(size);
@@ -3254,7 +3469,8 @@ do_chan_buffer(dbref player, const char *name, const char *lines)
 }
 
 static void
-format_channel_broadcast(CHAN *chan, CHANUSER *u, dbref victim, int flags,
+format_channel_broadcast(CHAN *chan, CHANUSER *u,
+			 dbref victim, int flags,
 			 const char *msg, const char *extra)
 {
   const char *title = NULL;
@@ -3262,7 +3478,6 @@ format_channel_broadcast(CHAN *chan, CHANUSER *u, dbref victim, int flags,
     title = extra;
   else if (u &&CUtitle(u))
      title = CUtitle(u);
-
   if (Channel_NoNames(chan)) {
     if (Channel_NoTitles(chan) || !title)
       channel_broadcast(chan, victim, flags, msg, ChanName(chan), "Someone");
@@ -3285,10 +3500,8 @@ eval_chan_lock(CHAN *c, dbref p, enum clock_type type)
   char *oldenv[10];
   boolexp b = TRUE_BOOLEXP;
   int retval, n;
-
   if (!c || !GoodObject(p))
     return 0;
-
   switch (type) {
   case CLOCK_SEE:
     b = ChanSeeLock(c);
@@ -3312,7 +3525,5 @@ eval_chan_lock(CHAN *c, dbref p, enum clock_type type)
     global_eval_context.wenv[n] = NULL;
   retval = eval_boolexp(p, b, p);
   restore_global_env("eval_chan_lock", oldenv);
-
   return retval;
-
 }
