@@ -101,6 +101,9 @@ static MAIL *real_mail_fetch(dbref player, int num, int folder);
 static MAIL *mailfun_fetch(dbref player, int nargs, char *arg1, char *arg2);
 static void count_mail(dbref player,
 		       int folder, int *rcount, int *ucount, int *ccount);
+static int real_send_mail(dbref player,
+			  dbref target, char *subject, char *message,
+			  mail_flag flags, int silent, int nosig);
 static void send_mail(dbref player,
 		      dbref target, char *subject, char *message,
 		      mail_flag flags, int silent, int nosig);
@@ -108,6 +111,8 @@ static int send_mail_alias(dbref player,
 			   char *aname, char *subject,
 			   char *message, mail_flag flags, int silent,
 			   int nosig);
+static void filter_mail(dbref from, dbref player, char *subject,
+			char *message, int mailnumber, mail_flag flags);
 static MAIL *find_insertion_point(dbref player);
 static int get_folder_number(dbref player, char *name);
 static char *get_folder_name(dbref player, int fld);
@@ -122,7 +127,7 @@ static char *status_chars(MAIL *mp);
 static char *status_string(MAIL *mp);
 static int sign(int x);
 static char *get_message(MAIL *mp);
-static char *get_compressed_message(MAIL *mp);
+static unsigned char *get_compressed_message(MAIL *mp);
 static char *get_subject(MAIL *mp);
 static char *get_sender(MAIL *mp, int full);
 static int was_sender(dbref player, MAIL *mp);
@@ -161,7 +166,7 @@ static char *
 get_message(MAIL *mp)
 {
   static char text[BUFFER_LEN * 2];
-  char tbuf[BUFFER_LEN * 2];
+  unsigned char tbuf[BUFFER_LEN * 2];
 
   if (!mp)
     return NULL;
@@ -172,15 +177,15 @@ get_message(MAIL *mp)
 }
 
 /* Return the compressed text of a @mail in a static buffer */
-static char *
+static unsigned char *
 get_compressed_message(MAIL *mp)
 {
-  static char text[BUFFER_LEN * 2];
+  static unsigned char text[BUFFER_LEN * 2];
 
   if (!mp)
     return NULL;
 
-  chunk_fetch(mp->msgid, text, sizeof text);
+  chunk_fetch(mp->msgid, (unsigned char *) text, sizeof text);
   return text;
 }
 
@@ -768,11 +773,12 @@ do_mail_fwd(dbref player, char *msglist, char *tolist)
 	    if (!temp) {
 	      notify(player, T("MAIL: You can't reply to nonexistant mail."));
 	    } else {
-	      char tbuf1[BUFFER_LEN], tbuf2[BUFFER_LEN];
+	      char tbuf1[BUFFER_LEN];
+	      unsigned char tbuf2[BUFFER_LEN];
 	      strcpy(tbuf1, uncompress(mp->subject));
-	      strcpy(tbuf2, get_compressed_message(mp));
-	      send_mail(player, temp->from, tbuf1, tbuf2, M_FORWARD | M_REPLY,
-			1, 0);
+	      u_strcpy(tbuf2, get_compressed_message(mp));
+	      send_mail(player, temp->from, tbuf1, (char *) tbuf2,
+			M_FORWARD | M_REPLY, 1, 0);
 	      num_recpts++;
 	    }
 	  } else {
@@ -787,10 +793,11 @@ do_mail_fwd(dbref player, char *msglist, char *tolist)
 	    if (!GoodObject(target) || !IsPlayer(target)) {
 	      notify_format(player, T("No such unique player: %s."), current);
 	    } else {
-	      char tbuf1[BUFFER_LEN], tbuf2[BUFFER_LEN];
+	      char tbuf1[BUFFER_LEN];
+	      unsigned char tbuf2[BUFFER_LEN];
 	      strcpy(tbuf1, uncompress(mp->subject));
-	      strcpy(tbuf2, get_compressed_message(mp));
-	      send_mail(player, target, tbuf1, tbuf2, M_FORWARD, 1, 0);
+	      u_strcpy(tbuf2, get_compressed_message(mp));
+	      send_mail(player, target, tbuf1, (char *) tbuf2, M_FORWARD, 1, 0);
 	      num_recpts++;
 	    }
 	  }
@@ -964,7 +971,56 @@ static void
 send_mail(dbref player, dbref target, char *subject, char *message,
 	  mail_flag flags, int silent, int nosig)
 {
-  /* send a mail message */
+  /* send a message to a target, consulting the target's mailforward.
+   * If mailforward isn't set, just deliver to targt.
+   * If mailforward is set, deliver to the list if we're allowed
+   *  (but don't check mailforward further!)
+   */
+  ATTR *a;
+  int good = 0;
+
+  a = atr_get_noparent(target, "MAILFORWARDLIST");
+  if (!a) {
+    /* Easy, no forwarding */
+    good =
+      real_send_mail(player, target, subject, message, flags, silent, nosig);
+    return;
+  } else {
+    /* We have a forward list. Run through it. */
+    char *fwdstr, *orig, *curr;
+    dbref fwd;
+    orig = safe_atr_value(a);
+    fwdstr = trim_space_sep(orig, ' ');
+    while ((curr = split_token(&fwdstr, ' ')) != NULL) {
+      if (is_objid(curr)) {
+	fwd = parse_objid(curr);
+	if (GoodObject(fwd) && Can_MailForward(target, fwd)) {
+	  good +=
+	    real_send_mail(player, fwd, subject, message, flags, 1, nosig);
+	} else
+	  notify_format(target, T("Failed attempt to forward @mail to #%d"),
+			fwd);
+      }
+    }
+    free((Malloc_t) orig);
+  }
+  if (!silent) {
+    if (good)
+      notify_format(player,
+		    T("MAIL: You sent your message to %s."), Name(target));
+    else
+      notify_format(player,
+		    T
+		    ("MAIL: Your message was not sent to %s due to a mail forwarding problem."),
+		    Name(target));
+  }
+}
+
+static int
+real_send_mail(dbref player, dbref target, char *subject, char *message,
+	       mail_flag flags, int silent, int nosig)
+{
+  /* deliver a mail message to a target, period */
 
   MAIL *newp, *mp;
   int rc, uc, cc;
@@ -976,25 +1032,28 @@ send_mail(dbref player, dbref target, char *subject, char *message,
   char *cp;
 
   if (!IsPlayer(target)) {
-    notify(player, T("MAIL: You cannot send mail to non-existent people."));
-    return;
+    if (!silent)
+      notify(player, T("MAIL: You cannot send mail to non-existent people."));
+    return 0;
   }
   if (!strcasecmp(message, "clear")) {
     notify(player,
 	   T("MAIL: You probably don't wanna send mail saying 'clear'."));
-    return;
+    return 0;
   }
   if (!(Hasprivs(player) || eval_lock(player, target, Mail_Lock))) {
-    notify_format(player,
-		  T("MAIL: %s is not accepting mail from you right now."),
-		  Name(target));
-    return;
+    if (!silent)
+      notify_format(player,
+		    T("MAIL: %s is not accepting mail from you right now."),
+		    Name(target));
+    return 0;
   }
   count_mail(target, 0, &rc, &uc, &cc);
   if ((rc + uc + cc) >= MAIL_LIMIT) {
-    notify_format(player, T("MAIL: %s's mailbox is full. Can't send."),
-		  Name(target));
-    return;
+    if (!silent)
+      notify_format(player, T("MAIL: %s's mailbox is full. Can't send."),
+		    Name(target));
+    return 0;
   }
 
   /* initialize the appropriate fields */
@@ -1021,7 +1080,7 @@ send_mail(dbref player, dbref target, char *subject, char *message,
   if (flags & M_FORWARD) {
     /* Forwarding passes the message already compressed */
     u_int_16 len = strlen(message) + 1;
-    newp->msgid = chunk_create(message, len, 1);
+    newp->msgid = chunk_create((unsigned char *) message, len, 1);
   } else {
     u_int_16 len;
     unsigned char *text;
@@ -1046,7 +1105,7 @@ send_mail(dbref player, dbref target, char *subject, char *message,
     }
     *nm = '\0';
     text = compress(newmsg);
-    len = strlen(text) + 1;
+    len = u_strlen(text) + 1;
     newp->msgid = chunk_create(text, len, 1);
     free(text);
     mush_free((Malloc_t) newmsg, "string");
@@ -1101,11 +1160,14 @@ send_mail(dbref player, dbref target, char *subject, char *message,
 		T("MAIL: You have a new message (%d) from %s."),
 		rc + uc + cc + 1, Name(player));
 
+  /* Check @mailfilter */
+  filter_mail(player, target, subject, message, rc + uc + cc + 1, flags);
+
   if (AMAIL_ATTR && (atr_get_noparent(target, "AMAIL"))
       && (player != target) && Hasprivs(target))
     did_it(player, target, NULL, NULL, NULL, NULL, "AMAIL", NOTHING);
 
-  return;
+  return 1;
 }
 
 
@@ -1404,6 +1466,7 @@ void
 do_mail(dbref player, char *arg1, char *arg2)
 {
   dbref sender;
+  char *p;
   /* Force player to be a real player, but keep track of the
    * enactor in case we're sending mail, which objects can do
    */
@@ -1440,10 +1503,11 @@ do_mail(dbref player, char *arg1, char *arg2)
       do_mail_send(sender, arg1, arg2, 0, 0, 0);
   } else {
     /* Must be reading or listing mail - no arg2 */
-    if (isdigit((unsigned char) *arg1) && !strchr(arg1, '-'))
-      do_mail_read(player, arg1);
-    else
+    if (((p = strchr(arg1, ':')) && (*(p + 1) == '\0'))
+	|| !(isdigit((unsigned char) *arg1) && !strchr(arg1, '-')))
       do_mail_list(player, arg1);
+    else
+      do_mail_read(player, arg1);
   }
   return;
 }
@@ -2020,7 +2084,7 @@ load_mail(FILE * fp)
     tbuf = compress(getstring_noalloc(fp));
   }
   text = compress(getstring_noalloc(fp));
-  len = strlen(text) + 1;
+  len = u_strlen(text) + 1;
   mp->msgid = chunk_create(text, len, 1);
   free(text);
   if (mail_flags & MDBF_SUBJECT)
@@ -2054,7 +2118,7 @@ load_mail(FILE * fp)
     else
       tbuf = NULL;
     text = compress(getstring_noalloc(fp));
-    len = strlen(text) + 1;
+    len = u_strlen(text) + 1;
     mp->msgid = chunk_create(text, len, 1);
     free(text);
     if (tbuf)
@@ -2497,6 +2561,42 @@ status_string(MAIL *mp)
   return tbuf1;
 }
 
+/** Scan all mail folders for unread mail.
+ * \param player player to check for new mail (and to notify).
+ */
+void
+check_all_mail(dbref player)
+{
+  int folder;			/* which folder */
+  int rc;			/* read messages */
+  int uc;			/* unread messages */
+  int cc;			/* cleared messages */
+  int subtotal;			/* total messages per iteration */
+  int total = 0;		/* total messages */
+
+  /* search through all the folders. */
+
+  for (folder = 0; folder <= MAX_FOLDERS; folder++) {
+    count_mail(player, folder, &rc, &uc, &cc);
+    subtotal = rc + uc + cc;
+    if (subtotal > 0) {
+      notify_format(player,
+		    T
+		    ("MAIL: %d messages in folder %d [%s] (%d unread, %d cleared)."),
+		    subtotal, folder, get_folder_name(player, folder), uc, cc);
+      total += subtotal;
+      if (folder == 0 && (subtotal + 5) > MAIL_LIMIT)
+	notify_format(player,
+		      T("MAIL: Warning! Limit on inbox messages is %d!"),
+		      MAIL_LIMIT);
+    }
+  }
+
+  if (!total)
+    notify(player, T("\nMAIL: You have no mail.\n"));
+  return;
+}
+
 /** Check for new mail.
  * \param player player to check for new mail (and to notify).
  * \param folder folder number to check.
@@ -2581,13 +2681,20 @@ parse_message_spec(dbref player, const char *s, int *msglow, int *msghigh,
       }
     } else {
       /* f:m */
-      if (!is_integer(p))
-	return 0;
-      *msglow = parse_integer(p);
-      if (*msglow == 0)
-	*msglow = -1;
-      if (msghigh)
-	*msghigh = *msglow;
+      if (!*p) {
+	/* f: */
+	*msglow = 0;
+	if (msghigh)
+	  *msghigh = HUGE_INT;
+      } else {
+	if (!is_integer(p))
+	  return 0;
+	*msglow = parse_integer(p);
+	if (*msglow == 0)
+	  *msglow = -1;
+	if (msghigh)
+	  *msghigh = *msglow;
+      }
     }
     if (*msglow < 0 || (msghigh && *msghigh < 0) || *folder < 0
 	|| *folder > MAX_FOLDERS)
@@ -2666,3 +2773,79 @@ send_mail_alias(dbref player, char *aname, char *subject, char *message,
   }
   return 1;			/* Success */
 }
+
+
+/** Event for @mailfilter.
+ * \param from the @mailing player.
+ * \param player the player to act on.
+ * \param subject the subject of the @mail.
+ * \param message the body of the @mail.
+ * \param mailnumber the number of the @mail to file.
+ * \param flags the flags of the @mail.
+ */
+void
+filter_mail(dbref from, dbref player, char *subject,
+	    char *message, int mailnumber, mail_flag flags)
+{
+  ATTR *f;
+  char buff[BUFFER_LEN], *bp, *asave;
+  char buf[FOLDER_NAME_LEN + 1];
+  char *wsave[10], *rsave[NUMQ];
+  char *arg, *arg2, *arg3, *arg4;
+  int j;
+  char const *ap;
+  static char tbuf1[6];
+
+  /* Does the player have a @mailfilter? */
+  f = atr_get(player, "MAILFILTER");
+  if (!f)
+    return;
+
+  /* Handle this now so it doesn't clutter code */
+  tbuf1[0] = '\0';
+  if (flags & M_URGENT)
+    strcat(tbuf1, "U");
+  if (flags & M_FORWARD)
+    strcat(tbuf1, "F");
+  if (flags & M_REPLY)
+    strcat(tbuf1, "R");
+
+  arg = (char *) mush_malloc(BUFFER_LEN, "string");
+  arg2 = (char *) mush_malloc(BUFFER_LEN, "string");
+  arg3 = (char *) mush_malloc(BUFFER_LEN, "string");
+  arg4 = (char *) mush_malloc(BUFFER_LEN, "string");
+  if (!arg4)
+    mush_panic("Unable to allocate memory in mailfilter");
+  save_global_regs("filter_mail", rsave);
+  save_global_env("filter_mail", wsave);
+  for (j = 0; j < 10; j++)
+    global_eval_context.wenv[j] = NULL;
+  for (j = 0; j < NUMQ; j++)
+    global_eval_context.renv[j][0] = '\0';
+  strcpy(arg, unparse_dbref(from));
+  global_eval_context.wenv[0] = arg;
+  strcpy(arg2, subject);
+  global_eval_context.wenv[1] = arg2;
+  strcpy(arg3, message);
+  global_eval_context.wenv[2] = arg3;
+  strcpy(arg4, tbuf1);
+  global_eval_context.wenv[3] = arg4;
+
+  ap = asave = safe_atr_value(f);
+  bp = buff;
+  process_expression(buff, &bp, &ap, player, player, player,
+		     PE_DEFAULT, PT_DEFAULT, NULL);
+  *bp = '\0';
+  free((Malloc_t) asave);
+  if (*buff) {
+    sprintf(buf, "0:%d", mailnumber);
+    do_mail_file(player, buf, buff);
+  }
+
+  mush_free((Malloc_t) arg, "string");
+  mush_free((Malloc_t) arg2, "string");
+  mush_free((Malloc_t) arg3, "string");
+  mush_free((Malloc_t) arg4, "string");
+  restore_global_env("filter_mail", wsave);
+  restore_global_regs("filter_mail", rsave);
+}
