@@ -111,6 +111,7 @@ static FLAG flag_table[] = {
   {"NOSPOOF", '"', TYPE_PLAYER, PLAYER_NOSPOOF, F_ANY | F_ODARK,
    F_ANY | F_ODARK},
   {"SHARED", 'Z', TYPE_PLAYER, PLAYER_ZONE, F_ANY, F_ANY},
+  {"TRACK_MONEY", '\0', TYPE_PLAYER, 0, F_ANY, F_ANY},
   {"CONNECTED", 'c', TYPE_PLAYER, PLAYER_CONNECT, F_INTERNAL | F_MDARK,
    F_INTERNAL | F_MDARK},
   {"GAGGED", 'g', TYPE_PLAYER, PLAYER_GAGGED, F_WIZARD, F_WIZARD},
@@ -808,6 +809,8 @@ flag_add_additional(FLAGSPACE * n)
 	     F_INHERIT);
     add_flag("ORPHAN", 'i', NOTYPE, F_ANY, F_ANY);
     add_flag("HEAVY", '\0', NOTYPE, F_ROYAL, F_ANY);
+    add_flag("TRACK_MONEY", '\0', TYPE_PLAYER, F_ANY, F_ANY);
+    add_flag("LOUD", '\0', NOTYPE, F_ROYAL, F_ANY);
     if ((f = match_flag("TERSE")))
       f->type |= TYPE_THING;
     if ((f = match_flag("PUPPET")))
@@ -843,8 +846,24 @@ flag_add_additional(FLAGSPACE * n)
       for (i = 0; i < n->flagbits; i++)
 	n->flags[i]->perms |= F_LOG;
     }
-    add_power("Sql_Ok", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    flags = hashfind("POWER", &htab_flagspaces);
+    f = add_power("Sql_Ok", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    if (!match_power("Use_SQL"))
+      flag_add(flags, "Use_SQL", f);
     add_power("Debit", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    add_power("Pueblo_Send", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    add_power("Many_Attribs", '\0', NOTYPE, F_WIZARD | F_LOG, F_ANY);
+    /* Aliases for other servers */
+    if ((f = match_power("tport_anything")) && !match_power("tel_anything"))
+      flag_add(flags, "tel_anything", f);
+    if ((f = match_power("tport_anywhere")) && !match_power("tel_anywhere"))
+      flag_add(flags, "tel_anywhere", f);
+    if ((f = match_power("no_money")) && !match_power("free_money"))
+      flag_add(flags, "free_money", f);
+    if ((f = match_power("no_quota")) && !match_power("free_quota"))
+      flag_add(flags, "free_quota", f);
+    if ((f = match_power("debit")) && !match_power("steal_money"))
+      flag_add(flags, "steal_money", f);
   }
 
   local_flags(n);
@@ -1886,8 +1905,10 @@ add_flag_generic(const char *ns, const char *name, const char letter, int type,
   FLAGSPACE *n;
   Flagspace_Lookup(n, ns);
   /* Don't double-add */
-  if ((f = match_flag_ns(n, strupper(name))))
-    return f;
+  if ((f = match_flag_ns(n, strupper(name)))) {
+    if (strcasecmp(f->name, name) == 0)
+      return f;
+  }
   f = new_flag();
   f->name = mush_strdup(strupper(name), "flag name");
   f->letter = letter;
@@ -2203,8 +2224,9 @@ do_flag_add(const char *ns, dbref player, const char *name, char *args_right[])
 void
 do_flag_alias(const char *ns, dbref player, const char *name, const char *alias)
 {
-  FLAG *f;
+  FLAG *f, *af;
   FLAGSPACE *n;
+  int delete = 0;
   if (!God(player)) {
     notify(player, T("You don't look like God."));
     return;
@@ -2213,7 +2235,11 @@ do_flag_alias(const char *ns, dbref player, const char *name, const char *alias)
     notify(player, T("You must provide a name for the alias."));
     return;
   }
-  if (strlen(alias) == 1) {
+  if (*alias == '!') {
+    delete = 1;
+    alias++;
+  }
+  if (strlen(alias) <= 1) {
     notify_format(player, T("%s aliases must be longer than one character."),
 		  strinitial(ns));
     return;
@@ -2224,10 +2250,10 @@ do_flag_alias(const char *ns, dbref player, const char *name, const char *alias)
     return;
   }
   n = hashfind(ns, &htab_flagspaces);
-  f = match_flag_ns(n, alias);
-  if (f) {
+  af = match_flag_ns(n, alias);
+  if (!delete && af) {
     notify_format(player, T("That alias already matches the %s %s."),
-		  f->name, strlower(ns));
+		  af->name, strlower(ns));
     return;
   }
   f = match_flag_ns(n, name);
@@ -2239,16 +2265,63 @@ do_flag_alias(const char *ns, dbref player, const char *name, const char *alias)
     notify_format(player, T("That %s is disabled."), strlower(ns));
     return;
   }
-  /* Insert the flag in the ptab by the given alias */
-  ptab_start_inserts(n->tab);
-  ptab_insert(n->tab, alias, f);
-  ptab_end_inserts(n->tab);
-  if ((f = match_flag_ns(n, alias)))
-    do_flag_info(ns, player, alias);
-  else
-    notify(player, T("Unknown failure adding alias."));
+  if (delete && !af) {
+    notify_format(player, T("That isn't an alias of the %s %s."),
+		  f->name, strlower(ns));
+    return;
+  }
+  if (delete) {
+    /* Delete the alias in the ptab if it's really an alias! */
+    if (!strcasecmp(n->flags[f->bitpos]->name, alias)) {
+      notify_format(player, T("That's the %s's name, not an alias."),
+		    strlower(ns));
+      return;
+    }
+    ptab_delete(n->tab, alias);
+    if ((af = match_flag_ns(n, alias)))
+      notify(player, T("Unknown failure deleting alias."));
+    else
+      do_flag_info(ns, player, f->name);
+  } else {
+    /* Insert the flag in the ptab by the given alias */
+    if (alias_flag_generic(ns, name, alias))
+      do_flag_info(ns, player, alias);
+    else
+      notify(player, T("Unknown failure adding alias."));
+  }
 }
 
+ /** Add a new alias for a flag.
+  * \param ns name of the flagspace to use.
+  * \param name name of the flag
+  * \param alias new alias for the flag
+  * \retval 1 alias added successfully
+  * \retval 0 failed to add alias
+  */
+int
+alias_flag_generic(const char *ns, const char *name, const char *alias)
+{
+  FLAG *f;
+  FLAGSPACE *n;
+
+  Flagspace_Lookup(n, ns);
+
+  f = match_flag_ns(n, name);
+  if (!f) {
+    return 0;			/* no such flag 'name' */
+  }
+
+  if (ptab_find_exact(n->tab, strupper(alias))) {
+    return 0;			/* a flag called 'alias' already exists */
+  }
+
+  ptab_start_inserts(n->tab);
+  ptab_insert(n->tab, strupper(alias), f);
+  ptab_end_inserts(n->tab);
+  return ((f = match_flag_ns(n, alias)) ? 1 : 0);
+}
+
+
 /** Change a flag's letter. 
  * \param ns name of the flagspace to use.
  * \param player the enactor.
@@ -2484,7 +2557,7 @@ list_all_flags(const char *ns, const char *name, dbref privs, int which)
 	ptrs[numptrs++] = (char *) f->name;
     }
   }
-  do_gensort(privs, ptrs, numptrs, ALPHANUM_LIST);
+  do_gensort(privs, ptrs, NULL, numptrs, ALPHANUM_LIST);
   bp = buf;
   for (i = 0; i < numptrs; i++) {
     switch (which) {
