@@ -48,7 +48,6 @@ void
 save_global_regs(const char *funcname, char *preserve[])
 {
   int i;
-
   for (i = 0; i < NUMQ; i++) {
     if (!global_eval_context.renv[i][0])
       preserve[i] = NULL;
@@ -71,12 +70,39 @@ restore_global_regs(const char *funcname, char *preserve[])
     if (preserve[i]) {
       strcpy(global_eval_context.renv[i], preserve[i]);
       mush_free(preserve[i], funcname);
+      preserve[i] = NULL;
     } else {
       global_eval_context.renv[i][0] = '\0';
     }
   }
 }
 
+/** Free the storage array for the q-registers, without restoring
+ * \param funcname name of function calling (for memory leak testing)
+ * \param preserve pointer to array to free q-registers from.
+ */
+void
+free_global_regs(const char *funcname, char *preserve[])
+{
+  int i;
+  for (i = 0; i < NUMQ; i++) {
+    if (preserve[i])
+      mush_free(preserve[i], funcname);
+  }
+}
+
+/** Initilalize an array for the q-registers, setting all NULL.
+ * \param preserve pointer to array to free q-registers from.
+ */
+void
+init_global_regs(char *preserve[])
+{
+  int i;
+  for (i = 0; i < NUMQ; i++) {
+    preserve[i] = NULL;
+  }
+}
+
 /** Restore the q-registers, without freeing the storage array.
  * \param preserve pointer to array to restore the q-registers from.
  */
@@ -119,6 +145,73 @@ restore_global_env(const char *funcname __attribute__ ((__unused__)),
     global_eval_context.wenv[i] = preserve[i];
 }
 
+/** Save a copy of the wnxt and rnxt state
+ * This function must deal with both the addresses and the values
+ * of these variables, because they get modified in all sorts of
+ * nasty ways that we may not account for.
+ * \param funcname name of function calling (for memory leak testing)
+ * \param preservew pointer to array to store the wnxt address in.
+ * \param preserver pointer to array to store the rnxt address in.
+ * \param valw pointer to array to store the wnxt value in.
+ * \param valr pointer to array to store the rnxt value in.
+ */
+void
+save_global_nxt(const char *funcname, char *preservew[], char *preserver[],
+		char *valw[], char *valr[])
+{
+  int i;
+  for (i = 0; i < NUMQ; i++) {
+    preserver[i] = global_eval_context.rnxt[i];
+    if (!global_eval_context.rnxt[i])
+      valr[i] = NULL;
+    else {
+      valr[i] = (char *) mush_malloc(BUFFER_LEN, funcname);
+      strcpy(valr[i], global_eval_context.rnxt[i]);
+    }
+  }
+  for (i = 0; i < 10; i++) {
+    preservew[i] = global_eval_context.wnxt[i];
+    if (!global_eval_context.wnxt[i])
+      valw[i] = NULL;
+    else {
+      valw[i] = (char *) mush_malloc(BUFFER_LEN, funcname);
+      strcpy(valw[i], global_eval_context.wnxt[i]);
+    }
+  }
+}
+
+/** Restore a copy of the wnxt and rnxt state
+ * \param funcname name of function calling (for memory leak testing)
+ * \param preservew pointer to array to restore the wnxt address from.
+ * \param preserver pointer to array to restore the rnxt address from.
+ * \param valw pointer to array to restore the wnxt value from.
+ * \param valr pointer to array to restore the rnxt value from.
+ */
+void
+restore_global_nxt(const char *funcname, char *preservew[], char *preserver[],
+		   char *valw[], char *valr[])
+{
+  int i;
+  for (i = 0; i < NUMQ; i++) {
+    global_eval_context.rnxt[i] = preserver[i];
+    if (preserver[i]) {
+      /* There was a former address, so we can restore to it */
+      strcpy(global_eval_context.rnxt[i], valr[i]);
+      mush_free(valr[i], funcname);
+      valr[i] = NULL;
+    }
+  }
+  for (i = 0; i < 10; i++) {
+    global_eval_context.wnxt[i] = preservew[i];
+    if (preservew[i]) {
+      /* There was a former address, so we can restore to it */
+      strcpy(global_eval_context.wnxt[i], valw[i]);
+      mush_free(valw[i], funcname);
+      valw[i] = NULL;
+    }
+  }
+}
+
 /** Check for a delimiter in an argument of a function call.
  * This function checks a given argument of a function call and sees
  * if it could be used as a delimiter. A delimiter must be a single
@@ -179,6 +272,7 @@ FUNTAB flist[] = {
   {"ACCNAME", fun_accname, 1, 1, FN_REG},
   {"ADD", fun_add, 2, INT_MAX, FN_REG},
   {"AFTER", fun_after, 2, 2, FN_REG},
+  {"ALIAS", fun_alias, 1, 2, FN_REG},
   {"ALIGN", fun_align, 2, INT_MAX, FN_REG},
   {"ALLOF", fun_allof, 2, INT_MAX, FN_NOPARSE},
   {"ALPHAMAX", fun_alphamax, 1, INT_MAX, FN_REG},
@@ -192,6 +286,7 @@ FUNTAB flist[] = {
   {"APOSS", fun_aposs, 1, 1, FN_REG},
   {"ART", fun_art, 1, 1, FN_REG},
   {"ATRLOCK", fun_atrlock, 1, 2, FN_REG},
+  {"ATTRIB_SET", fun_attrib_set, 1, -2, FN_REG},
   {"BAND", fun_band, 1, INT_MAX, FN_REG},
   {"BASECONV", fun_baseconv, 3, 3, FN_REG},
   {"BEEP", fun_beep, 0, 1, FN_REG},
@@ -207,14 +302,21 @@ FUNTAB flist[] = {
   {"CASE", fun_switch, 3, INT_MAX, FN_NOPARSE},
   {"CASEALL", fun_switch, 3, INT_MAX, FN_NOPARSE},
   {"CAT", fun_cat, 1, INT_MAX, FN_REG},
+  {"CBUFFER", fun_cinfo, 1, 1, FN_REG},
+  {"CDESC", fun_cinfo, 1, 1, FN_REG},
   {"CEMIT", fun_cemit, 2, 3, FN_REG},
   {"CFLAGS", fun_cflags, 1, 2, FN_REG},
   {"CHANNELS", fun_channels, 0, 2, FN_REG},
+  {"CLFLAGS", fun_cflags, 1, 2, FN_REG},
   {"CLOCK", fun_clock, 1, 2, FN_REG},
+  {"CMSGS", fun_cinfo, 1, 1, FN_REG},
   {"COWNER", fun_cowner, 1, 1, FN_REG},
+  {"CRECALL", fun_crecall, 1, 5, FN_REG},
+  {"CSTATUS", fun_cstatus, 2, 2, FN_REG},
   {"CTITLE", fun_ctitle, 2, 2, FN_REG},
+  {"CUSERS", fun_cinfo, 1, 1, FN_REG},
   {"CWHO", fun_cwho, 1, 1, FN_REG},
-  {"CENTER", fun_center, 2, 3, FN_REG},
+  {"CENTER", fun_center, 2, 4, FN_REG},
   {"CHILDREN", fun_lsearch, 1, 1, FN_REG},
   {"CHR", fun_chr, 1, 1, FN_REG},
   {"CHECKPASS", fun_checkpass, 2, 2, FN_REG | FN_WIZARD},
@@ -232,8 +334,9 @@ FUNTAB flist[] = {
   {"CREATE", fun_create, 1, 2, FN_REG},
   {"CTIME", fun_ctime, 1, 1, FN_REG},
   {"DEC", fun_dec, 1, 1, FN_REG},
+  {"DECOMPOSE", fun_decompose, 1, -1, FN_REG},
   {"DECRYPT", fun_decrypt, 2, 2, FN_REG},
-  {"DEFAULT", fun_default, 2, 2, FN_NOPARSE},
+  {"DEFAULT", fun_default, 2, INT_MAX, FN_NOPARSE},
   {"DELETE", fun_delete, 3, 3, FN_REG},
   {"DIE", fun_die, 2, 3, FN_REG},
   {"DIG", fun_dig, 1, 3, FN_REG},
@@ -256,14 +359,14 @@ FUNTAB flist[] = {
   {"EVAL", fun_eval, 2, 2, FN_REG},
   {"ESCAPE", fun_escape, 1, -1, FN_REG},
   {"EXIT", fun_exit, 1, 1, FN_REG},
-  {"EXTRACT", fun_extract, 3, 4, FN_REG},
+  {"EXTRACT", fun_extract, 1, 4, FN_REG},
   {"FILTER", fun_filter, 2, 4, FN_REG},
   {"FILTERBOOL", fun_filter, 2, 4, FN_REG},
   {"FINDABLE", fun_findable, 2, 2, FN_REG},
   {"FIRST", fun_first, 1, 2, FN_REG},
   {"FIRSTOF", fun_firstof, 0, INT_MAX, FN_NOPARSE},
   {"FLAGS", fun_flags, 0, 1, FN_REG},
-  {"FLIP", fun_flip, 1, 1, FN_REG},
+  {"FLIP", fun_flip, 1, -1, FN_REG},
   {"FLOORDIV", fun_floordiv, 2, 2, FN_REG},
   {"FOLD", fun_fold, 2, 4, FN_REG},
   {"FOLDERSTATS", fun_folderstats, 0, 2, FN_REG},
@@ -272,6 +375,7 @@ FUNTAB flist[] = {
   {"FOREACH", fun_foreach, 2, 4, FN_REG},
   {"FRACTION", fun_fraction, 1, 1, FN_REG},
   {"FUNCTIONS", fun_functions, 0, 0, FN_REG},
+  {"FULLALIAS", fun_fullalias, 1, 1, FN_REG},
   {"FULLNAME", fun_fullname, 1, 1, FN_REG},
   {"GET", fun_get, 1, 1, FN_REG},
   {"GET_EVAL", fun_get_eval, 1, 1, FN_REG},
@@ -288,7 +392,7 @@ FUNTAB flist[] = {
   {"HASFLAG", fun_hasflag, 2, 2, FN_REG},
   {"HASPOWER", fun_haspower, 2, 2, FN_REG},
   {"HASTYPE", fun_hastype, 2, 2, FN_REG},
-  {"HEIGHT", fun_height, 1, 1, FN_REG},
+  {"HEIGHT", fun_height, 1, 2, FN_REG},
   {"HIDDEN", fun_hidden, 1, 1, FN_REG},
   {"HOME", fun_home, 1, 1, FN_REG},
   {"HOST", fun_hostname, 1, 1, FN_REG},
@@ -341,8 +445,8 @@ FUNTAB flist[] = {
   {"LPLAYERS", fun_dbwalker, 1, 1, FN_REG},
   {"LPORTS", fun_lports, 0, 0, FN_REG},
   {"LPOS", fun_lpos, 2, 2, FN_REG},
-  {"LSEARCH", fun_lsearch, 1, 5, FN_REG},
-  {"LSEARCHR", fun_lsearch, 1, 5, FN_REG},
+  {"LSEARCH", fun_lsearch, 1, INT_MAX, FN_REG},
+  {"LSEARCHR", fun_lsearch, 1, INT_MAX, FN_REG},
   {"LSET", fun_lset, 2, 2, FN_REG},
   {"LSTATS", fun_lstats, 0, 1, FN_REG},
   {"LT", fun_lt, 2, 2, FN_REG},
@@ -353,6 +457,7 @@ FUNTAB flist[] = {
   {"LVPLAYERS", fun_dbwalker, 1, 1, FN_REG},
   {"LVTHINGS", fun_dbwalker, 1, 1, FN_REG},
   {"LWHO", fun_lwho, 0, 1, FN_REG},
+  {"LWHOID", fun_lwho, 0, 1, FN_REG},
   {"MAIL", fun_mail, 0, 2, FN_REG},
   {"MAILFROM", fun_mailfrom, 1, 2, FN_REG},
   {"MAILSEND", fun_mailsend, 2, 2, FN_REG},
@@ -364,6 +469,7 @@ FUNTAB flist[] = {
   {"MAILTIME", fun_mailtime, 1, 2, FN_REG},
   {"MALIAS", fun_malias, 0, 2, FN_REG},
   {"MAP", fun_map, 2, 4, FN_REG},
+  {"MAPSQL", fun_mapsql, 2, 4, FN_REG},
   {"MATCH", fun_match, 2, 3, FN_REG},
   {"MATCHALL", fun_matchall, 2, 4, FN_REG},
   {"MAX", fun_max, 1, INT_MAX, FN_REG},
@@ -381,28 +487,38 @@ FUNTAB flist[] = {
   {"MUL", fun_mul, 2, INT_MAX, FN_REG},
   {"MUNGE", fun_munge, 3, 5, FN_REG},
   {"MWHO", fun_lwho, 0, 0, FN_REG},
+  {"MWHOID", fun_lwho, 0, 0, FN_REG},
   {"NAME", fun_name, 0, 2, FN_REG},
+  {"NAMELIST", fun_namelist, 1, 1, FN_REG},
+  {"NAMEGRAB", fun_namegrab, 2, 3, FN_REG},
+  {"NAMEGRABALL", fun_namegraball, 2, 3, FN_REG},
   {"NAND", fun_nand, 1, INT_MAX, FN_REG},
   {"NATTR", fun_nattr, 1, 1, FN_REG},
   {"NATTRP", fun_nattr, 1, 1, FN_REG},
+  {"NCHILDREN", fun_lsearch, 1, 1, FN_REG},
   {"NCON", fun_dbwalker, 1, 1, FN_REG},
   {"NEXITS", fun_dbwalker, 1, 1, FN_REG},
   {"NPLAYERS", fun_dbwalker, 1, 1, FN_REG},
   {"NEARBY", fun_nearby, 2, 2, FN_REG},
   {"NEQ", fun_neq, 2, 2, FN_REG},
   {"NEXT", fun_next, 1, 1, FN_REG},
+  {"NEXTDBREF", fun_nextdbref, 0, 0, FN_REG},
+  {"NLSEARCH", fun_lsearch, 1, INT_MAX, FN_REG},
   {"NMWHO", fun_nwho, 0, 0, FN_REG},
   {"NOR", fun_nor, 1, INT_MAX, FN_REG},
   {"NOT", fun_not, 1, 1, FN_REG},
   {"NSCEMIT", fun_cemit, 2, 3, FN_REG},
+  {"NSEARCH", fun_lsearch, 1, INT_MAX, FN_REG},
   {"NSEMIT", fun_emit, 1, -1, FN_REG},
   {"NSLEMIT", fun_lemit, 1, -1, FN_REG},
   {"NSOEMIT", fun_oemit, 2, -2, FN_REG},
   {"NSPEMIT", fun_pemit, 2, -2, FN_REG},
+  {"NSPROMPT", fun_prompt, 2, -2, FN_REG},
   {"NSREMIT", fun_remit, 2, -2, FN_REG},
   {"NSZEMIT", fun_zemit, 2, -2, FN_REG},
   {"NTHINGS", fun_dbwalker, 1, 1, FN_REG},
   {"NUM", fun_num, 1, 1, FN_REG},
+  {"NUMVERSION", fun_numversion, 0, 0, FN_REG},
   {"NULL", fun_null, 1, INT_MAX, FN_REG},
   {"NVCON", fun_dbwalker, 1, 1, FN_REG},
   {"NVEXITS", fun_dbwalker, 1, 1, FN_REG},
@@ -417,13 +533,14 @@ FUNTAB flist[] = {
   {"OPEN", fun_open, 2, 2, FN_REG},
   {"OR", fun_or, 2, INT_MAX, FN_REG},
   {"ORD", fun_ord, 1, 1, FN_REG},
+  {"ORDINAL", fun_spellnum, 1, 1, FN_REG},
   {"ORFLAGS", fun_orflags, 2, 2, FN_REG},
   {"ORLFLAGS", fun_orlflags, 2, 2, FN_REG},
   {"ORLPOWERS", fun_orlflags, 2, 2, FN_REG},
   {"ORPOWERS", fun_orflags, 2, 2, FN_REG},
   {"OWNER", fun_owner, 1, 1, FN_REG},
   {"PARENT", fun_parent, 1, 2, FN_REG},
-  {"PCREATE", fun_pcreate, 1, 2, FN_REG},
+  {"PCREATE", fun_pcreate, 2, 2, FN_REG},
   {"PEMIT", fun_pemit, 2, -2, FN_REG},
   {"PLAYERMEM", fun_playermem, 1, 1, FN_REG},
   {"PMATCH", fun_pmatch, 1, 1, FN_REG},
@@ -432,6 +549,7 @@ FUNTAB flist[] = {
   {"POS", fun_pos, 2, 2, FN_REG},
   {"POSS", fun_poss, 1, 1, FN_REG},
   {"POWERS", fun_powers, 1, 2, FN_REG},
+  {"PROMPT", fun_prompt, 2, -2, FN_REG},
   {"PUEBLO", fun_pueblo, 1, 1, FN_REG},
   {"QUOTA", fun_quota, 1, 1, FN_REG},
   {"R", fun_r, 1, 1, FN_REG},
@@ -462,7 +580,7 @@ FUNTAB flist[] = {
   {"REST", fun_rest, 1, 2, FN_REG},
   {"RESTARTS", fun_restarts, 0, 0, FN_REG},
   {"RESTARTTIME", fun_restarttime, 0, 0, FN_REG},
-  {"REVERSE", fun_flip, 1, 1, FN_REG},
+  {"REVERSE", fun_flip, 1, -1, FN_REG},
   {"REVWORDS", fun_revwords, 1, 3, FN_REG},
   {"RIGHT", fun_right, 2, 2, FN_REG},
   {"RJUST", fun_rjust, 2, 3, FN_REG},
@@ -477,8 +595,8 @@ FUNTAB flist[] = {
   {"SECURE", fun_secure, 1, -1, FN_REG},
   {"SENT", fun_sent, 1, 1, FN_REG},
   {"SET", fun_set, 2, 2, FN_REG},
-  {"SETQ", fun_setq, 2, 2, FN_REG},
-  {"SETR", fun_setq, 2, 2, FN_REG},
+  {"SETQ", fun_setq, 2, INT_MAX, FN_REG},
+  {"SETR", fun_setq, 2, INT_MAX, FN_REG},
   {"SETDIFF", fun_setdiff, 2, 5, FN_REG},
   {"SETINTER", fun_setinter, 2, 5, FN_REG},
   {"SETUNION", fun_setunion, 2, 5, FN_REG},
@@ -489,9 +607,11 @@ FUNTAB flist[] = {
   {"SIGN", fun_sign, 1, 1, FN_REG},
   {"SORT", fun_sort, 1, 4, FN_REG},
   {"SORTBY", fun_sortby, 2, 4, FN_REG},
+  {"SORTKEY", fun_sortkey, 2, 5, FN_REG},
   {"SOUNDEX", fun_soundex, 1, 1, FN_REG},
   {"SOUNDSLIKE", fun_soundlike, 2, 2, FN_REG},
   {"SPACE", fun_space, 1, 1, FN_REG},
+  {"SPEAK", fun_speak, 2, 7, FN_REG},
   {"SPELLNUM", fun_spellnum, 1, 1, FN_REG},
   {"SPLICE", fun_splice, 3, 4, FN_REG},
   {"SQL", fun_sql, 1, 3, FN_REG},
@@ -501,6 +621,7 @@ FUNTAB flist[] = {
   {"STARTTIME", fun_starttime, 0, 0, FN_REG},
   {"STEP", fun_step, 3, 5, FN_REG},
   {"STRCAT", fun_strcat, 1, INT_MAX, FN_REG},
+  {"STRINGSECS", fun_stringsecs, 1, 1, FN_REG},
   {"STRINSERT", fun_strinsert, 3, -3, FN_REG},
   {"STRIPACCENTS", fun_stripaccents, 1, 1, FN_REG},
   {"STRIPANSI", fun_stripansi, 1, -1, FN_REG},
@@ -515,6 +636,7 @@ FUNTAB flist[] = {
   {"TABLE", fun_table, 1, 5, FN_REG},
   {"TEL", fun_tel, 2, 4, FN_REG},
   {"TERMINFO", fun_terminfo, 1, 1, FN_REG},
+  {"TEXTENTRIES", fun_textentries, 2, 3, FN_REG},
   {"TEXTFILE", fun_textfile, 2, 2, FN_REG},
   {"TIME", fun_time, 0, 1, FN_REG},
   {"TIMEFMT", fun_timefmt, 1, 2, FN_REG},
@@ -528,8 +650,10 @@ FUNTAB flist[] = {
   {"UCSTR", fun_ucstr, 1, -1, FN_REG},
   {"UDEFAULT", fun_uldefault, 2, 12, FN_NOPARSE},
   {"UFUN", fun_ufun, 1, 11, FN_REG},
+  {"ULAMBDA", fun_ulambda, 1, 11, FN_REG},
   {"ULDEFAULT", fun_uldefault, 1, 12, FN_NOPARSE},
   {"ULOCAL", fun_ulocal, 1, 11, FN_REG},
+  {"UNIQUE", fun_unique, 1, 4, FN_REG},
   {"UTCTIME", fun_time, 0, 0, FN_REG},
   {"U", fun_ufun, 1, 11, FN_REG},
   {"V", fun_v, 1, 1, FN_REG},
@@ -537,7 +661,9 @@ FUNTAB flist[] = {
   {"VERSION", fun_version, 0, 0, FN_REG},
   {"VISIBLE", fun_visible, 2, 2, FN_REG},
   {"WHERE", fun_where, 1, 1, FN_REG},
-  {"WIDTH", fun_width, 1, 1, FN_REG},
+  {"WIDTH", fun_width, 1, 2, FN_REG},
+  {"WILDGREP", fun_grep, 3, 3, FN_REG},
+  {"WILDGREPI", fun_grep, 3, 3, FN_REG},
   {"WIPE", fun_wipe, 1, 1, FN_REG},
   {"WORDPOS", fun_wordpos, 2, 3, FN_REG},
   {"WORDS", fun_words, 1, 2, FN_REG},
@@ -547,6 +673,7 @@ FUNTAB flist[] = {
   {"XCON", fun_dbwalker, 3, 3, FN_REG},
   {"XEXITS", fun_dbwalker, 3, 3, FN_REG},
   {"XMWHO", fun_xwho, 2, 2, FN_REG},
+  {"XMWHOID", fun_xwho, 2, 2, FN_REG},
   {"XPLAYERS", fun_dbwalker, 3, 3, FN_REG},
   {"XGET", fun_xget, 2, 2, FN_REG},
   {"XOR", fun_xor, 2, INT_MAX, FN_REG},
@@ -556,6 +683,7 @@ FUNTAB flist[] = {
   {"XVPLAYERS", fun_dbwalker, 3, 3, FN_REG},
   {"XVTHINGS", fun_dbwalker, 3, 3, FN_REG},
   {"XWHO", fun_xwho, 2, 2, FN_REG},
+  {"XWHOID", fun_xwho, 2, 2, FN_REG},
   {"ZEMIT", fun_zemit, 2, -2, FN_REG},
   {"ZFUN", fun_zfun, 1, 11, FN_REG},
   {"ZONE", fun_zone, 1, 2, FN_REG},
@@ -639,7 +767,7 @@ list_functions(void)
   }
   /* do_gensort needs a dbref now, but only for sort types that aren't
    * used here anyway */
-  do_gensort((dbref) 0, (char **) ptrs, nptrs, ALPHANUM_LIST);
+  do_gensort((dbref) 0, (char **) ptrs, NULL, nptrs, ALPHANUM_LIST);
   bp = buff;
   safe_str(ptrs[0], buff, &bp);
   for (i = 1; i < nptrs; i++) {
@@ -863,6 +991,8 @@ apply_restrictions(unsigned int result, const char *restriction)
       flag = FN_LOGNAME;
     } else if (!strcasecmp(restriction, "noparse")) {
       flag = FN_NOPARSE;
+    } else if (!strcasecmp(restriction, "localize")) {
+      flag = FN_LOCALIZE;
     }
     if (clear)
       result &= ~flag;
@@ -888,6 +1018,7 @@ apply_restrictions(unsigned int result, const char *restriction)
  *   god        can only be used by god
  *   noplayer   can't be used by players, just objects/rooms/exits
  *   nosidefx   can't be used to do side-effect thingies
+ *   localize   localize q-registers
  * \endverbatim
  * \param name name of function to restrict.
  * \param restriction name of restriction to apply to function.
@@ -919,11 +1050,14 @@ restrict_function(const char *name, const char *restriction)
 void
 do_function_restrict(dbref player, const char *name, const char *restriction)
 {
+  FUN *fp;
+  unsigned int flags;
+
   if (!Wizard(player)) {
     notify(player, T("Permission denied."));
     return;
   }
-  if (!name) {
+  if (!name || !*name) {
     notify(player, T("Restrict what function?"));
     return;
   }
@@ -931,10 +1065,17 @@ do_function_restrict(dbref player, const char *name, const char *restriction)
     notify(player, T("Do what with the function?"));
     return;
   }
-  if (restrict_function(name, restriction))
-    notify(player, T("Restrictions modified."));
+  fp = func_hash_lookup(name);
+  if (!fp) {
+    notify(player, T("No such function."));
+    return;
+  }
+  flags = fp->flags;
+  fp->flags = apply_restrictions(flags, restriction);
+  if (fp->flags == flags)
+    notify(player, T("Restrictions unchanged."));
   else
-    notify(player, T("Restrict attempt failed."));
+    notify(player, T("Restrictions modified."));
 }
 
 
@@ -947,9 +1088,11 @@ do_function_restrict(dbref player, const char *name, const char *restriction)
  * \param player the enactor.
  * \param name name of function to add.
  * \param argv array of arguments.
+ * \param preserve Treat the function like it was called with ulocal() instead
+ *  of u().
  */
 void
-do_function(dbref player, char *name, char *argv[])
+do_function(dbref player, char *name, char *argv[], int preserve)
 {
   char tbuf1[BUFFER_LEN];
   char *bp = tbuf1;
@@ -1016,7 +1159,12 @@ do_function(dbref player, char *name, char *argv[])
   if ((thing = noisy_match_result(player, argv[1], NOTYPE, MAT_EVERYTHING))
       == NOTHING)
     return;
-  if (!Can_Examine(player, thing)) {
+  if (SAFER_UFUN) {
+    if (!controls(player, thing)) {
+      notify(player, T("No permission to control object."));
+      return;
+    }
+  } else if (!Can_Examine(player, thing)) {
     notify(player, T("No permission to examine object."));
     return;
   }
@@ -1063,6 +1211,8 @@ do_function(dbref player, char *name, char *argv[])
       fp->flags = apply_restrictions(0, argv[5]);
     else
       fp->flags = 0;
+    if (preserve)
+      fp->flags |= FN_LOCALIZE;
     hashadd(name, fp, &htab_user_function);
 
     /* now add it to the user function table */
@@ -1116,6 +1266,15 @@ do_function(dbref player, char *name, char *argv[])
 	fp->maxargs = 10;
     } else
       fp->maxargs = 10;
+
+    /* Set new flags */
+    if (argv[5] && *argv[5])
+      fp->flags = apply_restrictions(0, argv[5]);
+    else
+      fp->flags = 0;
+    if (preserve)
+      fp->flags |= FN_LOCALIZE;
+
     notify(player, T("Function updated."));
   }
 }
@@ -1331,6 +1490,13 @@ do_function_report(dbref player, char *name)
       first = 0;
   }
 
+  if (fp->flags & FN_LOCALIZE) {
+    if (first == 0)
+      safe_strl(", ", 2, tbuf, &tp);
+    safe_str("Localize", tbuf, &tp);
+    first = 0;
+  }
+
   if (fp->flags & FN_LITERAL) {
     if (first == 0)
       safe_strl(", ", 2, tbuf, &tp);
