@@ -31,7 +31,7 @@
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
 
-extern PRIV attr_privs[];
+extern PRIV attr_privs_view[];
 static lock_type get_locktype(char *str);
 extern struct db_stat_info *get_stats(dbref owner);
 static int lattr_helper(dbref player, dbref thing, dbref parent,
@@ -257,22 +257,26 @@ FUNCTION(fun_default)
   char *dp;
   const char *sp;
   char mstr[BUFFER_LEN];
+  int i;
   /* find our object and attribute */
-  dp = mstr;
-  sp = args[0];
-  process_expression(mstr, &dp, &sp, executor, caller, enactor,
-		     PE_DEFAULT, PT_DEFAULT, pe_info);
-  *dp = '\0';
-  parse_attrib(executor, mstr, &thing, &attrib);
-  if (GoodObject(thing) && attrib && Can_Read_Attr(executor, thing, attrib)) {
-    /* Ok, we've got it */
-    dp = safe_atr_value(attrib);
-    safe_str(dp, buff, bp);
-    free(dp);
-    return;
+  for (i = 1; i < nargs; i++) {
+    mstr[0] = '\0';
+    dp = mstr;
+    sp = args[i - 1];
+    process_expression(mstr, &dp, &sp, executor, caller, enactor, PE_DEFAULT,
+		       PT_DEFAULT, pe_info);
+    *dp = '\0';
+    parse_attrib(executor, mstr, &thing, &attrib);
+    if (GoodObject(thing) && attrib && Can_Read_Attr(executor, thing, attrib)) {
+      /* Ok, we've got it */
+      dp = safe_atr_value(attrib);
+      safe_str(dp, buff, bp);
+      free(dp);
+      return;
+    }
   }
-  /* We couldn't get it. Evaluate args[1] and return it */
-  sp = args[1];
+  /* We couldn't get it. Evaluate the last arg and return it */
+  sp = args[nargs - 1];
   process_expression(buff, bp, &sp, executor, caller, enactor,
 		     PE_DEFAULT, PT_DEFAULT, pe_info);
   return;
@@ -468,7 +472,7 @@ FUNCTION(fun_flags)
       safe_str("#-1", buff, bp);
       return;
     }
-    safe_str(privs_to_letters(attr_privs, AL_FLAGS(a)), buff, bp);
+    safe_str(privs_to_letters(attr_privs_view, AL_FLAGS(a)), buff, bp);
     if (atr_sub_branch(a))
       safe_chr('`', buff, bp);
   } else {
@@ -501,7 +505,7 @@ FUNCTION(fun_lflags)
       safe_str("#-1", buff, bp);
       return;
     }
-    safe_str(privs_to_string(attr_privs, AL_FLAGS(a)), buff, bp);
+    safe_str(privs_to_string(attr_privs_view, AL_FLAGS(a)), buff, bp);
   } else {
     /* Object flags, visible to all */
     safe_str(bits_to_string("FLAG", Flags(thing), executor, thing), buff, bp);
@@ -655,7 +659,7 @@ dbwalk(char *buff, char **bp, dbref executor, dbref enactor,
        */
       if (!(Typeof(thing) & type) ||
 	  !can_interact(thing, executor, INTERACT_SEE) ||
-	  (skipdark && Dark(thing)) ||
+	  (skipdark && Dark(thing) && !Light(thing) && !Light(loc)) ||
 	  ((type == TYPE_PLAYER) && skipdark && !Connected(thing)))
 	continue;
       nthing += 1;
@@ -1049,36 +1053,52 @@ FUNCTION(fun_hasflag)
 /* ARGSUSED */
 FUNCTION(fun_hastype)
 {
+  char *r, *s;
+  int found = 0;
   dbref it = match_thing(executor, args[0]);
   if (!GoodObject(it)) {
     safe_str(T(e_notvis), buff, bp);
     return;
   }
-  switch (*args[1]) {
-  case 'r':
-  case 'R':
-    safe_boolean(IsRoom(it), buff, bp);
-    break;
-  case 'e':
-  case 'E':
-    safe_boolean(IsExit(it), buff, bp);
-    break;
-  case 'p':
-  case 'P':
-    safe_boolean(IsPlayer(it), buff, bp);
-    break;
-  case 't':
-  case 'T':
-    safe_boolean(IsThing(it), buff, bp);
-    break;
-  case 'g':
-  case 'G':
-    safe_boolean(IsGarbage(it), buff, bp);
-    break;
-  default:
-    safe_str(T("#-1 NO SUCH TYPE"), buff, bp);
-    break;
-  };
+  s = trim_space_sep(args[1], ' ');
+  r = split_token(&s, ' ');
+  do {
+    switch (*r) {
+    case 'r':
+    case 'R':
+      if (IsRoom(it))
+	found = 1;
+      break;
+    case 'e':
+    case 'E':
+      if (IsExit(it))
+	found = 1;
+      break;
+    case 'p':
+    case 'P':
+      if (IsPlayer(it))
+	found = 1;
+      break;
+    case 't':
+    case 'T':
+      if (IsThing(it))
+	found = 1;
+      break;
+    case 'g':
+    case 'G':
+      if (IsGarbage(it))
+	found = 1;
+      break;
+    default:
+      safe_str(T("#-1 NO SUCH TYPE"), buff, bp);
+      return;
+    }
+    if (found) {
+      safe_boolean(found, buff, bp);
+      return;
+    }
+  } while ((r = split_token(&s, ' ')));
+  safe_boolean(0, buff, bp);
 }
 
 /* ARGSUSED */
@@ -1555,6 +1575,42 @@ FUNCTION(fun_owner)
   }
 }
 
+/* ARGSUSED */
+FUNCTION(fun_alias)
+{
+  dbref it;
+
+  it = match_thing(executor, args[0]);
+  if (!GoodObject(it))
+    safe_str(T(e_notvis), buff, bp);
+
+  /* Support changing alias via function if side-effects are enabled */
+  if (nargs == 2) {
+    if (!command_check_byname(executor, "ATTRIB_SET")
+	|| fun->flags & FN_NOSIDEFX) {
+      safe_str(T(e_perm), buff, bp);
+      return;
+    }
+    if (!FUNCTION_SIDE_EFFECTS)
+      safe_str(T(e_disabled), buff, bp);
+    else
+      do_set_atr(it, "ALIAS", args[1], executor, 0);
+    return;
+  } else {
+    safe_str(shortalias(it), buff, bp);
+  }
+}
+
+/* ARGSUSED */
+FUNCTION(fun_fullalias)
+{
+  dbref it = match_thing(executor, args[0]);
+  if (GoodObject(it))
+    safe_str(fullalias(it), buff, bp);
+  else
+    safe_str(T(e_notvis), buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_name)
 {
@@ -1614,7 +1670,8 @@ FUNCTION(fun_iname)
       safe_str(T(e_perm), buff, bp);
       return;
     }
-    if (nameformat(executor, it, tbuf1))
+    if (nameformat(executor, it, tbuf1,
+		   IsExit(it) ? shortname(it) : (char *) accented_name(it)))
       safe_str(tbuf1, buff, bp);
     else if (IsExit(it))
       safe_str(shortname(it), buff, bp);
@@ -1663,6 +1720,36 @@ FUNCTION(fun_pmatch)
   }
 }
 
+/* ARGUSED */
+FUNCTION(fun_namelist)
+{
+
+  int first = 1;
+  char *current;
+  dbref target;
+  const char *start;
+
+  start = args[0];
+  while (start && *start) {
+    if (!first)
+      safe_str(" ", buff, bp);
+    first = 0;
+    current = next_in_list(&start);
+    if (*current == '*')
+      current = current + 1;
+    target = lookup_player(current);
+    if (!GoodObject(target))
+      target = visible_short_page(executor, current);
+    if (target == NOTHING) {
+      safe_str("#-1", buff, bp);
+    } else if (target == AMBIGUOUS) {
+      safe_str("#-2", buff, bp);
+    } else {
+      safe_dbref(target, buff, bp);
+    }
+  }
+}
+
 /* ARGSUSED */
 FUNCTION(fun_locate)
 {
@@ -1687,23 +1774,23 @@ FUNCTION(fun_locate)
   }
 
   /* find out our preferred match type and flags */
-  pref_type = NOTYPE;
+  pref_type = 0;
   for (p = args[2]; *p; p++) {
     switch (*p) {
     case 'N':
-      pref_type = NOTYPE;
+      pref_type |= NOTYPE;
       break;
     case 'E':
-      pref_type = TYPE_EXIT;
+      pref_type |= TYPE_EXIT;
       break;
     case 'P':
-      pref_type = TYPE_PLAYER;
+      pref_type |= TYPE_PLAYER;
       break;
     case 'R':
-      pref_type = TYPE_ROOM;
+      pref_type |= TYPE_ROOM;
       break;
     case 'T':
-      pref_type = TYPE_THING;
+      pref_type |= TYPE_THING;
       break;
     case 'L':
       keys = 1;
@@ -1755,6 +1842,8 @@ FUNCTION(fun_locate)
       break;
     }
   }
+  if (!pref_type)
+    pref_type = NOTYPE;
 
   if (keys)
     match_flags = MAT_CHECK_KEYS;
@@ -1770,7 +1859,7 @@ FUNCTION(fun_locate)
     return;
   }
 
-  if (force_type && pref_type != NOTYPE && !(Typeof(item) == pref_type)) {
+  if (force_type && !(Typeof(item) & pref_type)) {
     safe_dbref(NOTHING, buff, bp);
     return;
   }
@@ -1783,13 +1872,13 @@ FUNCTION(fun_locate)
     if (Can_Examine(executor, loc))
       safe_dbref(item, buff, bp);
     else if (can_interact(item, executor, INTERACT_SEE)
-	     && (!DarkLegal(item) || (Dark(loc) && Light(item))))
+	     && (!DarkLegal(item) || Light(loc) || Light(item)))
       safe_dbref(item, buff, bp);
     else
       safe_dbref(NOTHING, buff, bp);
   } else {
     if (can_interact(item, executor, INTERACT_SEE)
-	&& (See_All(executor) || !DarkLegal(item)))
+	&& (See_All(executor) || !DarkLegal(item) || Light(item)))
       safe_dbref(item, buff, bp);
     else
       safe_dbref(NOTHING, buff, bp);
@@ -1926,6 +2015,40 @@ FUNCTION(fun_wipe)
   do_wipe(executor, args[0]);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_attrib_set)
+{
+  dbref thing;
+  char *s;
+
+  if (!FUNCTION_SIDE_EFFECTS) {
+    safe_str(T(e_disabled), buff, bp);
+    return;
+  }
+  if (!command_check_byname(executor, "ATTRIB_SET") || fun->flags & FN_NOSIDEFX) {
+    safe_str(T(e_perm), buff, bp);
+    return;
+  }
+  s = strchr(args[0], '/');
+  if (!s) {
+    safe_str(T("#-1 BAD ARGUMENT FORMAT TO ATTRIB_SET"), buff, bp);
+    return;
+  }
+  *s++ = '\0';
+  thing = match_thing(executor, args[0]);
+  if (!GoodObject(thing)) {
+    safe_str(T(e_notvis), buff, bp);
+    return;
+  }
+  if (nargs == 1) {
+    do_set_atr(thing, s, NULL, executor, 1);
+  } else if (strlen(args[1]) == 0 && !EMPTY_ATTRS) {
+    do_set_atr(thing, s, " ", executor, 1);
+  } else {
+    do_set_atr(thing, s, args[1], executor, 1);
+  }
+}
+
 
 /* --------------------------------------------------------------------------
  * Misc functions: TEL
@@ -1962,6 +2085,8 @@ FUNCTION(fun_isdbref)
 FUNCTION(fun_grep)
 {
   char *tp;
+  int wild;
+  int sensitive;
 
   dbref it = match_thing(executor, args[0]);
   if (!GoodObject(it)) {
@@ -1977,8 +2102,10 @@ FUNCTION(fun_grep)
     safe_str(T("#-1 INVALID GREP PATTERN"), buff, bp);
     return;
   }
-  tp = grep_util(executor, it, args[1], args[2], arglens[2],
-		 strcmp(called_as, "GREP"));
+
+  sensitive = !strcmp(called_as, "GREP") || !strcmp(called_as, "WILDGREP");
+  wild = !strcmp(called_as, "WILDGREPI") || !strcmp(called_as, "WILDGREP");
+  tp = grep_util(executor, it, args[1], args[2], sensitive, wild);
   add_check("fun_grep.attr_list");
   safe_str(tp, buff, bp);
   mush_free((Malloc_t) tp, "fun_grep.attr_list");
@@ -2113,3 +2240,13 @@ FUNCTION(fun_following)
   safe_str(res, buff, bp);
   return;;
 }
+
+/* ARGSUSED */
+FUNCTION(fun_nextdbref)
+{
+  if (first_free != NOTHING) {
+    safe_dbref(first_free, buff, bp);
+  } else {
+    safe_dbref(db_top, buff, bp);
+  }
+}
