@@ -42,8 +42,9 @@ static int regrep_helper(dbref who, dbref what, dbref parent,
 /** Type definition for a qsort comparison function */
 typedef int (*comp_func) (const void *, const void *);
 static void sane_qsort(void **array, int left, int right, comp_func compare);
+enum itemfun_op { IF_DELETE, IF_REPLACE, IF_INSERT };
 static void do_itemfuns(char *buff, char **bp, char *str, char *num,
-			char *word, char *sep, int flag);
+			char *word, char *sep, enum itemfun_op flag);
 
 char *iter_rep[MAX_ITERS];  /**< itext values */
 int iter_place[MAX_ITERS];  /**< inum numbers */
@@ -374,89 +375,48 @@ FUNCTION(fun_fold)
    * can provide a starting point.
    */
 
-  dbref thing;
-  ATTR *attrib;
-  char const *ap;
-  char *abuf, *result, *rp, *rsave;
+  ufun_attrib ufun;
   char *cp;
-  char *tptr[2];
+  char *wenv[2];
   char sep;
   int funccount, per;
-  int pe_flags = PE_DEFAULT;
+  char base[BUFFER_LEN];
+  char result[BUFFER_LEN];
 
   if (!delim_check(buff, bp, nargs, args, 4, &sep))
     return;
 
-  /* find our object and attribute */
-  parse_anon_attrib(executor, args[0], &thing, &attrib);
-
-  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
+  if (!fetch_ufun_attrib(args[0], executor, &ufun, 1))
     return;
-  }
-  if (!CanEvalAttr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
-    return;
-  }
-
-  /* Now we can go to work */
-  if (AF_Debug(attrib))
-    pe_flags |= PE_DEBUG;
-  result = (char *) mush_malloc(BUFFER_LEN, "string");
-  rsave = (char *) mush_malloc(BUFFER_LEN, "string");
-  if (!result || !rsave)
-    mush_panic("Unable to allocate memory in fun_fold");
-
-  abuf = safe_atr_value(attrib);
-
-  /* save our stack */
-  tptr[0] = global_eval_context.wenv[0];
-  tptr[1] = global_eval_context.wenv[1];
 
   cp = args[1];
 
   /* If we have three or more arguments, the third one is the base case */
   if (nargs >= 3) {
-    global_eval_context.wenv[0] = args[2];
-    global_eval_context.wenv[1] = split_token(&cp, sep);
+    strncpy(base, args[2], BUFFER_LEN);
   } else {
-    global_eval_context.wenv[0] = split_token(&cp, sep);
-    global_eval_context.wenv[1] = split_token(&cp, sep);
+    strncpy(base, split_token(&cp, sep), BUFFER_LEN);
   }
-  rp = result;
-  ap = abuf;
-  process_expression(result, &rp, &ap, thing, executor, enactor,
-		     pe_flags, PT_DEFAULT, pe_info);
-  *rp = '\0';
-  strcpy(rsave, result);
+  wenv[0] = base;
+  wenv[1] = split_token(&cp, sep);
+
+  call_ufun(&ufun, wenv, 2, result, executor, enactor, pe_info);
+
+  strncpy(base, result, BUFFER_LEN);
+
   funccount = pe_info->fun_invocations;
 
   /* handle the rest of the cases */
   while (cp && *cp) {
-    global_eval_context.wenv[0] = rsave;
-    global_eval_context.wenv[1] = split_token(&cp, sep);
-    rp = result;
-    ap = abuf;
-    per = process_expression(result, &rp, &ap, thing, executor, enactor,
-			     pe_flags, PT_DEFAULT, pe_info);
-    *rp = '\0';
+    wenv[1] = split_token(&cp, sep);
+    per = call_ufun(&ufun, wenv, 2, result, executor, enactor, pe_info);
     if (per || (pe_info->fun_invocations >= FUNCTION_LIMIT &&
-		pe_info->fun_invocations == funccount &&
-		!strcmp(rsave, result)))
+		pe_info->fun_invocations == funccount && !strcmp(base, result)))
       break;
     funccount = pe_info->fun_invocations;
-    strcpy(rsave, result);
+    strcpy(base, result);
   }
-  safe_str(rsave, buff, bp);
-
-  /* restore the stack */
-  global_eval_context.wenv[0] = tptr[0];
-  global_eval_context.wenv[1] = tptr[1];
-
-  free((Malloc_t) abuf);
-  mush_free((Malloc_t) result, "string");
-  mush_free((Malloc_t) rsave, "string");
-  free_anon_attrib(attrib);
+  safe_str(base, buff, bp);
 }
 
 /* ARGSUSED */
@@ -518,62 +478,37 @@ FUNCTION(fun_filter)
    * of the list for which the function evaluates to 1.
    */
 
-  dbref thing;
-  ATTR *attrib;
-  char const *ap;
-  char *abuf, result[BUFFER_LEN], *rp;
+  ufun_attrib ufun;
+  char result[BUFFER_LEN];
   char *cp;
-  char *tptr;
+  char *wenv[1];
   char sep;
   int first;
   int check_bool = 0;
   int funccount;
   char *osep, osepd[2] = { '\0', '\0' };
-  int pe_flags = PE_DEFAULT;
 
   if (!delim_check(buff, bp, nargs, args, 3, &sep))
     return;
 
-  if (nargs == 4)
-    osep = args[3];
-  else {
-    osepd[0] = sep;
-    osep = osepd;
-  }
+  osepd[0] = sep;
+  osep = (nargs >= 4) ? args[3] : osepd;
 
   if (strcmp(called_as, "FILTERBOOL") == 0)
     check_bool = 1;
 
   /* find our object and attribute */
-  parse_anon_attrib(executor, args[0], &thing, &attrib);
-
-  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
+  if (!fetch_ufun_attrib(args[0], executor, &ufun, 1))
     return;
-  }
-  if (!CanEvalAttr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
-    return;
-  }
-
-  if (AF_Debug(attrib))
-    pe_flags |= PE_DEBUG;
-
-  abuf = safe_atr_value(attrib);
-
-  tptr = global_eval_context.wenv[0];
 
+  /* Go through each argument */
   cp = trim_space_sep(args[1], sep);
   first = 1;
   funccount = pe_info->fun_invocations;
   while (cp && *cp) {
-    global_eval_context.wenv[0] = split_token(&cp, sep);
-    ap = abuf;
-    rp = result;
-    if (process_expression(result, &rp, &ap, thing, executor, enactor,
-			   pe_flags, PT_DEFAULT, pe_info))
+    wenv[0] = split_token(&cp, sep);
+    if (call_ufun(&ufun, wenv, 1, result, executor, enactor, pe_info))
       break;
-    *rp = '\0';
     if ((check_bool == 0)
 	? (*result == '1' && *(result + 1) == '\0')
 	: parse_boolean(result)) {
@@ -581,7 +516,7 @@ FUNCTION(fun_filter)
 	first = 0;
       else
 	safe_str(osep, buff, bp);
-      safe_str(global_eval_context.wenv[0], buff, bp);
+      safe_str(wenv[0], buff, bp);
     }
     /* Can't do *bp == oldbp like in all the others, because bp might not
      * move even when not full, if one of the list elements is null and
@@ -590,11 +525,6 @@ FUNCTION(fun_filter)
       break;
     funccount = pe_info->fun_invocations;
   }
-
-  global_eval_context.wenv[0] = tptr;
-
-  free((Malloc_t) abuf);
-  free_anon_attrib(attrib);
 }
 
 /* ARGSUSED */
@@ -710,6 +640,7 @@ typedef void (*makerecord) (s_rec *, dbref player, char *sortflags);
  * its 'key' to sort by. Sort of a hardcode munge.
  */
 struct sort_record {
+  char *ptr;	 /**< NULL except for sortkey */
   char *val;	 /**< The string this is */
   dbref db;	 /**< dbref (default 0, bad is -1) */
   char *str;	 /**< string comparisons */
@@ -743,7 +674,7 @@ struct sort_record {
 #define Compare(r,x,y) \
         ((x->db < 0 || y->db < 0) ? \
            ((x->db < 0 && y->db < 0) ? 0 : (x->db < 0 ? 2 : -2)) \
-           : ((r != 0) ? (r < 0 ? -2 : 2) \
+           : ((fabs(r) > 0.000000001) ? (r < 0 ? -2 : 2) \
              : (x->db == y->db ? 0 : (x->db < y->db ? -1 : 1)) \
            ) \
          )
@@ -790,7 +721,7 @@ f_comp(const void *s1, const void *s2)
 
 GENRECORD(gen_alphanum)
 {
-  int len;
+  size_t len;
   if (strchr(rec->val, ESC_CHAR)) {
     rec->str = mush_strdup(remove_markup(rec->val, &len), "genrecord");
     rec->freestr = 1;
@@ -1061,7 +992,7 @@ gencomp(dbref player, char *a, char *b, char *sort_type)
  */
 
 void
-do_gensort(dbref player, char *s[], int n, char *sort_type)
+do_gensort(dbref player, char *keys[], char *strs[], int n, char *sort_type)
 {
   char *ptr;
   static char stype[BUFFER_LEN];
@@ -1087,12 +1018,17 @@ do_gensort(dbref player, char *s[], int n, char *sort_type)
   }
   sp = (s_rec *) mush_malloc(n * sizeof(s_rec), "do_gensort");
   for (i = 0; i < n; i++) {
-    sp[i].val = s[i];
+    sp[i].val = keys[i];
     sp[i].freestr = 0;
     sp[i].db = 0;
+    if (strs) {
+      sp[i].ptr = strs[i];
+    } else {
+      sp[i].ptr = NULL;
+    }
     sp[i].str = NULL;
     if (ltypelist[sorti].isdbs) {
-      sp[i].db = parse_objid(s[i]);
+      sp[i].db = parse_objid(keys[i]);
       if (!RealGoodObject(sp[i].db))
 	sp[i].db = NOTHING;
     }
@@ -1101,7 +1037,10 @@ do_gensort(dbref player, char *s[], int n, char *sort_type)
   qsort((void *) sp, n, sizeof(s_rec), ltypelist[sorti].sorter);
 
   for (i = 0; i < n; i++) {
-    s[i] = sp[i].val;
+    keys[i] = sp[i].val;
+    if (strs) {
+      strs[i] = sp[i].ptr;
+    }
     if (sp[i].freestr)
       mush_free(sp[i].str, "genrecord");
   }
@@ -1131,7 +1070,7 @@ FUNCTION(fun_sort)
 
   nptrs = list2arr(ptrs, MAX_SORTSIZE, args[0], sep);
   sort_type = get_list_type(args, nargs, 2, ptrs, nptrs);
-  do_gensort(executor, ptrs, nptrs, sort_type);
+  do_gensort(executor, ptrs, NULL, nptrs, sort_type);
   arr2list(ptrs, nptrs, buff, bp, outsep);
 }
 
@@ -1199,6 +1138,74 @@ loop:
   }
 }
 
+/* ARGSUSED */
+FUNCTION(fun_sortkey)
+{
+  char *ptrs[MAX_SORTSIZE];
+  char *keys[MAX_SORTSIZE];
+  int nptrs;
+  char *sort_type;
+  char sep;
+  char outsep[BUFFER_LEN];
+  int i;
+  char tbuff[BUFFER_LEN];
+  char *tp;
+  char const *cp;
+  char result[BUFFER_LEN];
+  char *rp;
+  ATTR *attrib;
+  dbref thing;
+
+  /* sortkey(attr,list,sort_type,delim,osep) */
+
+  if (!nargs || !*args[0] || !*args[1])
+    return;
+
+  if (!delim_check(buff, bp, nargs, args, 4, &sep))
+    return;
+
+  if (nargs < 5) {
+    outsep[0] = sep;
+    outsep[1] = '\0';
+  } else
+    strcpy(outsep, args[4]);
+
+  /* Find object and attribute to get sortby function from. */
+  parse_anon_attrib(executor, args[0], &thing, &attrib);
+  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
+    free_anon_attrib(attrib);
+    return;
+  }
+  if (!CanEvalAttr(executor, thing, attrib)) {
+    free_anon_attrib(attrib);
+    return;
+  }
+  tp = tbuff;
+  safe_str(atr_value(attrib), tbuff, &tp);
+  *tp = '\0';
+
+
+  nptrs = list2arr(ptrs, MAX_SORTSIZE, args[1], sep);
+
+  /* Now we make a list of keys */
+  for (i = 0; i < nptrs; i++) {
+    global_eval_context.wenv[0] = (char *) ptrs[i];
+    rp = result;
+    cp = tbuff;
+    process_expression(result, &rp, &cp,
+		       thing, executor, enactor,
+		       PE_DEFAULT, PT_DEFAULT, pe_info);
+    *rp = '\0';
+    keys[i] = mush_strdup(result, "sortkey");
+  }
+
+  sort_type = get_list_type(args, nargs, 3, keys, nptrs);
+  do_gensort(executor, keys, ptrs, nptrs, sort_type);
+  arr2list(ptrs, nptrs, buff, bp, outsep);
+  for (i = 0; i < nptrs; i++) {
+    mush_free(keys[i], "sortkey");
+  }
+}
 
 /* ARGSUSED */
 FUNCTION(fun_sortby)
@@ -1255,72 +1262,6 @@ FUNCTION(fun_sortby)
   free_anon_attrib(attrib);
 }
 
-/* ARGSUSED */
-FUNCTION(fun_setunion)
-{
-  char sep;
-  char **a1, **a2;
-  char *tempbuff;
-  int n1, i, a;
-  char *sort_type = ALPHANUM_LIST;
-  char *osep = NULL, osepd[2] = { '\0', '\0' };
-
-  /* if no lists, then no work */
-  if (!*args[0] && !*args[1])
-    return;
-
-  if (!delim_check(buff, bp, nargs, args, 3, &sep))
-    return;
-
-  tempbuff = (char *) mush_malloc((BUFFER_LEN * 2) + 4, "string");
-  a1 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
-  a2 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
-  if (!tempbuff || !a1 || !a2)
-    mush_panic("Unable to allocate memory in fun_setunion");
-  /* Concat both lists, make array, sort */
-  if (!*args[0])
-    memcpy(tempbuff, args[1], arglens[1] + 1);
-  else if (!*args[1])
-    memcpy(tempbuff, args[0], arglens[0] + 1);
-  else
-    sprintf(tempbuff, "%s%c%s", args[0], sep, args[1]);
-  n1 = list2arr(a1, MAX_SORTSIZE, tempbuff, sep);
-
-  if (nargs < 4) {
-    osepd[0] = sep;
-    osep = osepd;
-  } else if (nargs == 4) {
-    sort_type = get_list_type_noauto(args, nargs, 4);
-    if (sort_type == UNKNOWN_LIST) {
-      sort_type = ALPHANUM_LIST;
-      osep = args[3];
-    } else {
-      osepd[0] = sep;
-      osep = osepd;
-    }
-  } else if (nargs == 5) {
-    sort_type = get_list_type(args, nargs, 4, a1, n1);
-    osep = args[4];
-  }
-
-  do_gensort(executor, a1, n1, sort_type);
-
-  /* Strip the duplicates and make a2 contain the list */
-  a = 0;
-  for (i = 0; i < n1; i++) {
-    if (((a == 0) || (gencomp(executor, a1[i], a2[a - 1], sort_type) != 0))) {
-      a2[a] = a1[i];
-      a++;
-    }
-  }
-
-  /* Return our sorted result */
-  arr2list(a2, a, buff, bp, osep);
-  mush_free((Malloc_t) tempbuff, "string");
-  mush_free((Malloc_t) a1, "ptrarray");
-  mush_free((Malloc_t) a2, "ptrarray");
-}
-
 /* ARGSUSED */
 FUNCTION(fun_setinter)
 {
@@ -1341,7 +1282,7 @@ FUNCTION(fun_setinter)
   a1 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
   a2 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
   if (!a1 || !a2)
-    mush_panic("Unable to allocate memory in fun_setunion");
+    mush_panic("Unable to allocate memory in fun_inter");
 
   /* make arrays out of the lists */
   n1 = list2arr(a1, MAX_SORTSIZE, args[0], sep);
@@ -1370,8 +1311,8 @@ FUNCTION(fun_setinter)
     osepl = arglens[4];
   }
   /* sort each array */
-  do_gensort(executor, a1, n1, sort_type);
-  do_gensort(executor, a2, n2, sort_type);
+  do_gensort(executor, a1, NULL, n1, sort_type);
+  do_gensort(executor, a2, NULL, n2, sort_type);
 
   /* get the first value for the intersection, removing duplicates */
   x1 = x2 = 0;
@@ -1437,11 +1378,12 @@ FUNCTION(fun_setinter)
 }
 
 /* ARGSUSED */
-FUNCTION(fun_setdiff)
+FUNCTION(fun_setunion)
 {
   char sep;
   char **a1, **a2;
   int n1, n2, x1, x2, val;
+  int lastx1, lastx2, found;
   char *sort_type = ALPHANUM_LIST;
   int osepl = 0;
   char *osep = NULL, osepd[2] = { '\0', '\0' };
@@ -1462,6 +1404,142 @@ FUNCTION(fun_setdiff)
   n1 = list2arr(a1, MAX_SORTSIZE, args[0], sep);
   n2 = list2arr(a2, MAX_SORTSIZE, args[1], sep);
 
+  if (nargs < 4) {
+    osepd[0] = sep;
+    osep = osepd;
+    if (sep)
+      osepl = 1;
+  } else if (nargs == 4) {
+    sort_type = get_list_type_noauto(args, nargs, 4);
+    if (sort_type == UNKNOWN_LIST) {
+      sort_type = ALPHANUM_LIST;
+      osep = args[3];
+      osepl = arglens[3];
+    } else {
+      osepd[0] = sep;
+      osep = osepd;
+      if (sep)
+	osepl = 1;
+    }
+  } else if (nargs == 5) {
+    sort_type = get_list_type(args, nargs, 4, a1, n1);
+    osep = args[4];
+    osepl = arglens[4];
+  }
+  /* sort each array */
+  do_gensort(executor, a1, NULL, n1, sort_type);
+  do_gensort(executor, a2, NULL, n2, sort_type);
+
+  /* get values for the union, in order, skipping duplicates */
+  lastx1 = lastx2 = -1;
+  found = x1 = x2 = 0;
+  if (n1 == 1 && !*a1[0])
+    n1 = 0;
+  if (n2 == 1 && !*a2[0])
+    n2 = 0;
+  while ((x1 < n1) || (x2 < n2)) {
+    /* If we've already copied off something from a1, and our current
+     * look at a1 is the same element, or we've copied from a2 and
+     * our current look at a1 is the same element, skip forward in a1.
+     */
+    if (x1 < n1 && lastx1 >= 0) {
+      val = gencomp(executor, a1[lastx1], a1[x1], sort_type);
+      if (val == 0) {
+	x1++;
+	continue;
+      }
+    }
+    if (x1 < n1 && lastx2 >= 0) {
+      val = gencomp(executor, a2[lastx2], a1[x1], sort_type);
+      if (val == 0) {
+	x1++;
+	continue;
+      }
+    }
+    if (x2 < n2 && lastx1 >= 0) {
+      val = gencomp(executor, a1[lastx1], a2[x2], sort_type);
+      if (val == 0) {
+	x2++;
+	continue;
+      }
+    }
+    if (x2 < n2 && lastx2 >= 0) {
+      val = gencomp(executor, a2[lastx2], a2[x2], sort_type);
+      if (val == 0) {
+	x2++;
+	continue;
+      }
+    }
+    if (x1 >= n1) {
+      /* Just copy off the rest of a2 */
+      if (x2 < n2) {
+	if (found)
+	  safe_strl(osep, osepl, buff, bp);
+	safe_str(a2[x2], buff, bp);
+	lastx2 = x2;
+	x2++;
+	found = 1;
+      }
+    } else if (x2 >= n2) {
+      /* Just copy off the rest of a1 */
+      if (x1 < n1) {
+	if (found)
+	  safe_strl(osep, osepl, buff, bp);
+	safe_str(a1[x1], buff, bp);
+	lastx1 = x1;
+	x1++;
+	found = 1;
+      }
+    } else {
+      /* At this point, we're merging. Take the lower of the two. */
+      val = gencomp(executor, a1[x1], a2[x2], sort_type);
+      if (val <= 0) {
+	if (found)
+	  safe_strl(osep, osepl, buff, bp);
+	safe_str(a1[x1], buff, bp);
+	lastx1 = x1;
+	x1++;
+	found = 1;
+      } else {
+	if (found)
+	  safe_strl(osep, osepl, buff, bp);
+	safe_str(a2[x2], buff, bp);
+	lastx2 = x2;
+	x2++;
+	found = 1;
+      }
+    }
+  }
+  mush_free((Malloc_t) a1, "ptrarray");
+  mush_free((Malloc_t) a2, "ptrarray");
+}
+
+/* ARGSUSED */
+FUNCTION(fun_setdiff)
+{
+  char sep;
+  char **a1, **a2;
+  int n1, n2, x1, x2, val;
+  char *sort_type = ALPHANUM_LIST;
+  int osepl = 0;
+  char *osep = NULL, osepd[2] = { '\0', '\0' };
+
+  /* if no lists, then no work */
+  if (!*args[0] && !*args[1])
+    return;
+
+  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+    return;
+
+  a1 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
+  a2 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
+  if (!a1 || !a2)
+    mush_panic("Unable to allocate memory in fun_diff");
+
+  /* make arrays out of the lists */
+  n1 = list2arr(a1, MAX_SORTSIZE, args[0], sep);
+  n2 = list2arr(a2, MAX_SORTSIZE, args[1], sep);
+
   if (nargs < 4) {
     osepd[0] = sep;
     osep = osepd;
@@ -1486,8 +1564,8 @@ FUNCTION(fun_setdiff)
   }
 
   /* sort each array */
-  do_gensort(executor, a1, n1, sort_type);
-  do_gensort(executor, a2, n2, sort_type);
+  do_gensort(executor, a1, NULL, n1, sort_type);
+  do_gensort(executor, a2, NULL, n2, sort_type);
 
   /* get the first value for the difference, removing duplicates */
   x1 = x2 = 0;
@@ -1550,6 +1628,70 @@ FUNCTION(fun_setdiff)
 
 #define CACHE_SIZE 8  /**< Maximum size of the lnum cache */
 
+FUNCTION(fun_unique)
+{
+  char sep;
+  char **a1, **a2;
+  int n1, x1, x2;
+  char *sort_type = ALPHANUM_LIST;
+  int osepl = 0;
+  char *osep = NULL, osepd[2] = { '\0', '\0' };
+
+  /* if no lists, then no work */
+  if (!*args[0])
+    return;
+
+  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+    return;
+
+  a1 = (char **) mush_malloc(MAX_SORTSIZE * sizeof(char *), "ptrarray");
+
+  if (!a1)
+    mush_panic("Unable to allocate memory in fun_unique");
+
+  /* make array out of the list */
+  n1 = list2arr(a1, MAX_SORTSIZE, args[0], sep);
+
+  a2 = mush_malloc(n1 * sizeof(char *), "ptrarray");
+  if (!a2)
+    mush_panic("Unable to allocate memory in fun_unique");
+
+  if (nargs >= 2)
+    sort_type = get_list_type_noauto(args, nargs, 2);
+
+  if (sort_type == UNKNOWN_LIST)
+    sort_type = ALPHANUM_LIST;
+
+  if (nargs < 4) {
+    osepd[0] = sep;
+    osep = osepd;
+    if (sep)
+      osepl = 1;
+  } else if (nargs == 4) {
+    osep = args[3];
+    osepl = arglens[3];
+  }
+
+
+  a2[0] = a1[0];
+  for (x1 = x2 = 1; x1 < n1; x1++) {
+    if (gencomp(executor, a1[x1], a2[x2 - 1], sort_type) == 0)
+      continue;
+    a2[x2] = a1[x1];
+    x2++;
+  }
+
+  for (x1 = 0; x1 < x2; x1++) {
+    if (x1 > 0)
+      safe_strl(osep, osepl, buff, bp);
+    safe_str(a2[x1], buff, bp);
+  }
+
+  mush_free(a1, "ptrarray");
+  mush_free(a2, "ptrarray");
+
+}
+
 /* ARGSUSED */
 FUNCTION(fun_lnum)
 {
@@ -1761,6 +1903,112 @@ FUNCTION(fun_grab)
   } while (s);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_namegraball)
+{
+  /* Given a list of dbrefs and a string, it matches the
+   * name of the dbrefs against the string.
+   * grabnameall(#1 #2 #3,god) -> #1
+   */
+
+  char *r, *s, sep;
+  dbref victim;
+  dbref absolute;
+  int first = 1;
+
+  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+    return;
+
+  absolute = parse_dbref(args[1]);
+  if (!RealGoodObject(absolute))
+    absolute = NOTHING;
+
+  if (*args[1]) {
+    s = trim_space_sep(args[0], sep);
+    do {
+      r = split_token(&s, sep);
+      victim = parse_dbref(r);
+      if (!RealGoodObject(victim))
+	continue;		/* Don't bother with garbage */
+      if (!(string_match(Name(victim), args[1]) || (absolute == victim)))
+	continue;
+      if (!can_interact(victim, executor, INTERACT_MATCH))
+	continue;
+      /* It matches, and is interact-able */
+      if (!first)
+	safe_chr(sep, buff, bp);
+      safe_str(r, buff, bp);
+      first = 0;
+    } while (s);
+  } else {
+    /* Pull out all good objects (those that _have_ names) */
+    s = trim_space_sep(args[0], sep);
+    do {
+      r = split_token(&s, sep);
+      victim = parse_dbref(r);
+      if (!RealGoodObject(victim))
+	continue;		/* Don't bother with garbage */
+      if (!can_interact(victim, executor, INTERACT_MATCH))
+	continue;
+      /* It's real, and is interact-able */
+      if (!first)
+	safe_chr(sep, buff, bp);
+      safe_str(r, buff, bp);
+      first = 0;
+    } while (s);
+  }
+}
+
+/* ARGSUSED */
+FUNCTION(fun_namegrab)
+{
+  /* Given a list of dbrefs and a string, it matches the
+   * name of the dbrefs against the string.
+   */
+
+  char *r, *s, sep;
+  dbref victim;
+  dbref absolute;
+  char *exact_res, *res;
+
+  exact_res = res = NULL;
+
+  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+    return;
+
+  absolute = parse_dbref(args[1]);
+  if (!RealGoodObject(absolute))
+    absolute = NOTHING;
+
+  /* Walk the wordstring, until we find the word we want. */
+  s = trim_space_sep(args[0], sep);
+  do {
+    r = split_token(&s, sep);
+    victim = parse_dbref(r);
+    if (!RealGoodObject(victim))
+      continue;			/* Don't bother with garbage */
+    /* Dbref match has top priority */
+    if ((absolute == victim) && can_interact(victim, executor, INTERACT_MATCH)) {
+      safe_str(r, buff, bp);
+      return;
+    }
+    /* Exact match has second priority */
+    if (!exact_res && !strcasecmp(Name(victim), args[1]) &&
+	can_interact(victim, executor, INTERACT_MATCH)) {
+      exact_res = r;
+    }
+    /* Non-exact match. */
+    if (!res && string_match(Name(victim), args[1]) &&
+	can_interact(victim, executor, INTERACT_MATCH)) {
+      res = r;
+    }
+  } while (s);
+  if (exact_res)
+    safe_str(exact_res, buff, bp);
+  else if (res)
+    safe_str(res, buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_match)
 {
@@ -1823,18 +2071,27 @@ FUNCTION(fun_wordpos)
 /* ARGSUSED */
 FUNCTION(fun_extract)
 {
-  char sep;
-  int start, len;
+  char sep = ' ';
+  int start = 1, len = 1;
   char *s, *r;
 
-  if (!is_integer(args[1]) || !is_integer(args[2])) {
-    safe_str(T(e_ints), buff, bp);
-    return;
-  }
   s = args[0];
-  start = parse_integer(args[1]);
-  len = parse_integer(args[2]);
-  if (!delim_check(buff, bp, nargs, args, 4, &sep))
+
+  if (nargs > 1) {
+    if (!is_integer(args[1])) {
+      safe_str(T(e_ints), buff, bp);
+      return;
+    }
+    start = parse_integer(args[1]);
+  }
+  if (nargs > 2) {
+    if (!is_integer(args[2])) {
+      safe_str(T(e_ints), buff, bp);
+      return;
+    }
+    len = parse_integer(args[2]);
+  }
+  if ((nargs > 3) && (!delim_check(buff, bp, nargs, args, 4, &sep)))
     return;
 
   if ((start < 1) || (len < 1))
@@ -2025,11 +2282,11 @@ FUNCTION(fun_index)
  * \param num string containing the element number to operate on.
  * \param word string to insert/delete/replace.
  * \param sep separator string.
- * \param flag operation to perform: 0 - delete, 1 - replace, 2 - insert
+ * \param flag operation to perform: IF_DELETE, IF_REPLACE, IF_INSERT
  */
 static void
 do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
-	    char *sep, int flag)
+	    char *sep, enum itemfun_op flag)
 {
   char c;
   int el, count, len = -1;
@@ -2048,7 +2305,7 @@ do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
     c = ' ';
 
   /* we can't remove anything before the first position */
-  if ((el < 1 && flag != 2) || el == 0) {
+  if ((el < 1 && flag != IF_INSERT) || el == 0) {
     safe_str(str, buff, bp);
     return;
   }
@@ -2098,7 +2355,7 @@ do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
     sptr[-1] = '\0';
 
   switch (flag) {
-  case 0:
+  case IF_DELETE:
     /* deletion */
     if (!eptr) {		/* last element in the string */
       if (el != 1)
@@ -2111,7 +2368,7 @@ do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
       safe_str(eptr, buff, bp);
     }
     break;
-  case 1:
+  case IF_REPLACE:
     /* replacing */
     if (!eptr) {		/* last element in string */
       if (el != 1) {
@@ -2129,7 +2386,7 @@ do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
       safe_str(eptr, buff, bp);
     }
     break;
-  case 2:
+  case IF_INSERT:
     /* insertion */
     if (sptr == str) {		/* first element in string */
       safe_str(word, buff, bp);
@@ -2139,8 +2396,10 @@ do_itemfuns(char *buff, char **bp, char *str, char *num, char *word,
       safe_str(str, buff, bp);
       safe_chr(c, buff, bp);
       safe_str(word, buff, bp);
-      safe_chr(c, buff, bp);
-      safe_str(sptr, buff, bp);
+      if (sptr && *sptr) {	/* Don't add an osep to the end of the list */
+	safe_chr(c, buff, bp);
+	safe_str(sptr, buff, bp);
+      }
     }
     break;
   }
@@ -2152,7 +2411,7 @@ FUNCTION(fun_ldelete)
 {
   /* delete a word at position X of a list */
 
-  do_itemfuns(buff, bp, args[0], args[1], NULL, args[2], 0);
+  do_itemfuns(buff, bp, args[0], args[1], NULL, args[2], IF_DELETE);
 }
 
 /* ARGSUSED */
@@ -2160,7 +2419,7 @@ FUNCTION(fun_replace)
 {
   /* replace a word at position X of a list */
 
-  do_itemfuns(buff, bp, args[0], args[1], args[2], args[3], 1);
+  do_itemfuns(buff, bp, args[0], args[1], args[2], args[3], IF_REPLACE);
 }
 
 /* ARGSUSED */
@@ -2168,7 +2427,7 @@ FUNCTION(fun_insert)
 {
   /* insert a word at position X of a list */
 
-  do_itemfuns(buff, bp, args[0], args[1], args[2], args[3], 2);
+  do_itemfuns(buff, bp, args[0], args[1], args[2], args[3], IF_INSERT);
 }
 
 /* ARGSUSED */
@@ -2567,78 +2826,50 @@ FUNCTION(fun_map)
    * This function takes delimiters.
    */
 
-  dbref thing;
-  ATTR *attrib;
-  char const *ap;
-  char *asave, *lp;
-  char *tptr[2];
+  ufun_attrib ufun;
+  char *lp;
+  char *wenv[2];
   char place[16];
   int placenr = 1;
   char sep;
   int funccount;
-  char *oldbp;
   char *osep, osepd[2] = { '\0', '\0' };
-  int pe_flags = PE_DEFAULT;
+  char rbuff[BUFFER_LEN];
 
   if (!delim_check(buff, bp, nargs, args, 3, &sep))
     return;
 
-  if (nargs == 4)
-    osep = args[3];
-  else {
-    osepd[0] = sep;
-    osep = osepd;
-  }
+  osepd[0] = sep;
+  osep = (nargs >= 4) ? args[3] : osepd;
 
   lp = trim_space_sep(args[1], sep);
   if (!*lp)
     return;
 
-  /* find our object and attribute */
-  parse_anon_attrib(executor, args[0], &thing, &attrib);
-  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
+  if (!fetch_ufun_attrib(args[0], executor, &ufun, 1))
     return;
-  }
-  if (!CanEvalAttr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
-    return;
-  }
-  if (AF_Debug(attrib))
-    pe_flags |= PE_DEBUG;
 
   strcpy(place, "1");
-  asave = safe_atr_value(attrib);
 
-  /* save our stack */
-  tptr[0] = global_eval_context.wenv[0];
-  tptr[1] = global_eval_context.wenv[1];
-  global_eval_context.wenv[1] = place;
+  /* Build our %0 args */
+  wenv[0] = split_token(&lp, sep);
+  wenv[1] = place;
 
-  global_eval_context.wenv[0] = split_token(&lp, sep);
-  ap = asave;
-  process_expression(buff, bp, &ap, thing, executor, enactor,
-		     pe_flags, PT_DEFAULT, pe_info);
-  oldbp = *bp;
+  call_ufun(&ufun, wenv, 2, rbuff, executor, enactor, pe_info);
   funccount = pe_info->fun_invocations;
+  safe_str(rbuff, buff, bp);
   while (lp) {
     safe_str(osep, buff, bp);
     strcpy(place, unparse_integer(++placenr));
-    global_eval_context.wenv[0] = split_token(&lp, sep);
-    ap = asave;
-    if (process_expression(buff, bp, &ap, thing, executor, enactor,
-			   pe_flags, PT_DEFAULT, pe_info))
+    wenv[0] = split_token(&lp, sep);
+
+    if (call_ufun(&ufun, wenv, 2, rbuff, executor, enactor, pe_info))
       break;
+    safe_str(rbuff, buff, bp);
     if (*bp == (buff + BUFFER_LEN - 1) && pe_info->fun_invocations == funccount)
       break;
-    oldbp = *bp;
     funccount = pe_info->fun_invocations;
   }
-
-  free((Malloc_t) asave);
-  free_anon_attrib(attrib);
-  global_eval_context.wenv[0] = tptr[0];
-  global_eval_context.wenv[1] = tptr[1];
 }
 
 
@@ -2650,17 +2881,15 @@ FUNCTION(fun_mix)
    * This function takes delimiters.
    */
 
-  dbref thing;
-  ATTR *attrib;
-  char const *ap;
-  char *asave, *lp[10];
-  char *tptr[10];
+  ufun_attrib ufun;
+  char rbuff[BUFFER_LEN];
+  char *lp[10];
+  char *list[10];
   char sep;
   int funccount;
   int n;
   int lists, words;
-  char *oldbp;
-  int pe_flags = PE_DEFAULT;
+  int first = 1;
 
   if (nargs > 3) {		/* Last arg must be the delimiter */
     n = nargs;
@@ -2677,69 +2906,31 @@ FUNCTION(fun_mix)
     lp[n] = trim_space_sep(args[n + 1], sep);
 
   /* find our object and attribute */
-  parse_anon_attrib(executor, args[0], &thing, &attrib);
-  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
+  if (!fetch_ufun_attrib(args[0], executor, &ufun, 1))
     return;
-  }
-  if (!CanEvalAttr(executor, thing, attrib)) {
-    free_anon_attrib(attrib);
-    return;
-  }
-  if (AF_Debug(attrib))
-    pe_flags |= PE_DEBUG;
 
-  asave = safe_atr_value(attrib);
-
-  /* save our stack */
-  save_global_env("fun_mix", tptr);
-
-  words = 0;
-  for (n = 0; n < 10; n++) {
-    if ((n < lists) && lp[n] && *lp[n]) {
-      global_eval_context.wenv[n] = split_token(&lp[n], sep);
-      if (global_eval_context.wenv[n])
-	words++;
-    } else
-      global_eval_context.wenv[n] = NULL;
-  }
-  if (words == 0) {
-    restore_global_env("fun_mix", tptr);
-    free((Malloc_t) asave);
-    free_anon_attrib(attrib);
-    return;
-  }
-  ap = asave;
-  process_expression(buff, bp, &ap, thing, executor, enactor,
-		     pe_flags, PT_DEFAULT, pe_info);
-  oldbp = *bp;
-  funccount = pe_info->fun_invocations;
+  first = 0;
   while (1) {
     words = 0;
-    for (n = 0; n < 10; n++) {
-      if ((n < lists) && lp[n] && *lp[n]) {
-	global_eval_context.wenv[n] = split_token(&lp[n], sep);
-	if (global_eval_context.wenv[n])
+    for (n = 0; n < lists; n++) {
+      if (lp[n] && *lp[n]) {
+	list[n] = split_token(&lp[n], sep);
+	if (list[n])
 	  words++;
-      } else
-	global_eval_context.wenv[n] = NULL;
+      } else {
+	list[n] = NULL;
+      }
     }
-    if (words == 0)
-      break;
-    safe_chr(sep, buff, bp);
-    ap = asave;
-    if (process_expression(buff, bp, &ap, thing, executor, enactor,
-			   pe_flags, PT_DEFAULT, pe_info))
-      break;
-    if (*bp == (buff + BUFFER_LEN - 1) && pe_info->fun_invocations == funccount)
-      break;
-    oldbp = *bp;
+    if (!words)
+      return;
+    if (first)
+      first = 0;
+    else
+      safe_chr(sep, buff, bp);
     funccount = pe_info->fun_invocations;
+    call_ufun(&ufun, list, lists, rbuff, executor, enactor, pe_info);
+    safe_str(rbuff, buff, bp);
   }
-
-  free((Malloc_t) asave);
-  free_anon_attrib(attrib);
-  restore_global_env("fun_mix", tptr);
 }
 
 /* ARGSUSED */
@@ -2851,9 +3042,9 @@ FUNCTION(fun_table)
 
 /* string, regexp, replacement string. Acts like sed or perl's s///g,
  * with an ig version */
-int re_subpatterns = -1;  /**< Number of subpatterns in regexp */
-int *re_offsets;	  /**< Array of offsets to subpatterns */
-char *re_from = NULL;	  /**< Pointer to last match position */
+// int re_subpatterns = -1;  /**< Number of subpatterns in regexp */
+// int *re_offsets;       /**< Array of offsets to subpatterns */
+// char *re_from = NULL;          /**< Pointer to last match position */
 FUNCTION(fun_regreplace)
 {
   pcre *re;
@@ -2868,10 +3059,18 @@ FUNCTION(fun_regreplace)
   char abuf[BUFFER_LEN], *abp;
   char prebuf[BUFFER_LEN], *prep;
   char postbuf[BUFFER_LEN], *postp;
-
+  pcre *old_re_code;
+  int old_re_subpatterns;
+  int *old_re_offsets;
+  char *old_re_from;
   int flags = 0, all = 0, match_offset = 0, len, funccount;
   int i;
 
+  old_re_code = global_eval_context.re_code;
+  old_re_subpatterns = global_eval_context.re_subpatterns;
+  old_re_offsets = global_eval_context.re_offsets;
+  old_re_from = global_eval_context.re_from;
+
   if (called_as[strlen(called_as) - 1] == 'I')
     flags = PCRE_CASELESS;
 
@@ -2946,9 +3145,10 @@ FUNCTION(fun_regreplace)
 
       /* Now copy in the replacement, putting in captured sub-expressions */
       obp = args[i + 1];
-      re_from = prebuf;
-      re_offsets = offsets;
-      re_subpatterns = subpatterns;
+      global_eval_context.re_code = re;
+      global_eval_context.re_from = prebuf;
+      global_eval_context.re_offsets = offsets;
+      global_eval_context.re_subpatterns = subpatterns;
       process_expression(postbuf, &postp, &obp, executor, caller, enactor,
 			 PE_DEFAULT | PE_DOLLAR, PT_DEFAULT, pe_info);
       if ((*bp == (buff + BUFFER_LEN - 1))
@@ -2976,9 +3176,10 @@ FUNCTION(fun_regreplace)
     if (study)
       mush_free((Malloc_t) study, "pcre.extra");
 
-    re_offsets = NULL;
-    re_subpatterns = -1;
-    re_from = NULL;
+    global_eval_context.re_code = old_re_code;
+    global_eval_context.re_offsets = old_re_offsets;
+    global_eval_context.re_subpatterns = old_re_subpatterns;
+    global_eval_context.re_from = old_re_from;
   }
 
   safe_str(postbuf, buff, bp);
@@ -2995,11 +3196,7 @@ FUNCTION(fun_regmatch)
  * the results of a regexp pattern match into a set of r()-registers.
  *
  * regmatch(string, pattern, list of registers)
- * If the number of matches exceeds the registers, those bits are tossed
- * out.
- * If -1 is specified as a register number, the matching bit is tossed.
- * Therefore, if the list is "-1 0 3 5", the regexp $0 is tossed, and
- * the regexp $1, $2, and $3 become r(0), r(3), and r(5), respectively.
+ * Registers are by position (old way) or name:register (new way)
  *
  */
   int i, nqregs, curq;
@@ -3039,17 +3236,38 @@ FUNCTION(fun_regmatch)
     subpatterns = 33;
   nqregs = list2arr(qregs, NUMQ, args[2], ' ');
   for (i = 0; i < nqregs; i++) {
-    if (qregs[i] && qregs[i][0] && !qregs[i][1] &&
-	((qindex = qreg_indexes[(unsigned char) qregs[i][0]]) != -1))
+    char *regname;
+    char *named_subpattern = NULL;
+    int subpattern = 0;
+    if ((regname = strchr(qregs[i], ':'))) {
+      /* subexpr:register */
+      *regname++ = '\0';
+      if (is_strict_integer(qregs[i]))
+	subpattern = parse_integer(qregs[i]);
+      else
+	named_subpattern = qregs[i];
+    } else {
+      /* Get subexper by position in list */
+      subpattern = i;
+      regname = qregs[i];
+    }
+
+    if (regname && regname[0] && !regname[1] &&
+	((qindex = qreg_indexes[(unsigned char) regname[0]]) != -1))
       curq = qindex;
     else
       curq = -1;
     if (curq < 0 || curq >= NUMQ)
       continue;
+
     if (subpatterns < 0)
       global_eval_context.renv[curq][0] = '\0';
+    else if (named_subpattern)
+      pcre_copy_named_substring(re, args[0], offsets, subpatterns,
+				named_subpattern,
+				global_eval_context.renv[curq], BUFFER_LEN);
     else
-      pcre_copy_substring(args[0], offsets, subpatterns, i,
+      pcre_copy_substring(args[0], offsets, subpatterns, subpattern,
 			  global_eval_context.renv[curq], BUFFER_LEN);
   }
   mush_free((Malloc_t) re, "pcre");
