@@ -28,7 +28,11 @@
 #define M_PI 3.14159265358979323846264338327
 #endif
 
+#define EPSILON 0.000000001  /**< limit of precision for float equality */
+#define EQ(x,y) (fabs(x-y) < EPSILON)  /**< floating point equality macro */
+
 static void do_spellnum(char *num, unsigned int len, char **buff, char ***bp);
+static void do_ordinalize(char **buff, char ***bp);
 static int nval_sort(const void *, const void *);
 static NVAL find_median(NVAL *, int);
 
@@ -50,7 +54,8 @@ static void math_hash_insert(const char *, MATH *);
 static MATH *math_hash_lookup(char *);
 static NVAL angle_to_rad(NVAL angle, const char *from);
 static NVAL rad_to_angle(NVAL angle, const char *to);
-static double frac(double v, int *RESTRICT n, int *RESTRICT d, double error);
+static double frac(double v, double *RESTRICT n, double *RESTRICT d,
+		   double error);
 void init_math_hashtab(void);
 
 extern int format_long(long n, char *buff, char **bp, int maxlen, int base);
@@ -164,7 +169,7 @@ FUNCTION(fun_eq)
     safe_str(T(e_nums), buff, bp);
     return;
   }
-  safe_boolean(parse_number(args[0]) == parse_number(args[1]), buff, bp);
+  safe_boolean(EQ(parse_number(args[0]), parse_number(args[1])), buff, bp);
 }
 
 /* ARGSUSED */
@@ -174,7 +179,7 @@ FUNCTION(fun_neq)
     safe_str(T(e_nums), buff, bp);
     return;
   }
-  safe_boolean(parse_number(args[0]) != parse_number(args[1]), buff, bp);
+  safe_boolean(!EQ(parse_number(args[0]), parse_number(args[1])), buff, bp);
 }
 
 /* ARGSUSED */
@@ -199,7 +204,7 @@ FUNCTION(fun_sign)
     return;
   }
   x = parse_number(args[0]);
-  if (x == 0)
+  if (EQ(x, 0))
     safe_chr('0', buff, bp);
   else if (x > 0)
     safe_chr('1', buff, bp);
@@ -734,7 +739,7 @@ FUNCTION(fun_vunit)
   }
   sum = sqrt(sum);
 
-  if (!sum) {
+  if (EQ(sum, 0)) {
     /* zero vector */
     p1 = tbuf;
     safe_chr('0', buff, bp);
@@ -802,7 +807,7 @@ FUNCTION(fun_fmod)
     return;
   }
   y = parse_number(args[1]);
-  if (y == 0) {
+  if (EQ(y, 0)) {
     safe_str(T("#-1 DIVISION BY ZERO"), buff, bp);
     return;
   }
@@ -992,7 +997,7 @@ FUNCTION(fun_power)
   }
   num = parse_number(args[0]);
   m = parse_number(args[1]);
-  if (num < 0 && (m != (int) m)) {
+  if (num < 0 && (!EQ(m, (int) m))) {
     safe_str(T("#-1 FRACTIONAL POWER OF NEGATIVE"), buff, bp);
     return;
   }
@@ -1145,7 +1150,7 @@ FUNCTION(fun_root)
  * \return -1.0 if (v < MIN || v > MAX || error < 0.0) | (v - n/d) / v | otherwise.
  */
 static double
-frac(double v, int *RESTRICT n, int *RESTRICT d, double error)
+frac(double v, double *RESTRICT n, double *RESTRICT d, double error)
 {
 
 /* Based on a routine found in netlib (http://www.netlib.org) by
@@ -1164,28 +1169,28 @@ frac(double v, int *RESTRICT n, int *RESTRICT d, double error)
   */
 
 
-  int D, N, t;
+  double D, N, t;
   int first = 1;
   double epsilon, r = 0.0, m;
 
   if (v < 0 || error < 0.0)
     return -1.0;
   *d = D = 1;
-  *n = (int) v;
+  *n = floor(v);
   N = *n + 1;
 
   do {
     if (!first) {
       if (r <= 1.0)
 	r = 1.0 / r;
-      N += *n * (int) r;
-      D += *d * (int) r;
+      N += *n * floor(r);
+      D += *d * floor(r);
       *n += N;
       *d += D;
     } else
       first = 0;
     r = 0.0;
-    if (v * (*d) != *n) {
+    if (!EQ(v * (*d), *n)) {
       r = (N - v * D) / (v * (*d) - *n);
       if (r <= 1.0) {
 	t = N;
@@ -1203,16 +1208,16 @@ frac(double v, int *RESTRICT n, int *RESTRICT d, double error)
     do {
       m *= 10.0;
     } while (m * epsilon < 1.0);
-    epsilon = 1.0 / m * ((int) (0.5 + m * epsilon));
+    epsilon = 1.0 / m * (floor(0.5 + m * epsilon));
     if (epsilon <= error)
       return epsilon;
-  } while (r != 0.0);
+  } while (!EQ(r, 0.0));
   return epsilon;
 }
 
 FUNCTION(fun_fraction)
 {
-  int num = 0, denom = 0;
+  double num = 0, denom = 0;
   NVAL n;
   int sign = 0;
 
@@ -1226,7 +1231,7 @@ FUNCTION(fun_fraction)
   if (n < 0) {
     n = fabs(n);
     sign = 1;
-  } else if (n == 0) {
+  } else if (EQ(n, 0)) {
     safe_chr('0', buff, bp);
     return;
   }
@@ -1236,10 +1241,10 @@ FUNCTION(fun_fraction)
   if (sign)
     safe_chr('-', buff, bp);
 
-  if (denom == 1)
-    safe_integer(num, buff, bp);
+  if (fabs(denom - 1) < 1.0e-10)
+    safe_format(buff, bp, "%.0lf", num);
   else
-    safe_format(buff, bp, "%d/%d", num, denom);
+    safe_format(buff, bp, "%.0lf/%.0lf", num, denom);
 }
 
 FUNCTION(fun_isint)
@@ -1394,6 +1399,40 @@ do_spellnum(char *num, unsigned int len, char **buff, char ***bp)
   }
 }				/* do_spellnum */
 
+/** Convert the end of a spelled number string to ordinal form. */
+static void
+do_ordinalize(char **buff, char ***bp)
+{
+  char *p;
+  int i, len;
+  static const char *singles[] = { "one", "two", "three", "four",
+    "five", "six", "seven", "eight", "nine"
+  };
+  static const char *singleths[] = { "first", "second", "third", "fourth",
+    "fifth", "sixth", "seventh", "eighth", "ninth"
+  };
+  /* Examine the end of the string */
+  for (i = 0; i < 9; i++) {
+    len = strlen(singles[i]);
+    p = **bp - len;
+    if ((p >= *buff) && !strncasecmp(p, singles[i], len)) {
+      **bp = p;
+      safe_str(singleths[i], *buff, *bp);
+      return;			/* done */
+    }
+  }
+  /* The string didn't end with a single. How about a y? */
+  p = **bp - 1;
+  if ((p >= *buff) && (*p == 'y')) {
+    **bp = p;
+    safe_str("ieth", *buff, *bp);
+    return;
+  }
+  /* Ok, tack on th */
+  safe_str("th", *buff, *bp);
+}
+
+
 /** adds zeros to the beginning of the string, untill its length is 
  * a multiple of 3.
  */
@@ -1420,8 +1459,10 @@ FUNCTION(fun_spellnum)
   char *number,			/* the whole number (without -/+ sign and leading zeros) */
   *pnumber = args[0], *pnum1, *pnum2 = NULL;	/* part 1 and 2 of the number respectively */
   unsigned int len, m, minus = 0,	/* is the number negative? */
-   dot = 0, len1, len2;		/* length of part 1 and 2 respectively */
+    dot = 0, len1, len2;	/* length of part 1 and 2 respectively */
+  int ordinal_mode;
 
+  ordinal_mode = (strcmp(called_as, "ORDINAL") == 0);
   pnumber = trim_space_sep(args[0], ' ');
 
   /* Is the number negative? */
@@ -1445,6 +1486,11 @@ FUNCTION(fun_spellnum)
   for (m = 0; m < len; m++) {
 
     if (*pnumber == '.') {
+      if (ordinal_mode) {
+	/* Only integers may be ordinalized */
+	safe_str(T(e_int), buff, bp);
+	return;
+      }
       if (dot) {
 	safe_str(T(e_num), buff, bp);
 	return;
@@ -1482,6 +1528,12 @@ FUNCTION(fun_spellnum)
     do_spellnum(pnum1, len1, &buff, &bp);
   }
 
+  if (ordinal_mode) {
+    /* In this case, we're done right here. */
+    do_ordinalize(&buff, &bp);
+    return;
+  }
+
   if (len2 > 0) {
     /* after the . */
 
@@ -1934,7 +1986,7 @@ MATH_FUNC(math_fdiv)
     }
     temp = parse_number(ptr[n]);
 
-    if (temp == 0) {
+    if (EQ(temp, 0)) {
       safe_str(T("#-1 DIVISION BY ZERO"), buff, bp);
       return;
     }
