@@ -17,6 +17,7 @@
 #include "version.h"
 #include "htab.h"
 #include "flags.h"
+#include "lock.h"
 #include "match.h"
 #include "mushdb.h"
 #include "dbdefs.h"
@@ -51,13 +52,13 @@ FUNCTION(fun_valid)
   else if (!strcasecmp(args[0], "attrname"))
     safe_boolean(good_atr_name(upcasestr(args[1])), buff, bp);
   else if (!strcasecmp(args[0], "playername"))
-    safe_boolean(ok_player_name(args[1], executor), buff, bp);
+    safe_boolean(ok_player_name(args[1], executor, executor), buff, bp);
   else if (!strcasecmp(args[0], "password"))
     safe_boolean(ok_password(args[1]), buff, bp);
   else if (!strcasecmp(args[0], "command"))
     safe_boolean(ok_command_name(upcasestr(args[1])), buff, bp);
   else if (!strcasecmp(args[0], "function"))
-    safe_boolean(ok_command_name(upcasestr(args[1])), buff, bp);
+    safe_boolean(ok_function_name(upcasestr(args[1])), buff, bp);
   else
     safe_str("#-1", buff, bp);
 }
@@ -149,6 +150,22 @@ FUNCTION(fun_zemit)
   do_zemit(executor, args[0], args[1], flags);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_prompt)
+{
+  int ns = string_prefix(called_as, "NS");
+  int flags = PEMIT_LIST | PEMIT_PROMPT;
+  if (!command_check_byname(executor, ns ? "@nspemit" : "@pemit") ||
+      fun->flags & FN_NOSIDEFX) {
+    safe_str(T(e_perm), buff, bp);
+    return;
+  }
+  orator = executor;
+  if (ns)
+    flags |= PEMIT_SPOOF;
+  do_pemit_list(executor, args[0], args[1], flags);
+}
+
 
 extern signed char qreg_indexes[UCHAR_MAX + 1];
 /* ARGSUSED */
@@ -156,15 +173,25 @@ FUNCTION(fun_setq)
 {
   /* sets a variable into a local register */
   int qindex;
+  int n;
 
-  if (*args[0] && (*(args[0] + 1) == '\0') &&
-      ((qindex = qreg_indexes[(unsigned char) args[0][0]]) != -1)
-      && global_eval_context.renv[qindex]) {
-    strcpy(global_eval_context.renv[qindex], args[1]);
-    if (!strcmp(called_as, "SETR"))
-      safe_strl(args[1], arglens[1], buff, bp);
-  } else
-    safe_str(T("#-1 REGISTER OUT OF RANGE"), buff, bp);
+  if ((nargs % 2) != 0) {
+    safe_format(buff, bp,
+		T("#-1 FUNCTION (%s) EXPECTS AN EVEN NUMBER OF ARGUMENTS"),
+		called_as);
+    return;
+  }
+
+  for (n = 0; n < nargs; n += 2) {
+    if (*args[n] && (*(args[n] + 1) == '\0') &&
+	((qindex = qreg_indexes[(unsigned char) args[n][0]]) != -1)
+	&& global_eval_context.renv[qindex]) {
+      strcpy(global_eval_context.renv[qindex], args[n + 1]);
+      if (n == 0 && !strcmp(called_as, "SETR"))
+	safe_strl(args[n + 1], arglens[n + 1], buff, bp);
+    } else
+      safe_str(T("#-1 REGISTER OUT OF RANGE"), buff, bp);
+  }
 }
 
 /* ARGSUSED */
@@ -422,6 +449,12 @@ FUNCTION(fun_version)
 	      VERSION, PATCHLEVEL, PATCHDATE);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_numversion)
+{
+  safe_integer(NUMVERSION, buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_starttime)
 {
@@ -577,6 +610,8 @@ FUNCTION(fun_list)
     safe_str(list_commands(), buff, bp);
   else if (string_prefix("attribs", args[0]))
     safe_str(list_attribs(), buff, bp);
+  else if (string_prefix("locks", args[0]))
+    list_locks(buff, bp, NULL);
   else if (string_prefix("flags", args[0]))
     safe_str(list_all_flags("FLAG", "", executor, 0x3), buff, bp);
   else if (string_prefix("powers", args[0]))
