@@ -301,12 +301,19 @@ FUNCTION(fun_mid)
   pos = parse_integer(args[1]);
   len = parse_integer(args[2]);
 
-  if ((pos < 0) || (len < 0)) {
+  if (pos < 0) {
     safe_str(T(e_range), buff, bp);
     free_ansi_string(as);
     return;
   }
 
+  if (len < 0) {
+    pos = pos + len + 1;
+    if (pos < 0)
+      pos = 0;
+    len = -len;
+  }
+
   safe_ansi_string(as, pos, len, buff, bp);
   free_ansi_string(as);
 }
@@ -397,8 +404,7 @@ FUNCTION(fun_strinsert)
 FUNCTION(fun_delete)
 {
   ansi_string *as;
-  int pos, num;
-
+  int pos, pos2, num;
 
   if (!is_integer(args[1]) || !is_integer(args[2])) {
     safe_str(T(e_ints), buff, bp);
@@ -415,14 +421,24 @@ FUNCTION(fun_delete)
 
   as = parse_ansi_string(args[0]);
 
-  if ((size_t) pos > as->len || num <= 0) {
+  if ((size_t) pos > as->len || num == 0) {
     safe_strl(args[0], arglens[0], buff, bp);
     free_ansi_string(as);
     return;
   }
 
+  if (num < 0) {
+    pos2 = pos + 1;
+    pos = pos2 + num;
+    if (pos < 0)
+      pos = 0;
+  } else
+    pos2 = pos + num;
+
+
+
   safe_ansi_string(as, 0, pos, buff, bp);
-  safe_ansi_string(as, pos + num, as->len, buff, bp);
+  safe_ansi_string(as, pos2, as->len, buff, bp);
   free_ansi_string(as);
 }
 
@@ -606,28 +622,13 @@ FUNCTION(fun_strcat)
 FUNCTION(fun_flip)
 {
   ansi_string *as;
-  int p, n;
-
   as = parse_ansi_string(args[0]);
-  populate_codes(as);
-
-  for (p = 0, n = as->len - 1; p < n; p++, n--) {
-    char *tcode;
-    char t;
-
-    tcode = as->codes[p];
-    t = as->text[p];
-    as->codes[p] = as->codes[n];
-    as->text[p] = as->text[n];
-    as->codes[n] = tcode;
-    as->text[n] = t;
-  }
-
+  flip_ansi_string(as);
   safe_ansi_string(as, 0, as->len, buff, bp);
-
   free_ansi_string(as);
 }
 
+
 /* ARGSUSED */
 FUNCTION(fun_merge)
 {
@@ -931,10 +932,11 @@ FUNCTION(fun_rjust)
 /* ARGSUSED */
 FUNCTION(fun_center)
 {
-  /* pads a string with leading blanks (or other fill character) */
-
-  size_t width, len, lsp, rsp;
-  char sep;
+  /* pads a string with leading blanks (or other fill string) */
+  size_t width, len, lsp, rsp, filllen;
+  int fillq, fillr, i;
+  char fillstr[BUFFER_LEN], *fp;
+  ansi_string *as;
 
   if (!is_uinteger(args[1])) {
     safe_str(T(e_uint), buff, bp);
@@ -946,20 +948,75 @@ FUNCTION(fun_center)
     safe_strl(args[0], arglens[0], buff, bp);
     return;
   }
-  rsp = width - len;
-  lsp = rsp / 2;
-  rsp -= lsp;
+  lsp = rsp = (width - len) / 2;
+  rsp += (width - len) % 2;
   if (lsp >= BUFFER_LEN)
-    lsp = BUFFER_LEN - 1;
-  if (rsp >= BUFFER_LEN)
-    rsp = BUFFER_LEN - 1;
+    lsp = rsp = BUFFER_LEN - 1;
 
-  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+  if ((!args[2] || !*args[2]) && (!args[3] || !*args[3])) {
+    /* Fast case for default fill with spaces */
+    safe_fill(' ', lsp, buff, bp);
+    safe_strl(args[0], arglens[0], buff, bp);
+    safe_fill(' ', rsp, buff, bp);
     return;
+  }
 
-  safe_fill(sep, lsp, buff, bp);
+  /* args[2] contains the possibly ansi, multi-char fill string */
+  filllen = ansi_strlen(args[2]);
+  if (!filllen) {
+    safe_str(T("#-1 FILL ARGUMENT MAY NOT BE ZERO-LENGTH"), buff, bp);
+    return;
+  }
+  as = parse_ansi_string(args[2]);
+  fillq = lsp / filllen;
+  fillr = lsp % filllen;
+  fp = fillstr;
+  for (i = 0; i < fillq; i++)
+    safe_ansi_string(as, 0, as->len, fillstr, &fp);
+  safe_ansi_string(as, 0, fillr, fillstr, &fp);
+  *fp = '\0';
+  free_ansi_string(as);
+  safe_str(fillstr, buff, bp);
   safe_strl(args[0], arglens[0], buff, bp);
-  safe_fill(sep, rsp, buff, bp);
+  /* If we have args[3], that's the right-side fill string */
+  if (nargs > 3) {
+    if (args[3] && *args[3]) {
+      filllen = ansi_strlen(args[3]);
+      if (!filllen) {
+	safe_str(T("#-1 FILL ARGUMENT MAY NOT BE ZERO-LENGTH"), buff, bp);
+	return;
+      }
+      as = parse_ansi_string(args[3]);
+      fillq = rsp / filllen;
+      fillr = rsp % filllen;
+      fp = fillstr;
+      for (i = 0; i < fillq; i++)
+	safe_ansi_string(as, 0, as->len, fillstr, &fp);
+      safe_ansi_string(as, 0, fillr, fillstr, &fp);
+      *fp = '\0';
+      free_ansi_string(as);
+      safe_str(fillstr, buff, bp);
+    } else {
+      /* Null args[3], fill right side with spaces */
+      safe_fill(' ', rsp, buff, bp);
+    }
+    return;
+  }
+  /* No args[3], so we flip args[2] */
+  filllen = ansi_strlen(args[2]);
+  as = parse_ansi_string(args[2]);
+  fillq = rsp / filllen;
+  fillr = rsp % filllen;
+  fp = fillstr;
+  for (i = 0; i < fillq; i++)
+    safe_ansi_string(as, 0, as->len, fillstr, &fp);
+  safe_ansi_string(as, 0, fillr, fillstr, &fp);
+  *fp = '\0';
+  free_ansi_string(as);
+  as = parse_ansi_string(fillstr);
+  flip_ansi_string(as);
+  safe_ansi_string(as, 0, as->len, buff, bp);
+  free_ansi_string(as);
 }
 
 /* ARGSUSED */
@@ -1061,6 +1118,16 @@ FUNCTION(fun_foreach)
 
 extern char escaped_chars[UCHAR_MAX + 1];
 
+/* ARGSUSED */
+FUNCTION(fun_decompose)
+{
+  /* This function simply returns a decompose'd version of
+   * the included string, such that
+   * s(decompose(str)) == str, down to the last space, tab,
+   * and newline. Except for ansi. */
+  safe_str(decompose_str(args[0]), buff, bp);
+}
+
 /* ARGSUSED */
 FUNCTION(fun_secure)
 {
@@ -1274,13 +1341,13 @@ FUNCTION(fun_ord)
   char *m;
   size_t len = 0;
   if (!args[0] || !args[0][0]) {
-    safe_str(T("#-1 FUNCTION EXPECTS ONE CHARACTER"), buff, bp);
+    safe_str(T("#-1 FUNCTION (ORD) EXPECTS ONE CHARACTER"), buff, bp);
     return;
   }
   m = remove_markup(args[0], &len);
 
   if (len != 2)			/* len includes trailing nul */
-    safe_str(T("#-1 FUNCTION EXPECTS ONE CHARACTER"), buff, bp);
+    safe_str(T("#-1 FUNCTION (ORD) EXPECTS ONE CHARACTER"), buff, bp);
   else if (isprint((unsigned char) *m))
     safe_integer((unsigned char) *m, buff, bp);
   else
@@ -2004,11 +2071,13 @@ FUNCTION(fun_align)
     return;
   }
   if (nargs >= (ncols + 2)) {
-    if (!args[ncols + 1] || strlen(args[ncols + 1]) != 1) {
+    if (!args[ncols + 1] || strlen(args[ncols + 1]) > 1) {
       safe_str(T("#-1 FILLER MUST BE ONE CHARACTER"), buff, bp);
       return;
     }
-    filler = *(args[ncols + 1]);
+    if (*args[ncols + 1]) {
+      filler = *(args[ncols + 1]);
+    }
   }
   if (nargs >= (ncols + 3)) {
     fieldsep = args[ncols + 2];
@@ -2041,3 +2110,187 @@ FUNCTION(fun_align)
   }
   return;
 }
+
+FUNCTION(fun_speak)
+{
+  ufun_attrib transufun;
+  ufun_attrib nullufun;
+  dbref speaker;
+  char *speaker_str;
+  char *open, *close;
+  int transform = 0, null = 0, say = 0;
+  char *wenv[3];
+  int funccount;
+  int fragment = 0;
+  char *say_string;
+  char *string;
+  char rbuff[BUFFER_LEN];
+
+  speaker = match_thing(executor, args[0]);
+  if (speaker == NOTHING || speaker == AMBIGUOUS) {
+    safe_str(T(e_match), buff, bp);
+    return;
+  }
+  speaker_str = unparse_dbref(speaker);
+
+  if (!args[1] || !*args[1])
+    return;
+
+  string = args[1];
+
+  if (nargs > 2 && *args[2] != '\0' && *args[2] != ' ')
+    say_string = args[2];
+  else
+    say_string = (char *) "says,";
+
+  if (nargs > 3) {
+    if (args[3] != '\0') {
+      /* we have a transform attr */
+      transform = 1;
+      if (!fetch_ufun_attrib(args[3], executor, &transufun, 1)) {
+	safe_str(T(e_atrperm), buff, bp);
+	return;
+      }
+      if (nargs > 4) {
+	if (args[4] != '\0') {
+	  /* we have an attr to use when transform returns an empty string */
+	  null = 1;
+	  if (!fetch_ufun_attrib(args[4], executor, &nullufun, 1)) {
+	    safe_str(T(e_atrperm), buff, bp);
+	    return;
+	  }
+	}
+      }
+    }
+  }
+
+  if (nargs < 6 || args[5] == '\0')
+    open = (char *) "\"";
+  else
+    open = args[5];
+  if (nargs < 7 || args[6] == '\0')
+    close = open;
+  else
+    close = args[6];
+
+  switch (*string) {
+  case ':':
+    safe_str(Name(speaker), buff, bp);
+    string++;
+    if (*string == ' ') {
+      /* semipose it instead */
+      while (*string == ' ')
+	string++;
+    } else
+      safe_chr(' ', buff, bp);
+    break;
+  case ';':
+    string++;
+    safe_str(Name(speaker), buff, bp);
+    if (*string == ' ') {
+      /* pose it instead */
+      safe_chr(' ', buff, bp);
+      while (*string == ' ')
+	string++;
+    }
+    break;
+  case '|':
+    string++;
+    break;
+  case '"':
+    if (CHAT_STRIP_QUOTE)
+      string++;
+  default:
+    say = 1;
+    break;
+  }
+
+  if (!transform || (!say && !strstr(string, open))) {
+    /* nice and easy */
+    if (say)
+      safe_format(buff, bp, "%s %s \"%s\"", Name(speaker), say_string, string);
+    else
+      safe_str(string, buff, bp);
+    return;
+  }
+
+
+  if (say && !strstr(string, close)) {
+    /* the whole string has to be transformed */
+    wenv[0] = string;
+    wenv[1] = speaker_str;
+    wenv[2] = unparse_integer(fragment);
+    if (call_ufun(&transufun, wenv, 3, rbuff, executor, enactor, pe_info)) {
+      return;
+    }
+    if (strlen(rbuff) > 0) {
+      safe_format(buff, bp, "%s %s %s", Name(speaker), say_string, rbuff);
+      return;
+    } else {
+      wenv[0] = speaker_str;
+      wenv[1] = unparse_integer(fragment);
+      if (call_ufun(&nullufun, wenv, 2, rbuff, executor, enactor, pe_info))
+	return;
+      safe_str(rbuff, buff, bp);
+      return;
+    }
+  } else {
+    /* only transform portions of string between open and close */
+    char *speech;
+    int indx;
+    int finished = 0;
+    int delete = 0;
+
+    if (say) {
+      safe_str(Name(speaker), buff, bp);
+      safe_chr(' ', buff, bp);
+      safe_str(say_string, buff, bp);
+      safe_chr(' ', buff, bp);
+    }
+    funccount = pe_info->fun_invocations;
+    while (!finished && ((say && fragment == 0 && (speech = string))
+			 || (speech = strstr(string, open)))) {
+      fragment++;
+      indx = string - speech;
+      if (indx < 0)
+	indx *= -1;
+      if (string != NULL && strlen(string) > 0 && indx > 0)
+	safe_strl(string, indx, buff, bp);
+      if (!say || fragment > 1)
+	speech = speech + strlen(open);	/* move past open char */
+      /* find close-char */
+      string = strstr(speech, close);
+      if (!string || !(string = string + strlen(close))) {
+	/* no close char, or nothing after it; we're at the end! */
+	finished = 1;
+      }
+      delete = (string == NULL ? strlen(speech) : strlen(speech) -
+		(strlen(string) + strlen(close)));
+      speech = chopstr(speech, delete);
+      wenv[0] = speech;
+      wenv[1] = speaker_str;
+      wenv[2] = unparse_integer(fragment);
+      if (call_ufun(&transufun, wenv, 3, rbuff, executor, enactor, pe_info))
+	break;
+      if (*bp == (buff + BUFFER_LEN - 1) &&
+	  pe_info->fun_invocations == funccount)
+	break;
+      funccount = pe_info->fun_invocations;
+      if (strlen(rbuff) == 0) {
+	wenv[0] = speaker_str;
+	wenv[1] = unparse_integer(fragment);
+	if (call_ufun(&nullufun, wenv, 2, rbuff, executor, enactor, pe_info))
+	  break;
+      }
+      if (strlen(rbuff) > 0) {
+	safe_str(rbuff, buff, bp);
+      }
+      if (*bp == (buff + BUFFER_LEN - 1) &&
+	  pe_info->fun_invocations == funccount)
+	break;
+    }
+    if (string != NULL && strlen(string) > 0) {
+      safe_str(string, buff, bp);	/* remaining string (not speech, so not t) */
+    }
+  }
+}
