@@ -647,7 +647,7 @@ do_restart(void)
 	bp = buf;
 	safe_str(atr_value(s), buf, &bp);
 	*bp = '\0';
-	add_player(thing, buf);
+	add_player_alias(thing, buf);
       }
     }
   }
@@ -661,6 +661,12 @@ do_restart(void)
   for (j = 0; j < NUMQ; j++)
     global_eval_context.rnxt[j] = NULL;
 
+  /* Initialize the regexp patterns to nothing */
+  global_eval_context.re_code = NULL;
+  global_eval_context.re_subpatterns = -1;
+  global_eval_context.re_offsets = NULL;
+  global_eval_context.re_from = NULL;
+
   for (thing = 0; thing < db_top; thing++) {
     if (Name(thing) == NULL) {
       if (IsGarbage(thing))
@@ -770,7 +776,9 @@ init_game_postdb(const char *conf)
   do_restart();
 #ifdef HAS_OPENSSL
   /* Set up ssl */
-  if (!ssl_init()) {
+  if (!ssl_init
+      (options.ssl_private_key_file, options.ssl_ca_file,
+       options.ssl_require_client_cert)) {
     fprintf(stderr, "SSL initialization failure\n");
     options.ssl_port = 0;	/* Disable ssl */
   }
@@ -1024,7 +1032,7 @@ passwd_filter(const char *cmd)
 
 /** Attempt to match and execute a command.
  * This function performs some sanity checks and then attempts to
- * run a command. It checks, in order: home, built-in commands,
+ * run a command. It checks, in order: built-in commands,
  * enter aliases, leave aliases, $commands on neighboring objects or
  * the player, $commands on the container, $commands on inventory,
  * exits in the zone master room, $commands on objects in the ZMR,
@@ -1121,21 +1129,10 @@ process_command(dbref player, char *command, dbref cause, int from_port)
 	raw_notify(Owner(player), tprintf("#%d] %s", player, msg));
   }
 
-  /* important home checking comes first! */
-  if (strcmp(command, "home") == 0) {
-    if (!Mobile(player))
-      return;
-    if (Fixed(player))
-      notify(player, T("You can't do that IC!"));
-    else
-      do_move(player, command, MOVE_NORMAL);
-    return;
-  }
-
-
   strcpy(unp, command);
 
   cptr = command_parse(player, cause, command, from_port);
+  strcpy(global_eval_context.ucom, (cptr ? cptr : ""));
   if (cptr) {
     a = 0;
     if (!Gagged(player)) {
@@ -1902,7 +1899,7 @@ linux_uptime(dbref player __attribute__ ((__unused__)))
   {
     struct tm *t;
     t = localtime(&mudtime);
-    strftime(tbuf1, sizeof tbuf1, "%I:%M%p ", t);
+    strftime(tbuf1, sizeof tbuf1, "Server uptime: %I:%M%p ", t);
     nl = tbuf1 + strlen(tbuf1);
   }
   /* System uptime */
@@ -2157,6 +2154,18 @@ do_uptime(dbref player, int mortal)
 		  (options.warn_counter - mudtime) % 60, tbuf1);
   }
 
+  notify_format(player,
+		T
+		("PennMUSH Uptime: %ld days %ld hours %ld minutes %ld seconds"),
+		(mudtime - globals.first_start_time) / 86400,
+		((mudtime - globals.first_start_time) % 86400) / 3600,
+		(((mudtime - globals.first_start_time) % 86400) % 3600) / 60,
+		(((mudtime - globals.first_start_time) % 86400) % 3600) % 60);
+
+  /* Mortals, go no further! */
+  if (!Wizard(player) || mortal)
+    return;
+
   /* Mortals, go no further! */
   if (!Wizard(player) || mortal)
     return;
