@@ -77,6 +77,8 @@ const lock_type Control_Lock = "Control"; /**< Name of control lock */
 const lock_type Dropto_Lock = "Dropto";	  /**< Name of dropto lock */
 const lock_type Destroy_Lock = "Destroy"; /**< Name of destroy lock */
 const lock_type Interact_Lock = "Interact"; /**< Name of interaction lock */
+const lock_type MailForward_Lock = "MailForward"; /**< Name of mailforward lock */
+const lock_type Take_Lock = "Take"; /**< Name of take lock */
 /* Define new lock types here. */
 
 /** Table of lock names and permissions */
@@ -104,10 +106,13 @@ const lock_list lock_types[] = {
   {"Dropto", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   {"Destroy", TRUE_BOOLEXP, GOD, LF_PRIVATE | LF_OWNER, NULL},
   {"Interact", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"MailForward", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Take", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   /* Add new lock types just before this line. */
   {NULL, TRUE_BOOLEXP, GOD, 0, NULL}
 };
 
+
 /** Table of base attributes associated with success and failure of
  * locks. These are the historical ones; we automatically generate
  * such attribute names for those that aren't in this table using
@@ -148,6 +153,55 @@ static lock_list *free_list = NULL;
 static lock_list *next_free_lock(void);
 static void free_lock(lock_list *ll);
 
+/** Return a list of all available locks
+ * \param buff the buffer
+ * \param bp a pointer to the current position in the buffer.
+ */
+void
+list_locks(char *buff, char **bp, const char *name)
+{
+  char rbuff[BUFFER_LEN];
+  char *rp;
+  int first = 1;
+  const lock_list *ptr;
+  rp = rbuff;
+  for (ptr = lock_types; ptr->type != NULL; ptr++) {
+    /* Skip those that don't match */
+    if (name && !string_prefix(ptr->type, name))
+      continue;
+    if (first) {
+      first = 0;
+    } else {
+      safe_chr(' ', rbuff, &rp);
+    }
+    safe_str(ptr->type, rbuff, &rp);
+  }
+  *rp = '\0';
+  /* We strupper it for consistency with the other
+   * @list/foo and list(foo)s. */
+  safe_str(strupper(rbuff), buff, bp);
+}
+
+/** User interface to list locks.
+ * \verbatim
+ * This function implements @list/locks.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg wildcard pattern of flag names to list, or NULL for all.
+ * \param lc if 1, list flags in lowercase.
+ * \param label label to prefix to list.
+ */
+void
+do_list_locks(dbref player, const char *arg, int lc, const char *label)
+{
+  char buff[BUFFER_LEN];
+  char *bp = buff;
+  list_locks(buff, &bp, arg);
+  *bp = '\0';
+  notify_format(player, "%s: %s", label, lc ? strlower(buff) : buff);
+}
+
+
 /** Return a list of lock flag characters.
  * \param ll pointer to a lock.
  * \return string of lock flag characters.
@@ -734,9 +788,12 @@ do_lock(dbref player, const char *name, const char *keyname, lock_type type)
 			unparse_dbref(thing), real_type);
 	if (!IsPlayer(thing))
 	  ModTime(thing) = mudtime;
-      } else
+      } else {
 	notify(player, T("Permission denied."));
-    }
+	free_boolexp(key);
+      }
+    } else
+      free_boolexp(key);
   }
 }
 
@@ -772,6 +829,47 @@ eval_lock(dbref player, dbref thing, lock_type ltype)
   return eval_boolexp(player, b, thing);
 }
 
+/** Evaluate a lock.
+ * Evaluate lock ltype on thing for player, passing dbrefs for %0/%1.
+ * \param player dbref attempting to pass the lock.
+ * \param thing object containing the lock.
+ * \param ltype type of lock to check.
+ * \retval 1 player passes the lock.
+ * \retval 0 player does not pass the lock.
+ */
+int
+eval_lock_with(dbref player, dbref thing, lock_type ltype, dbref env0,
+	       dbref env1)
+{
+  char *myenv[10] = { NULL };
+  char e0[SBUF_LEN], e1[SBUF_LEN], *ep;
+  char *preserves[10];
+  int result;
+
+  if (env0 != NOTHING) {
+    ep = e0;
+    safe_dbref(env0, e0, &ep);
+    *ep = '\0';
+    myenv[0] = e0;
+  }
+
+  if (env1 != NOTHING) {
+    ep = e1;
+    safe_dbref(env1, e1, &ep);
+    *ep = '\0';
+    myenv[1] = e1;
+  }
+
+  save_global_env("eval_lock_save", preserves);
+  restore_global_env("eval_lock", myenv);
+
+  boolexp b = getlock(thing, ltype);
+  log_activity(LA_LOCK, thing, unparse_boolexp(player, b, UB_DBREF));
+  result = eval_boolexp(player, b, thing);
+  restore_global_env("eval_lock_save", preserves);
+  return result;
+}
+
 /** Active a lock's failure attributes.
  * \param player dbref failing to pass the lock.
  * \param thing object containing the lock.
