@@ -10,6 +10,7 @@
 #include "copyrite.h"
 
 #include <string.h>
+#include <ctype.h>
 
 #include "conf.h"
 #include "externs.h"
@@ -50,7 +51,7 @@ void decompile_locks(dbref player, dbref thing, const char *name,
 		     int skipdef, const char *prefix);
 static char *parent_chain(dbref player, dbref thing);
 
-extern PRIV attr_privs[];
+extern PRIV attr_privs_view[];
 
 static void
 look_exits(dbref player, dbref loc, const char *exit_name)
@@ -98,7 +99,7 @@ look_exits(dbref player, dbref loc, const char *exit_name)
       global_eval_context.renv[j][0] = '\0';
     bp = arg;
     DOLIST(thing, Exits(loc)) {
-      if ((!(DarkLegal(thing) || (Dark(loc) && !Light(thing))))
+      if (((Light(loc) || Light(thing)) || !(Dark(loc) || Dark(thing)))
 	  && can_interact(thing, player, INTERACT_SEE)) {
 	if (bp != arg)
 	  safe_chr(' ', arg, &bp);
@@ -126,10 +127,15 @@ look_exits(dbref player, dbref loc, const char *exit_name)
     return;
   }
   /* Scan the room and see if there are any visible exits */
-  if (Dark(loc)) {
+  if (Light(loc)) {
     for (thing = Exits(loc); thing != NOTHING; thing = Next(thing)) {
-      if (!DarkLegal(thing) && Light(thing) &&
-	  can_interact(thing, player, INTERACT_SEE)) {
+      total_count++;
+      if (!Transparented(loc) || Opaque(thing))
+	exit_count++;
+    }
+  } else if (Dark(loc)) {
+    for (thing = Exits(loc); thing != NOTHING; thing = Next(thing)) {
+      if (Light(thing) && can_interact(thing, player, INTERACT_SEE)) {
 	total_count++;
 	if (!Transparented(loc) || Opaque(thing))
 	  exit_count++;
@@ -137,7 +143,8 @@ look_exits(dbref player, dbref loc, const char *exit_name)
     }
   } else {
     for (thing = Exits(loc); thing != NOTHING; thing = Next(thing)) {
-      if (!DarkLegal(thing) && can_interact(thing, player, INTERACT_SEE)) {
+      if ((Light(thing) || !DarkLegal(thing)) &&
+	  can_interact(thing, player, INTERACT_SEE)) {
 	total_count++;
 	if (!Transparented(loc) || Opaque(thing))
 	  exit_count++;
@@ -158,7 +165,7 @@ look_exits(dbref player, dbref loc, const char *exit_name)
   notify_by(loc, player, pbuff);
 
   for (thing = Exits(loc); thing != NOTHING; thing = Next(thing)) {
-    if (!DarkLegal(thing) && (!Dark(loc) || Light(thing))
+    if ((Light(loc) || Light(thing) || (!DarkLegal(thing) && !Dark(loc)))
 	&& can_interact(thing, player, INTERACT_SEE)) {
       strcpy(pbuff, Name(thing));
       if ((p = strchr(pbuff, ';')))
@@ -234,12 +241,14 @@ look_contents(dbref player, dbref loc, const char *contents_name)
   if (a) {
     char *wsave[10], *rsave[NUMQ];
     char *arg, *buff, *bp, *save;
+    char *arg2, *bp2;
     char const *sp;
     int j;
 
     arg = (char *) mush_malloc(BUFFER_LEN, "string");
+    arg2 = (char *) mush_malloc(BUFFER_LEN, "string");
     buff = (char *) mush_malloc(BUFFER_LEN, "string");
-    if (!arg || !buff)
+    if (!arg || !buff || !arg2)
       mush_panic("Unable to allocate memory in look_contents");
     save_global_regs("look_contents", rsave);
     for (j = 0; j < 10; j++) {
@@ -249,15 +258,21 @@ look_contents(dbref player, dbref loc, const char *contents_name)
     for (j = 0; j < NUMQ; j++)
       global_eval_context.renv[j][0] = '\0';
     bp = arg;
+    bp2 = arg2;
     DOLIST(thing, Contents(loc)) {
       if (can_see(player, thing, can_see_loc)) {
 	if (bp != arg)
 	  safe_chr(' ', arg, &bp);
 	safe_dbref(thing, arg, &bp);
+	if (bp2 != arg2)
+	  safe_chr('|', arg2, &bp2);
+	safe_str(unparse_object_myopic(player, thing), arg2, &bp2);
       }
     }
     *bp = '\0';
+    *bp2 = '\0';
     global_eval_context.wenv[0] = arg;
+    global_eval_context.wenv[1] = arg2;
     sp = save = safe_atr_value(a);
     bp = buff;
     process_expression(buff, &bp, &sp, loc, player, player,
@@ -270,6 +285,7 @@ look_contents(dbref player, dbref loc, const char *contents_name)
     }
     restore_global_regs("look_contents", rsave);
     mush_free((Malloc_t) arg, "string");
+    mush_free((Malloc_t) arg2, "string");
     mush_free((Malloc_t) buff, "string");
     return;
   }
@@ -315,7 +331,7 @@ look_helper_veiled(dbref player, dbref thing __attribute__ ((__unused__)),
     return 0;
   if (parent == thing || !GoodObject(parent))
     parent = NOTHING;
-  strcpy(fbuf, privs_to_letters(attr_privs, AL_FLAGS(atr)));
+  strcpy(fbuf, privs_to_letters(attr_privs_view, AL_FLAGS(atr)));
   if (atr_sub_branch(atr))
     strcat(fbuf, "`");
   if (AF_Veiled(atr)) {
@@ -377,7 +393,7 @@ look_helper(dbref player, dbref thing __attribute__ ((__unused__)),
     return 0;
   if (parent == thing || !GoodObject(parent))
     parent = NOTHING;
-  strcpy(fbuf, privs_to_letters(attr_privs, AL_FLAGS(atr)));
+  strcpy(fbuf, privs_to_letters(attr_privs_view, AL_FLAGS(atr)));
   if (atr_sub_branch(atr))
     strcat(fbuf, "`");
   r = safe_atr_value(atr);
@@ -496,7 +512,7 @@ look_room(dbref player, dbref loc, enum look_type style)
       }
     }
     tag("HR");
-    tag_wrap("FONT", "SIZE=+3", unparse_room(player, loc));
+    tag_wrap("FONT", "SIZE=+2", unparse_room(player, loc));
     PEND;
     notify_by(loc, player, pbuff);
   }
@@ -787,7 +803,7 @@ do_examine(dbref player, const char *name, enum exam_type flag, int all,
   }
   if (ok) {
     PUSE;
-    tag_wrap("FONT", "SIZE=+3", object_header(player, thing));
+    tag_wrap("FONT", "SIZE=+2", object_header(player, thing));
     PEND;
     notify(player, pbuff);
     if (FLAGS_ON_EXAMINE)
@@ -966,6 +982,60 @@ void
 do_inventory(dbref player)
 {
   dbref thing;
+  ATTR *a;
+
+  a = atr_get(player, "INVFORMAT");
+  if (a) {
+    char *wsave[10], *rsave[NUMQ];
+    char *arg, *buff, *bp, *save;
+    char *arg2, *bp2;
+    char const *sp;
+    int j;
+
+    arg = (char *) mush_malloc(BUFFER_LEN, "string");
+    arg2 = (char *) mush_malloc(BUFFER_LEN, "string");
+    buff = (char *) mush_malloc(BUFFER_LEN, "string");
+    if (!arg || !buff || !arg2)
+      mush_panic("Unable to allocate memory in do_inventory");
+    save_global_regs("do_inventory", rsave);
+    for (j = 0; j < 10; j++) {
+      wsave[j] = global_eval_context.wenv[j];
+      global_eval_context.wenv[j] = NULL;
+    }
+    for (j = 0; j < NUMQ; j++)
+      global_eval_context.renv[j][0] = '\0';
+    bp = arg;
+    bp2 = arg2;
+    DOLIST(thing, Contents(player)) {
+      if (bp != arg)
+	safe_chr(' ', arg, &bp);
+      safe_dbref(thing, arg, &bp);
+      if (bp2 != arg2)
+	safe_chr('|', arg2, &bp2);
+      safe_str(unparse_object_myopic(player, thing), arg2, &bp2);
+    }
+    *bp = '\0';
+    *bp2 = '\0';
+    global_eval_context.wenv[0] = arg;
+    global_eval_context.wenv[1] = arg2;
+    sp = save = safe_atr_value(a);
+    bp = buff;
+    process_expression(buff, &bp, &sp, player, player, player,
+		       PE_DEFAULT, PT_DEFAULT, NULL);
+    *bp = '\0';
+    free((Malloc_t) save);
+    notify(player, buff);
+    for (j = 0; j < 10; j++) {
+      global_eval_context.wenv[j] = wsave[j];
+    }
+    restore_global_regs("do_inventory", rsave);
+    mush_free((Malloc_t) arg, "string");
+    mush_free((Malloc_t) arg2, "string");
+    mush_free((Malloc_t) buff, "string");
+    return;
+  }
+
+  /* Default if no INVFORMAT */
   if ((thing = Contents(player)) == NOTHING) {
     notify(player, T("You aren't carrying anything."));
   } else {
@@ -974,7 +1044,6 @@ do_inventory(dbref player)
       notify(player, unparse_object_myopic(player, thing));
     }
   }
-
   do_score(player);
 }
 
@@ -1320,6 +1389,223 @@ struct dh_args {
   int skipdef;		/**< Skip default flags on attributes if true */
 };
 
+extern char escaped_chars[UCHAR_MAX + 1];
+
+char *
+decompose_str(char *what)
+{
+  static char value[BUFFER_LEN];
+  char *ptr, *s, *codestart;
+  char ansi_letter;
+  int len;
+  int dospace;
+  int flag_depth, ansi_depth;
+  int digits;
+
+  len = strlen(what);
+  flag_depth = 0;
+  ansi_depth = 0;
+
+  /* Go through the string, escaping characters and
+   * turning every other space into %b. */
+
+  s = value;
+  ptr = what;
+#ifdef NEVER
+  /* Put a \ at the beginning if it won't already be put there,
+   * unless it's a space, which would require %b, %r, or %t anyway */
+  if (!escaped_chars[(unsigned int) *what] && !isspace(*what)) {
+    safe_chr('\\', value, &s);
+  }
+#endif
+  dospace = 1;
+  for (; *ptr; ptr++) {
+    switch (*ptr) {
+    case ' ':
+      if (dospace) {
+	safe_str("%b", value, &s);
+      } else {
+	safe_chr(' ', value, &s);
+      }
+      dospace = !dospace;
+      break;
+    case '\n':
+      dospace = 0;
+      safe_str("%r", value, &s);
+      break;
+    case '\t':
+      dospace = 0;
+      safe_str("%t", value, &s);
+      break;
+    case ESC_CHAR:
+      ptr++;
+      if (!*ptr) {
+	ptr--;
+	break;
+      }
+      /* Check if this is any sort of useful code. */
+      if (*ptr == '[' && *(ptr + 1) && *(ptr + 2)) {
+	codestart = ptr;	/* Save the address of the escape code. */
+	ptr++;
+	digits = 0;		/* Digit count is zero. */
+	/* The following code works in this way:
+	 * 1) If a character is a ;, we are allowed to count 2 more digits
+	 * 2) If the digits count is 3, break out of the "ansi" code.
+	 * 3) If the character is not a number or ;, break out.
+	 * 4) If an 'm' is encountered, the for-loop exits.
+	 * The only non-breaking exit occurs when the code ends with "m".
+	 * Otherwise, we break out with the ptr pointing to the end of
+	 * the invalid code, causing decompose() to ignore it entirely.
+	 */
+	for (; *ptr && (*ptr != 'm'); ptr++) {
+	  if (*ptr == ';') {
+	    if (digits == 0) {	/* No double-;s are allowed. */
+	      digits = 3;
+	      break;
+	    }
+	    digits = 0;
+	  } else if (digits >= 2) {
+	    digits = 3;		/* If we encounter a 3-number code, break out. */
+	    break;
+	  } else if (isdigit(*ptr)) {
+	    digits++;		/* Count the numbers we encounter. */
+	  } else {
+	    digits = 3;
+	    break;
+	  }
+	}
+
+	/* 3 is the break-code. 0 means there's no ANSI at all! */
+	if (!*ptr || digits == 3 || digits == 0) {
+	  break;
+	}
+
+	/* It IS an ansi code! It ends in "m" anyway.
+	 * Set ptr to point to the first digit in the code. We are
+	 * promised at this point that ptr+1 is not NUL. 
+	 */
+	ptr = codestart + 1;
+
+	/* Check if the first part of the code is two-digit (color) */
+	if (*(ptr + 1) >= '0' && *(ptr + 1) <= '7') {
+	  if (flag_depth < ansi_depth) {
+	    safe_str(")]", value, &s);
+	    ansi_depth--;
+	  }
+	} else {		/* ansi "flag", inverse, flash, underline, hilight */
+	  flag_depth = ansi_depth + 1;
+	}
+	/* Check to see if this is an 'ansi-reset' code. */
+	if (*ptr == '0' && *(ptr + 1) == 'm') {
+	  for (; ansi_depth > 0; ansi_depth--) {
+	    safe_str(")]", value, &s);
+	  }
+	  flag_depth = 0;
+	  ptr++;
+	  break;
+	}
+
+	ansi_depth++;
+	safe_str("[ansi(", value, &s);
+	dospace = 1;
+
+	/* code for decompiling ansi */
+	for (; isdigit(*ptr) || *ptr == ';'; ptr++) {
+	  if (*ptr == ';')	/* Yes, it is necessary to do it this way. */
+	    ptr++;
+	  /* Break if there is an 'm' here. */
+	  if (!*ptr || !isdigit(*ptr))
+	    break;
+	  /* Check to see if the code is one character long. */
+	  if (*(ptr + 1) == ';' || *(ptr + 1) == 'm') {
+	    /* ANSI flag */
+	    switch (*ptr) {
+	    case '1':
+	      safe_chr('h', value, &s);
+	      break;
+	    case '4':
+	      safe_chr('u', value, &s);
+	      break;
+	    case '5':
+	      safe_chr('f', value, &s);
+	      break;
+	    case '7':
+	      safe_chr('i', value, &s);
+	      break;
+	    default:		/* Not a valid code. */
+	      break;
+	    }
+	  } else {
+	    if (!*(ptr + 1))
+	      break;		/* Sudden string end or lack of real color code. */
+	    ptr++;
+
+	    /* Check if this could be a real color (starts with 3 or 4) */
+	    if (*(ptr - 1) == '3' || *(ptr - 1) == '4') {
+	      switch (*ptr) {
+	      case '0':
+		ansi_letter = 'x';
+		break;
+	      case '1':
+		ansi_letter = 'r';
+		break;
+	      case '2':
+		ansi_letter = 'g';
+		break;
+	      case '3':
+		ansi_letter = 'y';
+		break;
+	      case '4':
+		ansi_letter = 'b';
+		break;
+	      case '5':
+		ansi_letter = 'm';
+		break;
+	      case '6':
+		ansi_letter = 'c';
+		break;
+	      case '7':
+		ansi_letter = 'w';
+		break;
+	      default:
+		break;		/* Not a valid color. */
+	      }
+	      /* If background color, change the letter to a capital. */
+	      if (*(ptr - 1) == '4')
+		ansi_letter = toupper(ansi_letter);
+	      safe_chr(ansi_letter, value, &s);
+	    }
+	    /* No "else" here: If a two-digit code
+	     * doesn't start with 3 or 4, is isn't ANSI. */
+	  }
+	}
+	safe_chr(',', value, &s);
+      } else {
+	ptr--;
+	/* This shouldn't happen if we only have ansi codes
+	 * So if more dirty things must be added later... */
+      }
+      break;
+    default:
+      if (escaped_chars[(unsigned int) *ptr]) {
+	safe_chr('\\', value, &s);
+      }
+      safe_chr(*ptr, value, &s);
+      dospace = 0;
+    }
+  }
+  /* Now check the last space. */
+  if (*(s - 1) == ' ') {
+    s -= 1;
+    safe_str("%b", value, &s);
+  }
+  for (; ansi_depth > 0; ansi_depth--) {
+    safe_str(")]", value, &s);
+  }
+  *s = '\0';
+  return value;
+}
+
 static int
 decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
 		 dbref parent __attribute__ ((__unused__)),
@@ -1328,6 +1614,8 @@ decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
 {
   struct dh_args *dh = args;
   ATTR *ptr;
+  char *avalue;
+  int avlen;
   char msg[BUFFER_LEN];
   char *bp;
 
@@ -1337,17 +1625,32 @@ decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
   ptr = atr_match(AL_NAME(atr));
   bp = msg;
   safe_str(dh->prefix, msg, &bp);
-  if (ptr && !strcmp(AL_NAME(atr), AL_NAME(ptr)))
-    safe_chr('@', msg, &bp);
-  else {
-    ptr = NULL;			/* To speed later checks */
-    safe_chr('&', msg, &bp);
+
+  avalue = atr_value(atr);
+  avlen = strlen(avalue);
+  /* If avalue includes a %r, a %t, or begins or ends with a %b,
+   * then use @set on the decompose_str'd value instead of &atrname */
+  if (strchr(avalue, '\n') ||
+      strchr(avalue, '\t') || *avalue == ' ' || avalue[avlen - 1] == ' ') {
+    safe_str("@set ", msg, &bp);
+    safe_str(dh->name, msg, &bp);
+    safe_chr('=', msg, &bp);
+    safe_str(AL_NAME(atr), msg, &bp);
+    safe_chr(':', msg, &bp);
+    safe_str(decompose_str(avalue), msg, &bp);
+  } else {
+    if (ptr && !strcmp(AL_NAME(atr), AL_NAME(ptr))) {
+      safe_chr('@', msg, &bp);
+    } else {
+      ptr = NULL;		/* To speed later checks */
+      safe_chr('&', msg, &bp);
+    }
+    safe_str(AL_NAME(atr), msg, &bp);
+    safe_chr(' ', msg, &bp);
+    safe_str(dh->name, msg, &bp);
+    safe_chr('=', msg, &bp);
+    safe_str(avalue, msg, &bp);
   }
-  safe_str(AL_NAME(atr), msg, &bp);
-  safe_chr(' ', msg, &bp);
-  safe_str(dh->name, msg, &bp);
-  safe_chr('=', msg, &bp);
-  safe_str(atr_value(atr), msg, &bp);
   *bp = '\0';
   notify(player, msg);
   /* Now deal with attribute flags, if not FugueEditing */
@@ -1359,9 +1662,9 @@ decompile_helper(dbref player, dbref thing __attribute__ ((__unused__)),
       /* Are we different? If so, do as usual */
       int npmflags = AL_FLAGS(ptr) & (~AF_PREFIXMATCH);
       if (AL_FLAGS(atr) != AL_FLAGS(ptr) && AL_FLAGS(atr) != npmflags)
-	privs = privs_to_string(attr_privs, AL_FLAGS(atr));
+	privs = privs_to_string(attr_privs_view, AL_FLAGS(atr));
     } else {
-      privs = privs_to_string(attr_privs, AL_FLAGS(atr));
+      privs = privs_to_string(attr_privs_view, AL_FLAGS(atr));
     }
     if (privs && *privs)
       notify_format(player, "@set %s/%s=%s", dh->name, AL_NAME(atr), privs);
