@@ -451,7 +451,8 @@ parse_english(const char **name, long *flags)
     } else if (strcasecmp(e, "th")) {
       count = -1;
     }
-  }
+  } else
+    count = -1;
   *mname = ' ';
   if (count < 0) {
     /* An error (like '0th' or '12nd') - this wasn't really a count
@@ -677,10 +678,10 @@ choose_thing(const dbref match_who, const int preferred_type, long flags,
 
   /* If a type is given, and only one thing is of that type, return it */
   if (preferred_type != NOTYPE) {
-    if (Typeof(thing1) == preferred_type) {
-      if (Typeof(thing2) != preferred_type)
+    if (Typeof(thing1) & preferred_type) {
+      if (!(Typeof(thing2) & preferred_type))
 	return thing1;
-    } else if (Typeof(thing2) == preferred_type)
+    } else if (Typeof(thing2) & preferred_type)
       return thing2;
   }
 
