@@ -301,12 +301,15 @@ safe_tel(dbref player, dbref dest, int nomovemsgs)
 int
 can_move(dbref player, const char *direction)
 {
-  if (!strcasecmp(direction, "home") && !Fixed(player))
-    return 1;
-
-  /* otherwise match on exits - don't use GoodObject here! */
-  return (match_result(player, direction, TYPE_EXIT, MAT_ENGLISH | MAT_EXIT) !=
+  int ok;
+  if (!strcasecmp(direction, "home")) {
+    ok = command_check_byname(player, "HOME");
+  } else {
+    /* otherwise match on exits - don't use GoodObject here! */
+    ok = (match_result(player, direction, TYPE_EXIT, MAT_ENGLISH | MAT_EXIT) !=
 	  NOTHING);
+  }
+  return ok;			/* Written like this due to overeager compiler */
 }
 
 static dbref
@@ -355,7 +358,7 @@ void
 do_move(dbref player, const char *direction, enum move_type type)
 {
   dbref exit_m, loc, var_dest;
-  if (!strcasecmp(direction, "home")) {
+  if (!strcasecmp(direction, "home") && can_move(player, "home")) {
     /* send him home */
     /* but steal all his possessions */
     if (!Mobile(player) || !GoodObject(Home(player)) ||
@@ -413,8 +416,13 @@ do_move(dbref player, const char *direction, enum move_type type)
 	case AMBIGUOUS:
 	  var_dest = find_var_dest(player, exit_m);
 	  /* Only allowed if the owner of the exit could link to var_dest */
-	  if (GoodObject(var_dest) && !can_link_to(exit_m, var_dest))
-	    var_dest = NOTHING;
+	  if (!GoodObject(var_dest) || !can_link_to(exit_m, var_dest)) {
+	    notify_format(player,
+			  T
+			  ("Variable exit destination #%d is invalid or not permitted."),
+			  var_dest);
+	    return;
+	  }
 	  break;
 	default:
 	  var_dest = Location(exit_m);
@@ -542,7 +550,7 @@ do_get(dbref player, const char *what)
 	return;
       }
       /* to steal something, you have to be able to get it, and the
-       * object must be ENTER_OK and not locked against you.
+       * object must be ENTER_OK and not take-locked against you.
        */
       if (could_doit(player, thing) &&
 	  (POSSGET_ON_DISCONNECTED ||
@@ -550,7 +558,7 @@ do_get(dbref player, const char *what)
 	    Connected(Location(thing)))) &&
 	  (controls(player, thing) ||
 	   (EnterOk(Location(thing)) &&
-	    eval_lock(player, Location(thing), Enter_Lock)))) {
+	    eval_lock(player, Location(thing), Take_Lock)))) {
 	notify_format(Location(thing),
 		      T("%s was taken from you."), Name(thing));
 	notify_format(thing, T("%s took you."), Name(player));
@@ -596,6 +604,11 @@ do_get(dbref player, const char *what)
 	  notify(player, T("You cannot get yourself!"));
 	  return;
 	}
+	if (!eval_lock(player, Location(thing), Take_Lock)) {
+	  fail_lock(player, Location(thing), Take_Lock,
+		    T("You can't take that from there."), NOTHING);
+	  return;
+	}
 	if (could_doit(player, thing)) {
 	  moveto(thing, player);
 	  notify_format(thing, T("%s took you."), Name(player));
@@ -917,10 +930,6 @@ move_wrapper(dbref player, const char *command)
 {
   if (!Mobile(player))
     return;
-  if (!strcasecmp(command, "home") && Fixed(player)) {
-    notify(player, T("You can't do that IC!"));
-    return;
-  }
   if (can_move(player, command))
     do_move(player, command, MOVE_NORMAL);
   else if ((Zone(Location(player)) != NOTHING) && remote_exit(player, command))
