@@ -109,7 +109,7 @@ static SSL_CTX *ctx = NULL;
  * \return pointer to SSL context object.
  */
 SSL_CTX *
-ssl_init(void)
+ssl_init(char *private_key_file, char *ca_file, int req_client_cert)
 {
   SSL_METHOD *meth;
   unsigned char context[128];
@@ -134,24 +134,23 @@ ssl_init(void)
   ctx = SSL_CTX_new(meth);
 
   /* Load keys/certs */
-  if (*options.ssl_private_key_file) {
-    if (!SSL_CTX_use_certificate_chain_file(ctx, options.ssl_private_key_file)) {
+  if (private_key_file && *private_key_file) {
+    if (!SSL_CTX_use_certificate_chain_file(ctx, private_key_file)) {
       ssl_errordump
 	("Unable to load server certificate - only anonymous ciphers supported.");
     }
-    if (!SSL_CTX_use_PrivateKey_file
-	(ctx, options.ssl_private_key_file, SSL_FILETYPE_PEM)) {
+    if (!SSL_CTX_use_PrivateKey_file(ctx, private_key_file, SSL_FILETYPE_PEM)) {
       ssl_errordump
 	("Unable to load private key - only anonymous ciphers supported.");
     }
   }
 
   /* Load trusted CAs */
-  if (*options.ssl_ca_file) {
-    if (!SSL_CTX_load_verify_locations(ctx, options.ssl_ca_file, NULL)) {
+  if (ca_file && *ca_file) {
+    if (!SSL_CTX_load_verify_locations(ctx, ca_file, NULL)) {
       ssl_errordump("Unable to load CA certificates");
     } else {
-      if (options.ssl_require_client_cert)
+      if (req_client_cert)
 	SSL_CTX_set_verify(ctx,
 			   SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT,
 			   client_verify_callback);
@@ -177,7 +176,7 @@ ssl_init(void)
   SSL_CTX_set_cipher_list(ctx, "ALL:ADH:RC4+RSA:+SSLv2:@STRENGTH");
 
   /* Set up session cache if we can */
-  strncpy(context, MUDNAME, 128);
+  strncpy((char *) context, MUDNAME, 128);
   SSL_CTX_set_session_id_context(ctx, context, u_strlen(context));
 
   /* Load hash algorithms */
