@@ -90,16 +90,6 @@ extern CHAN *channels;
 
 static int under_limit = 1;
 
-/** Default connection, nothing special */
-#define CONN_DEFAULT 0
-/** Using Pueblo, Smial, Mushclient, Simplemu, or some other
- *  pueblo-style HTML aware client */
-#define CONN_HTML 0x1
-/** Using a client that understands telnet options */
-#define CONN_TELNET 0x2
-/** Send a telnet option to test client */
-#define CONN_TELNET_QUERY 0x4
-
 
 /* When the mush gets a new connection, it tries sending a telnet
  * option negotiation code for setting client-side line-editing mode
@@ -123,6 +113,7 @@ static int under_limit = 1;
 
 /* Telnet codes */
 #define IAC 255			/**< telnet: interpret as command */
+#define GOAHEAD 249		/**< telnet: go-ahead */
 
 /** Iterate through a list of descriptors, and do something with those
  * that are connected.
@@ -131,25 +122,6 @@ static int under_limit = 1;
         for(d = descriptor_list;(d);d=(d)->next) \
           if((d)->connected)
 
-#ifdef NT_TCP
-/* for Windows NT IO-completion-port method of TCP/IP - NJG */
-
-/* Windows NT TCP/IP routines written by Nick Gammon <nick@gammon.com.au> */
-
-#include <process.h>
-HANDLE CompletionPort;		/* IOs are queued up on this port */
-SOCKET MUDListenSocket;		/* for our listening thread */
-DWORD dwMUDListenThread;	/* thread handle for listening thread */
-SOCKADDR_IN saServer;		/* for listening thread */
-void __cdecl MUDListenThread(void *pVoid);	/* the listening thread */
-DWORD platform;			/* which version of Windows are we using? */
-OVERLAPPED lpo_aborted;		/* special to indicate a player has finished TCP IOs */
-OVERLAPPED lpo_shutdown;	/* special to indicate a player should do a shutdown */
-void ProcessWindowsTCP(void);	/* handle NT-style IOs */
-CRITICAL_SECTION cs;		/* for thread synchronisation */
-#endif
-
-
 static const char *flushed_message = "\r\n<Output Flushed>\x1B[0m\r\n";
 
 extern DESC *descriptor_list;
@@ -195,6 +167,7 @@ enum na_type {
 #define TA_BLINK 4	/**< Text attribute blinking/flashing */
 #define TA_ULINE 5	/**< Text attribute underline */
 
+
 static int na_depth = 0;
 
 /** A place to store a rendered message. */
@@ -212,7 +185,7 @@ static void free_strings(struct notify_strings messages[]);
 static void zero_strings(struct notify_strings messages[]);
 static unsigned char *notify_makestring(const char *message,
 					struct notify_strings messages[],
-					enum na_type type);
+					enum na_type type, int flags);
 
 
 static void
@@ -339,7 +312,7 @@ free_strings(struct notify_strings messages[])
 
 static unsigned char *
 notify_makestring(const char *message, struct notify_strings messages[],
-		  enum na_type type)
+		  enum na_type type, int flags)
 {
   char *o;
   const unsigned char *p;
@@ -351,9 +324,19 @@ notify_makestring(const char *message, struct notify_strings messages[],
   int strip = 0;
   int pueblo = 0;
   static char tbuf[BUFFER_LEN];
+  char *bp;
 
-  if (messages[type].made)
-    return messages[type].message;
+  if (messages[type].made) {
+    if (flags & NA_PROMPT) {
+      bp = tbuf;
+      safe_str((char *) messages[type].message, tbuf, &bp);
+      safe_chr(IAC, tbuf, &bp);
+      safe_chr(GOAHEAD, tbuf, &bp);
+      *bp = '\0';
+      return (unsigned char *) tbuf;
+    } else
+      return messages[type].message;
+  }
   messages[type].made = 1;
 
   p = (unsigned char *) message;
@@ -384,6 +367,14 @@ notify_makestring(const char *message, struct notify_strings messages[],
     *o = '\0';
     messages[type].message = (unsigned char *) mush_strdup(tbuf, "string");
     messages[type].len = o - tbuf;
+    if (flags & NA_PROMPT) {
+      bp = tbuf;
+      safe_str((char *) messages[type].message, tbuf, &bp);
+      safe_chr(IAC, tbuf, &bp);
+      safe_chr(GOAHEAD, tbuf, &bp);
+      *bp = '\0';
+      return (unsigned char *) tbuf;
+    }
     return messages[type].message;
   case NA_NPASCII:
     strip = 1;
@@ -426,6 +417,14 @@ notify_makestring(const char *message, struct notify_strings messages[],
     *o = '\0';
     messages[type].message = (unsigned char *) mush_strdup(tbuf, "string");
     messages[type].len = o - tbuf;
+    if (flags & NA_PROMPT) {
+      bp = tbuf;
+      safe_str((char *) messages[type].message, tbuf, &bp);
+      safe_chr(IAC, tbuf, &bp);
+      safe_chr(GOAHEAD, tbuf, &bp);
+      *bp = '\0';
+      return (unsigned char *) tbuf;
+    }
     return messages[type].message;
 
   case NA_PUEBLO:
@@ -530,6 +529,14 @@ notify_makestring(const char *message, struct notify_strings messages[],
   *o = '\0';
   messages[type].message = (unsigned char *) mush_strdup(tbuf, "string");
   messages[type].len = o - tbuf;
+  if (flags & NA_PROMPT) {
+    bp = tbuf;
+    safe_str((char *) messages[type].message, tbuf, &bp);
+    safe_chr(IAC, tbuf, &bp);
+    safe_chr(GOAHEAD, tbuf, &bp);
+    *bp = '\0';
+    return (unsigned char *) tbuf;
+  }
   return messages[type].message;
 }
 
@@ -800,6 +807,9 @@ notify_anything_loc(dbref speaker, na_lookup func,
 	if (pass_interact && (flags & NA_INTER_HEAR) &&
 	    !can_interact(speaker, target, INTERACT_HEAR))
 	  pass_interact = 0;
+	if (pass_interact && (flags & NA_INTER_LOCK) &&
+	    !Pass_Interact_Lock(speaker, target))
+	  pass_interact = 0;
 	if (!pass_interact)
 	  continue;
       }
@@ -819,25 +829,28 @@ notify_anything_loc(dbref speaker, na_lookup func,
 		paranoid = nsfunc(speaker, func, fdata, 1);
 		havepara = 1;
 	      }
-	      pstring = notify_makestring(paranoid, paranoids, poutput);
-	      plen = paranoids[poutput].len;
+	      pstring = notify_makestring(paranoid, paranoids, poutput, flags);
+	      plen = u_strlen(pstring);
 	    } else {
 	      if (!havespoof) {
 		nospoof = nsfunc(speaker, func, fdata, 0);
 		havespoof = 1;
 	      }
-	      pstring = notify_makestring(nospoof, nospoofs, poutput);
-	      plen = nospoofs[poutput].len;
+	      pstring = notify_makestring(nospoof, nospoofs, poutput, flags);
+	      plen = u_strlen(pstring);
 	    }
 	    queue_newwrite(d, pstring, plen);
 	  }
 
-	  pstring = notify_makestring(msgbuf, messages, poutput);
-	  plen = messages[poutput].len;
+	  pstring = notify_makestring(msgbuf, messages, poutput, flags);
+	  plen = u_strlen(pstring);
 	  if (pstring && *pstring)
 	    queue_newwrite(d, pstring, plen);
 
-	  if (!(flags & NA_NOENTER)) {
+	  if (!((flags & NA_NOENTER) || ((flags & NA_PROMPT)
+					 && !(d->
+					      conn_flags &
+					      CONN_PROMPT_NEWLINES)))) {
 	    if ((poutput == NA_PUEBLO) || (poutput == NA_NPUEBLO)) {
 	      if (flags & NA_NOPENTER)
 		queue_newwrite(d, (unsigned char *) "\n", 1);
@@ -878,6 +891,7 @@ notify_anything_loc(dbref speaker, na_lookup func,
       safe_str(Name(target), tbuf1, &bp);
       safe_str("> ", tbuf1, &bp);
       *bp = '\0';
+      flags &= ~NA_PROMPT;	/* No prompts to puppets */
       notify_anything(GOD, na_one, &Owner(target), NULL,
 		      NA_NOENTER | NA_PUPPET2 | NA_NORELAY | flags, tbuf1);
 
@@ -906,10 +920,12 @@ notify_anything_loc(dbref speaker, na_lookup func,
       if (AF_Regexp(a)
 	  ? regexp_match_case(tbuf1,
 			      (char *) notify_makestring(msgbuf, messages,
-							 NA_ASCII), AF_Case(a))
+							 NA_ASCII, 0),
+			      AF_Case(a))
 	  : wild_match_case(tbuf1,
 			    (char *) notify_makestring(msgbuf, messages,
-						       NA_ASCII), AF_Case(a))) {
+						       NA_ASCII, 0),
+			    AF_Case(a))) {
 	if (eval_lock(speaker, target, Listen_Lock))
 	  if (PLAYER_AHEAR || (!IsPlayer(target))) {
 	    if (speaker != target)
@@ -921,7 +937,7 @@ notify_anything_loc(dbref speaker, na_lookup func,
 	if (!(flags & NA_NORELAY) && (loc != target) &&
 	    !filter_found(target,
 			  (char *) notify_makestring(msgbuf, messages,
-						     NA_ASCII), 1)) {
+						     NA_ASCII, 0), 1)) {
 	  passalong[0] = target;
 	  passalong[1] = target;
 	  passalong[2] = Owner(target);
@@ -954,16 +970,15 @@ notify_anything_loc(dbref speaker, na_lookup func,
       }
     }
     /* if object is flagged LISTENER, check for ^ listen patterns
-     *    * these are like AHEAR - object cannot trigger itself.
      *    * unlike normal @listen, don't pass the message on.
      *    */
 
-    if ((speaker != target) && (ThingListen(target) || RoomListen(target))
+    if ((ThingListen(target) || RoomListen(target))
 	&& eval_lock(speaker, target, Listen_Lock)
       )
       atr_comm_match(target, speaker, '^', ':',
-		     (char *) notify_makestring(msgbuf, messages, NA_ASCII), 0,
-		     NULL, NULL, NULL);
+		     (char *) notify_makestring(msgbuf, messages, NA_ASCII, 0),
+		     0, NULL, NULL, NULL);
 
     /* If object is flagged AUDIBLE and has a @FORWARDLIST, send
      *  stuff on */
@@ -1426,10 +1441,10 @@ queue_write(DESC *d, const unsigned char *b, int n)
     PUSE;
     tag_wrap("SAMP", NULL, buff);
     PEND;
-    s = notify_makestring(pbuff, messages, NA_PUEBLO);
+    s = notify_makestring(pbuff, messages, NA_PUEBLO, 0);
     len = messages[NA_PUEBLO].len;
   } else {
-    s = notify_makestring(buff, messages, notify_type(d));
+    s = notify_makestring(buff, messages, notify_type(d), 0);
     len = messages[notify_type(d)].len;
   }
   queue_newwrite(d, s, len);
@@ -1450,82 +1465,6 @@ queue_newwrite(DESC *d, const unsigned char *b, int n)
 {
   int space;
 
-#ifdef NT_TCP
-
-  /* queue up an asynchronous write if using Windows NT */
-  if (platform == VER_PLATFORM_WIN32_NT) {
-
-    struct text_block **qp, *cur;
-    DWORD nBytes;
-    BOOL bResult;
-
-    /* don't write if connection dropped */
-    if (d->bConnectionDropped)
-      return n;
-
-    /* add to queue - this makes sure output arrives in the right order */
-    add_to_queue(&d->output, b, n);
-    d->output_size += n;
-
-    /* if we are already writing, exit and wait for IO completion */
-    if (d->bWritePending)
-      return n;
-
-    /* pull head item out of queue - not necessarily the one we just put in */
-    qp = &d->output.head;
-
-    if ((cur = *qp) == NULL)
-      return n;			/* nothing in queue - rather strange, but oh well. */
-
-
-    /* if buffer too long, write what we can and queue up the rest */
-
-    if (cur->nchars <= sizeof(d->output_buffer)) {
-      nBytes = cur->nchars;
-      memcpy(d->output_buffer, cur->start, nBytes);
-      if (!cur->nxt)
-	d->output.tail = qp;
-      *qp = cur->nxt;
-      free_text_block(cur);
-    }
-    /* end of able to write the lot */
-    else {
-      nBytes = sizeof(d->output_buffer);
-      memcpy(d->output_buffer, cur->start, nBytes);
-      cur->nchars -= nBytes;
-      cur->start += nBytes;
-    }				/* end of buffer too long */
-
-    d->OutboundOverlapped.Offset = 0;
-    d->OutboundOverlapped.OffsetHigh = 0;
-
-    bResult = WriteFile((HANDLE) d->descriptor,
-			d->output_buffer, nBytes, NULL, &d->OutboundOverlapped);
-
-    d->bWritePending = FALSE;
-
-    if (!bResult)
-      if (GetLastError() == ERROR_IO_PENDING)
-	d->bWritePending = TRUE;
-      else {
-	d->bConnectionDropped = TRUE;	/* do no more writes */
-	/* post a notification that the descriptor should be shutdown */
-	if (!PostQueuedCompletionStatus
-	    (CompletionPort, 0, (DWORD) d, &lpo_shutdown)) {
-	  char sMessage[200];
-	  DWORD nError = GetLastError();
-	  GetErrorMessage(nError, sMessage, sizeof sMessage);
-	  printf
-	    ("Error %ld (%s) on PostQueuedCompletionStatus in queue_newwrite\n",
-	     nError, sMessage);
-	  boot_desc(d);
-	}
-      }				/* end of had write */
-    return n;
-
-  }				/* end of NT TCP/IP way of doing it */
-#endif
-
   space = MAX_OUTPUT - d->output_size - n;
   if (space < SPILLOVER_THRESHOLD) {
     process_output(d);
@@ -1592,7 +1531,7 @@ queue_string(DESC *d, const char *s)
 
   poutput = notify_type(d);
 
-  n = notify_makestring(s, messages, poutput);
+  n = notify_makestring(s, messages, poutput, 0);
   ret = queue_newwrite(d, n, messages[poutput].len);
   free_strings(messages);
   return ret;
