@@ -35,11 +35,12 @@
 
 extern char *absp[], *obj[], *poss[], *subj[];	/* fundb.c */
 extern int inum, inum_limit;
+extern char *iter_rep[];
 int global_fun_invocations;
 int global_fun_recursions;
-extern int re_subpatterns;
-extern int *re_offsets;
-extern char *re_from;
+/* extern int re_subpatterns; */
+/* extern int *re_offsets; */
+/* extern char *re_from; */
 extern sig_atomic_t cpu_time_limit_hit;
 extern int cpu_limit_warning_sent;
 
@@ -454,6 +455,7 @@ process_expression(char *buff, char **bp, char const **str,
   char *debugstr = NULL, *sourcestr = NULL;
   char *realbuff = NULL, *realbp = NULL;
   int gender = -1;
+  int inum_this;
   char *startpos = *bp;
   int had_space = 0;
   char temp[3];
@@ -500,6 +502,7 @@ process_expression(char *buff, char **bp, char const **str,
     pe_info->nest_depth = 0;
     pe_info->call_depth = 0;
     pe_info->debug_strings = NULL;
+    pe_info->arg_count = 0;
   } else {
     old_iter_limit = -1;
   }
@@ -641,32 +644,55 @@ process_expression(char *buff, char **bp, char const **str,
       }
       break;
     case '$':			/* Dollar subs for regedit() */
-      if ((eflags & (PE_DOLLAR | PE_EVALUATE)) != (PE_DOLLAR | PE_EVALUATE)) {
-	safe_chr('$', buff, bp);
-	(*str)++;
-      } else {
+      if ((eflags & (PE_DOLLAR | PE_EVALUATE)) == (PE_DOLLAR | PE_EVALUATE) &&
+	  global_eval_context.re_subpatterns >= 0 &&
+	  global_eval_context.re_offsets != NULL &&
+	  global_eval_context.re_from != NULL) {
 	char obuf[BUFFER_LEN];
-	int p = 0;
+	int p = -1;
+	char subspace[BUFFER_LEN];
+	char *named_substring = NULL;
 
+	obuf[0] = '\0';
 	(*str)++;
-	/* Check the first two characters after the $ for a number */
+	/* Check the first character after the $ for a number */
 	if (isdigit((unsigned char) **str)) {
 	  p = **str - '0';
 	  (*str)++;
-	  if (isdigit((unsigned char) **str)) {
-	    p *= 10;
-	    p += **str - '0';
+	} else if (**str == '<') {
+	  /* Look for a named or numbered subexpression */
+	  char *nbuf = subspace;
+	  (*str)++;
+	  for (; *str && **str && **str != '>'; (*str)++)
+	    safe_chr(**str, subspace, &nbuf);
+	  *nbuf = '\0';
+	  if (*str && **str)
 	    (*str)++;
-	  }
-	} else
+	  if (is_strict_integer(subspace))
+	    p = abs(parse_integer(subspace));
+	  else
+	    named_substring = subspace;
+	} else {
+	  safe_chr('$', buff, bp);
 	  break;
+	}
 
-	if (p >= re_subpatterns || re_offsets == NULL || re_from == NULL)
-	  break;
-
-	pcre_copy_substring(re_from, re_offsets, re_subpatterns, p, obuf,
-			    BUFFER_LEN);
+	if (named_substring) {
+	  pcre_copy_named_substring(global_eval_context.re_code,
+				    global_eval_context.re_from,
+				    global_eval_context.re_offsets,
+				    global_eval_context.re_subpatterns,
+				    named_substring, obuf, BUFFER_LEN);
+	} else {
+	  pcre_copy_substring(global_eval_context.re_from,
+			      global_eval_context.re_offsets,
+			      global_eval_context.re_subpatterns,
+			      p, obuf, BUFFER_LEN);
+	}
 	safe_str(obuf, buff, bp);
+      } else {
+	safe_chr('$', buff, bp);
+	(*str)++;
       }
       break;
     case '%':			/* Percent substitutions */
@@ -705,8 +731,11 @@ process_expression(char *buff, char **bp, char const **str,
 
 	(*str)++;
 	savec = **str;
-	if (!savec)
+	if (!savec) {
+	  /* Line ended in %, so treat it as literal */
+	  safe_chr('%', buff, bp);
 	  goto exit_sequence;
+	}
 	savepos = *bp;
 	(*str)++;
 
@@ -714,6 +743,9 @@ process_expression(char *buff, char **bp, char const **str,
 	case '%':		/* %% - a real % */
 	  safe_chr('%', buff, bp);
 	  break;
+	case ' ':		/* "% " for more natural typing */
+	  safe_str("% ", buff, bp);
+	  break;
 	case '!':		/* executor dbref */
 	  safe_dbref(executor, buff, bp);
 	  break;
@@ -740,6 +772,12 @@ process_expression(char *buff, char **bp, char const **str,
 	case '~':		/* enactor accented name */
 	  safe_str(accented_name(enactor), buff, bp);
 	  break;
+	case '+':		/* argument count */
+	  if (pe_info)
+	    safe_integer(pe_info->arg_count, buff, bp);
+	  else
+	    safe_integer(0, buff, bp);
+	  break;
 	case '0':
 	case '1':
 	case '2':
@@ -767,6 +805,28 @@ process_expression(char *buff, char **bp, char const **str,
 	case 'c':		/* command line */
 	  safe_str(global_eval_context.ccom, buff, bp);
 	  break;
+	case 'I':
+	case 'i':
+	  nextc = **str;
+	  if (!nextc)
+	    goto exit_sequence;
+	  (*str)++;
+	  if (!isdigit(nextc)) {
+	    safe_str(T(e_int), buff, bp);
+	    break;
+	  }
+	  inum_this = nextc - '0';
+	  if (inum_this < 0 || inum_this >= inum
+	      || (inum - inum_this) <= inum_limit) {
+	    safe_str(T("#-1 ARGUMENT OUT OF RANGE"), buff, bp);
+	  } else {
+	    safe_str(iter_rep[inum - inum_this], buff, bp);
+	  }
+	  break;
+	case 'U':
+	case 'u':
+	  safe_str(global_eval_context.ucom, buff, bp);
+	  break;
 	case 'L':
 	case 'l':		/* enactor location dbref */
 	  /* The security implications of this have
@@ -1158,9 +1218,15 @@ process_expression(char *buff, char **bp, char const **str,
 		safe_chr('/', buff, bp);
 		safe_str(userfn_tab[fp->where.offset].name, buff, bp);
 		safe_chr(')', buff, bp);
-	      } else
+	      } else {
+		char *preserve[NUMQ];
+		if (fp->flags & FN_LOCALIZE)
+		  save_global_regs("@function.save", preserve);
 		do_userfn(buff, bp, thing, attrib, nfargs, fargs,
 			  executor, caller, enactor, pe_info);
+		if (fp->flags & FN_LOCALIZE)
+		  restore_global_regs("@function.save", preserve);
+	      }
 	    }
 	    pe_info->fun_depth--;
 	    global_fun_recursions--;
