@@ -47,7 +47,7 @@ static int grep_helper(dbref player, dbref thing, dbref parent,
 		       char const *pattern, ATTR *atr, void *args);
 void do_grep(dbref player, char *obj, char *lookfor, int flag, int insensitive);
 static int pay_quota(dbref, int);
-extern PRIV attr_privs[];
+extern PRIV attr_privs_view[];
 
 /** A generic function to generate a formatted string. The
  * return value is a statically allocated buffer.
@@ -406,13 +406,17 @@ can_see(dbref player, dbref thing, int can_see_loc)
       (IsPlayer(thing) && !Connected(thing)))
     return 0;
 
-  /* if the room is lit, you can see any non-dark objects */
+  /* if thing is in a room set LIGHT, it can be seen */
+  else if (IS(Location(thing), TYPE_ROOM, "LIGHT"))
+    return 1;
+
+  /* if the room is non-dark, you can see objects which are light or non-dark */
   else if (can_see_loc)
-    return (!DarkLegal(thing));
+    return (Light(thing) || !DarkLegal(thing));
 
   /* otherwise room is dark and you can only see lit things */
   else
-    return (Light(thing) && !DarkLegal(thing));
+    return (Light(thing));
 }
 
 /** Can a player control a thing?
@@ -535,7 +539,39 @@ int
 payfor(dbref who, int cost)
 {
   /* subtract cost from who's pennies */
+  int tmp;
+  dbref owner;
+  if ((cost == 0) || NoPay(who))
+    return 1;
+  owner = Owner(who);
+  if ((tmp = Pennies(owner)) >= cost) {
+    if (Track_Money(owner)) {
+      notify_format(owner, T("GAME: %s(%s) spent %d %s."),
+		    Name(who), unparse_dbref(who), cost,
+		    (cost == 1) ? MONEY : MONIES);
+    }
+    s_Pennies(owner, tmp - cost);
+    return 1;
+  } else {
+    if (Track_Money(owner)) {
+      notify_format(owner, T("GAME: %s(%s) tried to spend %d %s."),
+		    Name(who), unparse_dbref(who), cost,
+		    (cost == 1) ? MONEY : MONIES);
+    }
+    return 0;
+  }
+}
 
+/** Debit a player's pennies, if they can afford it.
+ * \param who player to debit.
+ * \param cost number of pennies to debit.
+ * \retval 1 player successfully debited.
+ * \retval 0 player can't afford the cost.
+ */
+int
+quiet_payfor(dbref who, int cost)
+{
+  /* subtract cost from who's pennies */
   int tmp;
   if (NoPay(who))
     return 1;
@@ -706,25 +742,25 @@ ok_name(const char *n)
 	  && strcasecmp((char *) name, "here"));
 }
 
-/** Is a name a valid player name?
+/** Is a name a valid player name when applied by player to thing?
  * Player names must be valid object names, but also not forbidden (unless
  * the player is a wizard). They are
  * subject to a different length limit, and subject to more stringent
  * restrictions on valid characters. Finally, it can't be the same as
- * an existing player name or alias.
+ * an existing player name or alias unless it's one of theirs.
  * \param name name to check.
  * \param player player for permission checks.
+ * \param thing player who will get the name.
  * \retval 1 name is valid for players.
  * \retval 0 name is not valid for players.
  */
 int
-ok_player_name(const char *name, dbref player)
+ok_player_name(const char *name, dbref player, dbref thing)
 {
   const unsigned char *scan, *good;
+  dbref lookup;
 
-  if (!ok_name(name)
-      || (forbidden_name(name) && !(GoodObject(player) && Wizard(player)))
-      || strlen(name) >= (size_t) PLAYER_NAME_LIMIT)
+  if (!ok_name(name) || strlen(name) >= (size_t) PLAYER_NAME_LIMIT)
     return 0;
 
   good = (unsigned char *) (PLAYER_NAME_SPACES ? " `$_-.,'" : "`$_-.,'");
@@ -737,9 +773,57 @@ ok_player_name(const char *name, dbref player)
       return 0;
   }
 
-  return (lookup_player(name) == NOTHING);
+  lookup = lookup_player(name);
+
+  /* A player may only change to a forbidden name if they're already
+     using that name. */
+  if (forbidden_name(name) && !((lookup == thing) ||
+				(GoodObject(player) && Wizard(player))))
+    return 0;
+
+  return ((lookup == NOTHING) || (lookup == thing));
 }
 
+
+/** Is a alias a valid player alias-list for thing?
+ * It must be a semicolon-separated list of valid player names
+ * with no more than than MAX_ALIASES names, if the player isn't
+ * a wizard.
+ * \param alias list to check.
+ * \param player player for permission checks.
+ * \param thing player who is being aliased.
+ * \return One of the OPAE_* constants defined in hdrs/attrib.h
+ */
+int
+ok_player_alias(const char *alias, dbref player, dbref thing)
+{
+  char tbuf1[BUFFER_LEN], *s, *sp;
+  int cnt = 0;
+
+  if (!alias || !*alias)
+    return OPAE_NULL;
+
+  strncpy(tbuf1, alias, BUFFER_LEN - 1);
+  tbuf1[BUFFER_LEN - 1] = '\0';
+  s = trim_space_sep(tbuf1, ALIAS_DELIMITER);
+  while (s) {
+    sp = split_token(&s, ALIAS_DELIMITER);
+    while (sp && *sp && *sp == ' ')
+      sp++;
+    if (!sp || !*sp)
+      return OPAE_NULL;		/* No null aliases */
+    if (!ok_player_name(sp, player, thing))
+      return OPAE_INVALID;
+    cnt++;
+  }
+  if (Wizard(player))
+    return OPAE_SUCCESS;
+  if (cnt > MAX_ALIASES)
+    return OPAE_TOOMANY;
+  return OPAE_SUCCESS;
+}
+
+
 /** Is a password acceptable?
  * Acceptable passwords must be non-null and must contain only
  * printable characters and no whitespace.
@@ -763,9 +847,10 @@ ok_password(const char *password)
   return 1;
 }
 
-/** Is a name ok for a command or function?
- * It must begin with an uppercase alpha, and contain only
- * uppercase alpha, numbers, or underscore thereafter.
+/** Is a name ok for a command?
+ * It must contain only uppercase alpha, numbers, or punctuation.
+ * It must contain at least one uppercase alpha.
+ * It may not begin with " : ; & ] \ and # (the special tokens).
  * \param name name to check.
  * \retval 1 name is acceptable.
  * \retval 0 name is not acceptable.
@@ -774,23 +859,87 @@ int
 ok_command_name(const char *name)
 {
   const unsigned char *p;
-  if (!isupper((unsigned char) *name))
+  int cnt = 0;
+  /* First char: uppercase alphanum or legal punctuation */
+  switch ((unsigned char) *name) {
+  case SAY_TOKEN:
+  case POSE_TOKEN:
+  case SEMI_POSE_TOKEN:
+  case EMIT_TOKEN:
+  case NOEVAL_TOKEN:
+  case NUMBER_TOKEN:
+  case '&':
     return 0;
-  for (p = (unsigned char *) name; p && *p; p++) {
-    if (!(isupper(*p) || isdigit(*p) || (*p == '_')))
+  default:
+    if (!isupper((unsigned char) *name) && !isdigit((unsigned char) *name)
+	&& !ispunct((unsigned char) *name))
       return 0;
   }
-
+  /* Everything else must be printable and non-space, and we need
+   * to find at least one uppercase alpha
+   */
+  for (p = (unsigned char *) name; p && *p; p++) {
+    if (isspace(*p))
+      return 0;
+    if (isupper(*p))
+      cnt++;
+  }
+  if (!cnt)
+    return 0;
   /* Not too long */
   if (strlen(name) >= COMMAND_NAME_LIMIT)
     return 0;
+  return 1;
+}
 
+/** Is a name ok for a function?
+ * It must start with uppercase alpha or punctuation and may contain only
+ * uppercase alpha, numbers, or punctuation thereafter.
+ * It must contain at least one uppercase alpha.
+ * It may not begin with " : ; & ] \ and # (the special tokens).
+ * \param name name to check.
+ * \retval 1 name is acceptable.
+ * \retval 0 name is not acceptable.
+ */
+int
+ok_function_name(const char *name)
+{
+  const unsigned char *p;
+  int cnt = 0;
+  /* First char: uppercase alpha or legal punctuation */
+  switch ((unsigned char) *name) {
+  case SAY_TOKEN:
+  case POSE_TOKEN:
+  case SEMI_POSE_TOKEN:
+  case EMIT_TOKEN:
+  case NOEVAL_TOKEN:
+  case NUMBER_TOKEN:
+  case '&':
+    return 0;
+  default:
+    if (!isupper((unsigned char) *name) && !ispunct((unsigned char) *name))
+      return 0;
+  }
+  /* Everything else must be printable and non-space, and we need
+   * to find at least one uppercase alpha
+   */
+  for (p = (unsigned char *) name; p && *p; p++) {
+    if (isspace(*p))
+      return 0;
+    if (isupper(*p))
+      cnt++;
+  }
+  if (!cnt)
+    return 0;
+  /* Not too long */
+  if (strlen(name) >= COMMAND_NAME_LIMIT)
+    return 0;
   return 1;
 }
 
 /** Does params contain only acceptable HTML tag attributes?
  * Right now, this means: filter out SEND and XCH_CMD if
- * the player isn't a Wizard. Params may contain a space-separated
+ * the player isn't privileged. Params may contain a space-separated
  * list of tag=value pairs. It's probably possible to fool this
  * checking. Needs more work, or removing HTML support. 
  * \param player player using the attribute, or NOTHING for internal.
@@ -803,7 +952,7 @@ ok_tag_attribute(dbref player, const char *params)
 {
   const unsigned char *p, *q;
 
-  if (!GoodObject(player) || Wizard(player))
+  if (!GoodObject(player) || Can_Pueblo_Send(player))
     return 1;
   p = (const unsigned char *) params;
   while (*p) {
@@ -814,9 +963,10 @@ ok_tag_attribute(dbref player, const char *params)
       q++;
     if (*q) {
       size_t n = q - p;
-      /* Invalid params for non-wizards. Turn to a hashtable if we ever
+      /* Invalid params for non-priv'd. Turn to a hashtable if we ever
          get more? */
-      if (strncasecmp(p, "SEND", n) == 0 || strncasecmp(p, "XCH_CMD", n) == 0)
+      if (strncasecmp((char *) p, "SEND", n) == 0
+	  || strncasecmp((char *) p, "XCH_CMD", n) == 0)
 	return 0;
       while (*q && isspace(*q))
 	q++;
@@ -1103,9 +1253,8 @@ do_verb(dbref player, dbref cause, char *arg1, char **argv)
 struct guh_args {
   char *buff;		/**< Buffer for output */
   char *bp;		/**< Pointer to buff's current position */
-  char *lookfor;	/**< String to grep for */
-  int len;		/**< Length of lookfor */
-  int insensitive;	/**< If 1, case-insensitive match; if 0, sensitive */
+  char *lookfor;	/**< Pattern to grep for */
+  int sensitive;	/**< If 1, case-sensitive match; if 0, insensitive */
 };
 
 static int
@@ -1116,14 +1265,16 @@ grep_util_helper(dbref player __attribute__ ((__unused__)),
 		 __attribute__ ((__unused__)), ATTR *atr, void *args)
 {
   struct guh_args *guh = args;
-  int found;
+  int found = 0;
   char *s;
+  int len;
 
   s = (char *) atr_value(atr);	/* warning: static */
+  len = strlen(guh->lookfor);
   found = 0;
   while (*s && !found) {
-    if ((!guh->insensitive && !strncmp(guh->lookfor, s, guh->len)) ||
-	(guh->insensitive && !strncasecmp(guh->lookfor, s, guh->len)))
+    if ((guh->sensitive && !strncmp(guh->lookfor, s, len)) ||
+	(!guh->sensitive && !strncasecmp(guh->lookfor, s, len)))
       found = 1;
     else
       s++;
@@ -1136,29 +1287,48 @@ grep_util_helper(dbref player __attribute__ ((__unused__)),
   return found;
 }
 
+static int
+wildgrep_util_helper(dbref player __attribute__ ((__unused__)),
+		     dbref thing __attribute__ ((__unused__)),
+		     dbref parent __attribute__ ((__unused__)),
+		     char const *pattern
+		     __attribute__ ((__unused__)), ATTR *atr, void *args)
+{
+  struct guh_args *guh = args;
+  int found = 0;
+
+  if (quick_wild_new(guh->lookfor, atr_value(atr), guh->sensitive)) {
+    if (guh->bp != guh->buff)
+      safe_chr(' ', guh->buff, &guh->bp);
+    safe_str(AL_NAME(atr), guh->buff, &guh->bp);
+    found = 1;
+  }
+  return found;
+}
+
 /** Utility function for grep funtions/commands.
  * This function returns a list of attributes on an object that
  * match a name pattern and contain another string.
  * \param player the enactor.
  * \param thing object to check attributes on.
- * \param pattern wildcard pattern for attributes to check.
+ * \param pattern wildcard or substring pattern for attributes to check.
  * \param lookfor string to find within each attribute.
- * \param len length of lookfor.
- * \param insensitive if 1, case-insensitive matching; if 0, case-sensitive.
+ * \param sensitive if 1, case-sensitive matching; if 0, case-insensitive.
+ * \param wild if 1, wildcard matching, if 0, substring
  * \return string containing list of attribute names with matching data.
  */
 char *
 grep_util(dbref player, dbref thing, char *pattern, char *lookfor,
-	  int len, int insensitive)
+	  int sensitive, int wild)
 {
   struct guh_args guh;
 
   guh.buff = (char *) mush_malloc(BUFFER_LEN + 1, "grep_util.buff");
   guh.bp = guh.buff;
   guh.lookfor = lookfor;
-  guh.len = len;
-  guh.insensitive = insensitive;
-  (void) atr_iter_get(player, thing, pattern, 0, grep_util_helper, &guh);
+  guh.sensitive = sensitive;
+  (void) atr_iter_get(player, thing, pattern, 0, wild ? wildgrep_util_helper
+		      : grep_util_helper, &guh);
   *guh.bp = '\0';
   return guh.buff;
 }
@@ -1207,7 +1377,7 @@ grep_helper(dbref player, dbref thing __attribute__ ((__unused__)),
     notify_format(player, "%s%s [#%d%s]%s %s",
 		  ANSI_HILITE, AL_NAME(atr),
 		  Owner(AL_CREATOR(atr)),
-		  privs_to_letters(attr_privs, AL_FLAGS(atr)),
+		  privs_to_letters(attr_privs_view, AL_FLAGS(atr)),
 		  ANSI_NORMAL, tbuf1);
   return found;
 }
@@ -1258,7 +1428,7 @@ do_grep(dbref player, char *obj, char *lookfor, int flag, int insensitive)
     if (!atr_iter_get(player, thing, pattern, 0, grep_helper, &gh))
       notify(player, T("No matching attributes."));
   } else {
-    tp = grep_util(player, thing, pattern, lookfor, len, insensitive);
+    tp = grep_util(player, thing, pattern, lookfor, !insensitive, 0);
     notify_format(player, T("Matches of '%s' on %s(#%d): %s"), lookfor,
 		  Name(thing), thing, tp);
     mush_free((Malloc_t) tp, "grep_util.buff");
