@@ -60,8 +60,6 @@ void
 do_name(dbref player, const char *name, char *newname)
 {
   dbref thing;
-  ATTR *a = NULL;
-  char alias[BUFFER_LEN];
   char *password;
   char *myenv[10];
   int i;
@@ -116,19 +114,10 @@ do_name(dbref player, const char *name, char *newname)
 	    password++;
 	}
       }
-      a = atr_get_noparent(thing, "ALIAS");
-      if (a)
-	strcpy(alias, atr_value(a));
-      if (strcasecmp(newname, Name(thing))
-	  && !(a && !strcasecmp(newname, alias))	/* Swap alias, name */
-	  &&!ok_player_name(newname, thing)) {
-	/* strcasecmp allows changing foo to Foo, etc. */
+      if (!ok_player_name(newname, player, thing)) {
 	notify(player, T("You can't give a player that name."));
 	return;
       }
-      if (a && strcasecmp(alias, newname)) {
-	a = NULL;
-      }
       /* everything ok, notify */
       do_log(LT_CONN, 0, 0, T("Name change by %s(#%d) to %s"),
 	     Name(thing), thing, newname);
@@ -153,16 +142,10 @@ do_name(dbref player, const char *name, char *newname)
     for (i = 2; i < 10; i++)
       myenv[i] = NULL;
 
-    if (IsPlayer(thing) && !a)
-      delete_player(thing, NULL);
-    else if (a)
-      atr_add(thing, "ALIAS", Name(thing), player, 0);
-
+    if (IsPlayer(thing))
+      reset_player_list(thing, Name(thing), NULL, newname, NULL);
     set_name(thing, newname);
 
-    if (IsPlayer(thing) && !a)
-      add_player(thing, NULL);
-
     if (!AreQuiet(player, thing))
       notify(player, T("Name set."));
     real_did_it(player, thing, NULL, NULL, "ONAME", NULL, "ANAME", NOTHING,
@@ -488,9 +471,10 @@ do_chzone(dbref player, char const *name, char const *newobj, int noisy)
 
 /** Structure for af_helper() data. */
 struct af_args {
-  int f;		/**< flag bits */
-  int clear;		/**< True to remove the flag */
-  char *flag;		/**< flag name */
+  int setf;		/**< flag bits to set */
+  int clrf;		/**< flag bits to clear */
+  char *setflags;	/**< list of names of flags to set */
+  char *clrflags;	/**< list of names of flags to clear */
 };
 
 static int
@@ -506,23 +490,25 @@ af_helper(dbref player, dbref thing,
    * There is one special case - the resetting of the SAFE flag.
    */
   if (!(Can_Write_Attr(player, thing, AL_ATTR(atr)) ||
-	(af->clear && (af->f & AF_SAFE) &&
+	((af->clrf & AF_SAFE) &&
 	 Can_Write_Attr_Ignore_Safe(player, thing, AL_ATTR(atr))))) {
     notify_format(player, T("You cannot change that flag on %s/%s"),
 		  Name(thing), AL_NAME(atr));
     return 0;
   }
 
-  if (af->clear) {
-    AL_FLAGS(atr) &= ~af->f;
+  /* Clear flags first, then set flags */
+  if (af->clrf) {
+    AL_FLAGS(atr) &= ~af->clrf;
     if (!AreQuiet(player, thing))
       notify_format(player, T("%s/%s - %s reset."), Name(thing), AL_NAME(atr),
-		    af->flag);
-  } else {
-    AL_FLAGS(atr) |= af->f;
+		    af->clrflags);
+  }
+  if (af->setf) {
+    AL_FLAGS(atr) |= af->setf;
     if (!AreQuiet(player, thing))
       notify_format(player, T("%s/%s - %s set."), Name(thing), AL_NAME(atr),
-		    af->flag);
+		    af->setflags);
   }
 
   return 1;
@@ -564,21 +550,22 @@ do_attrib_flags(dbref player, const char *obj, const char *atrname,
     return;
   }
 
-  af.clear = 0;
+  p = flag;
+  /* Skip leading spaces */
+  while (*p && isspace((unsigned char) *p))
+    p++;
 
-  /* move past NOT token if there is one */
-  for (p = flag; *p && ((*p == NOT_TOKEN) || isspace((unsigned char) *p)); p++)
-    if (*p == NOT_TOKEN)
-      af.clear = !af.clear;
-
-  if ((af.f = string_to_atrflag(player, p)) < 0) {
+  af.setf = af.clrf = 0;
+  if (string_to_atrflagsets(player, p, &af.setf, &af.clrf) < 0) {
     notify(player, T("Unrecognized attribute flag."));
     return;
   }
-  af.flag = mush_strdup(atrflag_to_string(af.f), "af_flag list");
+  af.clrflags = mush_strdup(atrflag_to_string(af.clrf), "af_flag list");
+  af.setflags = mush_strdup(atrflag_to_string(af.setf), "af_flag list");
   if (!atr_iter_get(player, thing, atrname, 0, af_helper, &af))
     notify(player, T("No attribute found to change."));
-  mush_free((Malloc_t) af.flag, "af_flag list");
+  mush_free((Malloc_t) af.clrflags, "af_flag list");
+  mush_free((Malloc_t) af.setflags, "af_flag list");
 }
 
 
@@ -740,6 +727,7 @@ do_cpattr(dbref player, char *oldpair, char **newpair, int move, int noflagcopy)
 /** Argument struct for gedit_helper */
 struct gedit_args {
   enum edit_type target; /**< The type of edit */
+  int doit;  /**< Do we actually replace the attribute, or just pretend? */
   char *from; /**< What is going to be replaced? */
   char *to; /**< What it gets replaced with. */
 };
@@ -878,8 +866,16 @@ gedit_helper(dbref player, dbref thing,
   *tbufp = '\0';
   *tbufap = '\0';
 
-  if (do_set_atr(thing, AL_NAME(a), tbuf1, player, 0) &&
-      !AreQuiet(player, thing)) {
+  if (gargs->doit) {
+    if (do_set_atr(thing, AL_NAME(a), tbuf1, player, 0) &&
+	!AreQuiet(player, thing)) {
+      if (!ansi_long_flag && ShowAnsi(player))
+	notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf_ansi);
+      else
+	notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf1);
+    }
+  } else {
+    // We don't do it - we just pemit it.
     if (!ansi_long_flag && ShowAnsi(player))
       notify_format(player, "%s - Set: %s", AL_NAME(a), tbuf_ansi);
     else
@@ -898,13 +894,14 @@ gedit_helper(dbref player, dbref thing,
  * \param argv array containing the search and replace strings.
  */
 void
-do_gedit(dbref player, char *it, char **argv, enum edit_type target)
+do_gedit(dbref player, char *it, char **argv, enum edit_type target, int doit)
 {
   dbref thing;
   char tbuf1[BUFFER_LEN];
   char *q;
   struct gedit_args args;
 
+
   if (!(it && *it)) {
     notify(player, T("I need to know what you want to edit."));
     return;
@@ -930,6 +927,8 @@ do_gedit(dbref player, char *it, char **argv, enum edit_type target)
   args.from = argv[1];
   args.to = argv[2];
   args.target = target;
+  args.doit = doit;
+
   if (!atr_iter_get(player, thing, q, 0, gedit_helper, &args))
     notify(player, T("No matching attributes."));
 }
