@@ -35,7 +35,6 @@ static void do_audible_stuff(dbref loc, dbref *excs, int numexcs,
 static void do_one_remit(dbref player, const char *target, const char *msg,
 			 int flags);
 dbref na_zemit(dbref current, void *data);
-extern int i_comp(const void *s1, const void *s2);
 
 const char *
 spname(dbref thing)
@@ -111,7 +110,7 @@ do_teach(dbref player, dbref cause, const char *tbuf1)
   if (!GoodObject(loc))
     return;
 
-  if (!eval_lock(player, loc, Speech_Lock)) {
+  if (!Loud(player) && !eval_lock(player, loc, Speech_Lock)) {
     fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
@@ -151,7 +150,7 @@ do_say(dbref player, const char *tbuf1)
   if (!GoodObject(loc))
     return;
 
-  if (!eval_lock(player, loc, Speech_Lock)) {
+  if (!Loud(player) && !eval_lock(player, loc, Speech_Lock)) {
     fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
@@ -166,6 +165,19 @@ do_say(dbref player, const char *tbuf1)
 		NA_INTER_HEAR);
 }
 
+/** A comparator for raw dbrefs.
+ * This exists because the i_comp function in funlist.c got hairy some time ago.
+ * \param a the first element to compare.
+ * \param b the second element to compare.
+ */
+static int
+dbref_comp(const void *a, const void *b)
+{
+  const dbref *x = (const dbref *) a;
+  const dbref *y = (const dbref *) b;
+  return (int) *x - (int) *y;
+}
+
 /** The oemit(/list) command.
  * \verbatim
  * This implements @oemit and @oemit/list.
@@ -214,7 +226,7 @@ do_oemit_list(dbref player, char *list, const char *message, int flags)
       return;
     }
 
-    if (!eval_lock(player, pass[1], Speech_Lock)) {
+    if (!Loud(player) && !eval_lock(player, pass[1], Speech_Lock)) {
       fail_lock(player, pass[1], Speech_Lock, T("You may not speak there!"),
 		NOTHING);
       return;
@@ -239,7 +251,7 @@ do_oemit_list(dbref player, char *list, const char *message, int flags)
      * locs[0..10] are corresponding dbrefs of locations
      */
     if (GoodObject(who) && GoodObject(Location(who))
-	&& eval_lock(player, Location(who), Speech_Lock)
+	&& (Loud(player) || eval_lock(player, Location(who), Speech_Lock))
       ) {
       if (pass[0] < 10) {
 	locs[pass[0]] = Location(who);
@@ -254,7 +266,7 @@ do_oemit_list(dbref player, char *list, const char *message, int flags)
 
   /* Sort the list of rooms to oemit to so we don't oemit to the same           
    * room twice */
-  qsort((void *) locs, pass[0], sizeof(locs[0]), i_comp);
+  qsort((void *) locs, pass[0], sizeof(locs[0]), dbref_comp);
 
   if (flags & PEMIT_SPOOF)
     na_flags |= NA_SPOOF;
@@ -413,7 +425,7 @@ do_pemit_list(dbref player, char *list, const char *message, int flags)
 {
   char *bp, *p;
   char *nsbuf, *nspbuf;
-  char *l;
+  const char *l;
   dbref who;
   int nospoof;
 
@@ -426,8 +438,8 @@ do_pemit_list(dbref player, char *list, const char *message, int flags)
   list[BUFFER_LEN - 1] = '\0';
   l = trim_space_sep(list, ' ');
 
-  while ((p = split_token(&l, ' '))) {
-    who = noisy_match_result(player, p, NOTYPE, MAT_ABSOLUTE);
+  while (l && *l && (p = next_in_list(&l))) {
+    who = noisy_match_result(player, p, NOTYPE, MAT_EVERYTHING);
     if (GoodObject(who) && okay_pemit(player, who)) {
       if (nospoof && Nospoof(who)) {
 	if (Paranoid(who)) {
@@ -442,17 +454,26 @@ do_pemit_list(dbref player, char *list, const char *message, int flags)
 			  Name(player), player, message);
 	    *bp = '\0';
 	  }
-	  notify(who, nspbuf);
+	  if (flags & PEMIT_PROMPT)
+	    notify_prompt(who, nspbuf);
+	  else
+	    notify(who, nspbuf);
 	} else {
 	  if (!nsbuf) {
 	    bp = nsbuf = mush_malloc(BUFFER_LEN, "string");
 	    safe_format(nsbuf, &bp, "[%s->] %s", Name(player), message);
 	    *bp = '\0';
 	  }
-	  notify(who, nsbuf);
+	  if (flags & PEMIT_PROMPT)
+	    notify_prompt(who, nsbuf);
+	  else
+	    notify(who, nsbuf);
 	}
       } else {
-	notify_must_puppet(who, message);
+	if (flags & PEMIT_PROMPT)
+	  notify_prompt_must_puppet(who, message);
+	else
+	  notify_must_puppet(who, message);
       }
     }
   }
@@ -527,7 +548,7 @@ do_pose(dbref player, const char *tbuf1, int space)
   if (!GoodObject(loc))
     return;
 
-  if (!eval_lock(player, loc, Speech_Lock)) {
+  if (!Loud(player) && !eval_lock(player, loc, Speech_Lock)) {
     fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
@@ -617,10 +638,11 @@ do_wall(dbref player, const char *message, enum wall_type target, int emit)
  * \param noeval if 1, page/noeval.
  * \param multipage if 1, a page/list; if 0, a page/blind.
  * \param override if 1, page/override.
+ * \param has_eq if 1, the command had an = in it.
  */
 void
 do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
-	int noeval, int multipage, int override)
+	int noeval, int multipage, int override, int has_eq)
 {
   dbref target;
   const char *message;
@@ -640,13 +662,14 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
   int fails_lock;
   int is_haven;
   ATTR *a;
+  char *alias;
 
   tp2 = tbuf2 = (char *) mush_malloc(BUFFER_LEN, "string");
   if (!tbuf2)
     mush_panic("Unable to allocate memory in do_page");
 
-  if (arg2 && *arg2 && *arg1) {
-    /* page to=msg. Always evaluate to, maybe evaluate msg */
+  if (*arg1 && has_eq) {
+    /* page to=[msg]. Always evaluate to, maybe evaluate msg */
     process_expression(tbuf2, &tp2, &arg1, player, cause, cause,
 		       PE_DEFAULT, PT_DEFAULT, NULL);
     *tp2 = '\0';
@@ -849,8 +872,8 @@ do_page(dbref player, const char *arg1, const char *arg2, dbref cause,
 		  message);
 
   /* Figure out the 'name' of the player */
-  if (PAGE_ALIASES && (a = atr_get_noparent(player, "ALIAS")))
-    current = tprintf("%s (%s)", Name(player), atr_value(a));
+  if (PAGE_ALIASES && (alias = shortalias(player)) && *alias)
+    current = tprintf("%s (%s)", Name(player), alias);
   else
     current = (char *) Name(player);
 
@@ -1149,7 +1172,7 @@ do_emit(dbref player, const char *tbuf1, int flags)
   if (!GoodObject(loc))
     return;
 
-  if (!eval_lock(player, loc, Speech_Lock)) {
+  if (!Loud(player) && !eval_lock(player, loc, Speech_Lock)) {
     fail_lock(player, loc, Speech_Lock, T("You may not speak here!"), NOTHING);
     return;
   }
@@ -1183,7 +1206,7 @@ do_one_remit(dbref player, const char *target, const char *msg, int flags)
       notify_format(player,
 		    T("I'm sorry, but %s wishes to be left alone now."),
 		    Name(room));
-    } else if (!eval_lock(player, room, Speech_Lock)) {
+    } else if (!Loud(player) && !eval_lock(player, room, Speech_Lock)) {
       fail_lock(player, room, Speech_Lock, T("You may not speak there!"),
 		NOTHING);
     } else {
@@ -1256,7 +1279,7 @@ do_lemit(dbref player, const char *tbuf1, int flags)
   if (rec > 15) {
     notify(player, T("Too many containers."));
     return;
-  } else if (!eval_lock(player, room, Speech_Lock)) {
+  } else if (!Loud(player) && !eval_lock(player, room, Speech_Lock)) {
     fail_lock(player, room, Speech_Lock, T("You may not speak there!"),
 	      NOTHING);
     return;
@@ -1285,7 +1308,7 @@ na_zemit(dbref current __attribute__ ((__unused__)), void *data)
     if (this == NOTHING) {
       for (room = dbrefs[1]; room < db_top; room++) {
 	if (IsRoom(room) && (Zone(room) == dbrefs[2])
-	    && eval_lock(dbrefs[3], room, Speech_Lock)
+	    && (Loud(dbrefs[3]) || eval_lock(dbrefs[3], room, Speech_Lock))
 	  )
 	  break;
       }
